<?

 /*****************************************************************************
 * Filename: include/config.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."errors.php");
 require_once($_path["include"]."data.php");

 $_config=new Config();

 class Config
 {
  function __construct()
  {
   $this->e=new Errors();
  }

  function __clone()
  {
   //clone objects
   $this->e=clone $this->e;
  }


  function set($key, $value=NULL)
  {
   global $_session;

   if(!$key)
    $key="config";
   else
    $key="config.".$key;

   return $_session->set($key, $value);
  }

  function get($key)
  {
   global $_session;

   if(!$key)
    $key="config";
   else
    $key="config.".$key;

   return $_session->get($key);
  }


  function userAnonymous()
  {
   if($this->get("usernr")==$this->get("anonnr"))
    return true;

   return false;
  }
  function userRoot()
  {
   if($this->get("usernr"))
    if($this->get("usernr")==$this->get("rootnr"))
     return true;

   return false;
  }
  function userBackend()
  {
   if($this->get("userbackend")||$this->userRoot())
    return true;

   return false;
  }

  function su($user=false)
  {
   if($user===true) //become root
   {
    if($this->get("suusernr")!==NULL) //already another user
     return false;

    $this->set("suusernr", $this->get("usernr")); //store current user
    $this->set("usernr", $this->get("rootnr"));
   }
   else if($user===false) //revert to original user
   {
    if($this->get("suusernr")===NULL) //not another user
     return false;

    $this->set("usernr", $this->get("suusernr")); //restore current user
    $this->set("suusernr"); //unset suusernr
   }
   else //become user
   {
    if(!$this->userRoot()) //must be root
     return false;

    if($this->get("suusernr")!==NULL) //already another user
     return false;

    $d=Data::cache($this->get("usermodule"), "view", "all");
    $d->setShowFields("nr");
    $d->clearValues();

    if($d->e->any()) //error initializing module
     return false;

    if(preg_match("/^\d+$/", $user)) //id
     $d->setValue($d->getID(), $user);
    else //username
    {
     if(!($userField=$d->check("properties", "username"))) //no username field
      return false;

     if(sizeof($userField)!=1) //multiple username fields, not supported
      return false;

     $userField=$userField[0];
     $d->setValue($userField, $user);
    }

    if(!$d->get()||$d->vmax!=1||!$d->next()) //no/multiple users found
     return false;

    $this->set("suusernr", $this->get("usernr")); //store current user
    $this->set("usernr", $d->v("id"));
   }

   return true;
  }


  function reload()
  {
   $this->set("reload", true);
   return $this->load();
  }

  function load()
  {
   global $_session, $_path;

   //load config if not yet done, on reload or if previous load failed
   if($this->get(false)&&!$this->get("reload")&&!$this->get("failed"))
   {
    //update script timezone
    if($this->get("timezone")&&$this->get("timezone")!=date_default_timezone_get())
     date_default_timezone_set($this->get("timezone"));

    return true;
   }

   //try to get (cached) config
   if(!($config=$this->getCached()))
   {
    if(!($config=$this->getConfig()))
    {
     //setup failed config
     $d=Data::cache("config", "view", "all");
     foreach($d->check("use", "show") as $k)
      if($k!="r")
       $config[$k]=$d->field[$k]["defaultvalue"];

     $config["failed"]=true; //keep reloading config till successful
     $config["language"]=strtolower($config["language"]);
     $config["title"]="Project configuration failure";
    }
   }


   //keep user logged in
   $keep=Array();
   if($this->get("usernr"))
   {
    $config["usernr"]=$this->get("usernr");
    $config["groupnrs"]=$this->get("groupnrs");

    if($this->get("userbackend"))
     $config["userbackend"]=$this->get("userbackend");
   }

   //store config
   $this->set(false, $config);

   //overwrite user specific configuration
   if(!$this->userAnonymous())
    $this->loadUserConfig();

   //XXX: useModRewrite has been replaced by prettyurls, remove in JAS 1.2
   $this->set("useModRewrite", $this->get("prettyurls"));


   //update script timezone
   if($this->get("timezone")&&$this->get("timezone")!=date_default_timezone_get())
    date_default_timezone_set($this->get("timezone"));

   return true;
  }

  function loadUserConfig()
  {
   $d=Data::cache("configUsers", "view", "all", Array("skipCheckTables" => true));

   if(!$d->tableExists())
    return false;

   $d->clearValues();
   $d->setValue("nr", $this->get("usernr"));

   $fields=$d->check("use", "show");
   $fields=array_remove($fields, "r");

   $this->su(true);
   $d->get();
   $this->su();

   if(!$d->next()) //fail, get config
    return false;

   foreach($fields as $field)
   {
    $this->set($field, $d->v[$field]);
    if($this->get("form".$field)!==NULL)
     $this->set("form".$field, $d->v[$field]);
   }

   return true;
  }


  function getCached()
  {
   //cached config (fake module to speed up initial config load)
   $d=Data::cache("empty", "view", "all", Array("format" => "text"));
   $d->module="config";

   $d->addField("cache", "text", "sql;show", "", 0, 65536);
   $d->addField("userrights", "rights", "", "allusers", 0, false, 256); //view rights for all

   if(!$d->get()||!($cache=$d->getValue("cache"))) //problem in loading config from cache
    return false;

   return unserialize($cache);
  }

  function updateCache()
  {
   if(!($config=$this->getConfig()))
    return false;

   //config nr
   $d=Data::cache("config", "view", "all");
   $d->setShowFields();

   if(!$d->get()||!($nr=$d->getValue("nr"))) //problem in loading config
    return false;

   $d->setAction("edit");
   $d->setExtra("item");

   $d->clearValues();
   $d->setValue("nr", $nr);
   $d->setValue("cache", serialize($config));

   $this->su(true);
   $success=$d->submit();
   $this->su();

   return $success;
  }

  function getConfig()
  {
   $config=Array();
   $customConfigModules=Array();
   $customConfigModules[]="configUsers"; //don't load configUsers automagically

   //general config
   $d=Data::cache("config", "view", "all");
   $customConfigModules[]=$d->module;

   if(!$d->get()||!$d->next()) //problem in loading config
    return false;

   foreach($d->check("use", "show") as $k)
    if($k!="r")
     $config[$k]=$d->v[$k];

   $config["language"]=strtolower($config["language"]);


   //load group rights
   $users=Data::cache($config["usermodule"], "view", "item");

   if($this->get("usernr")&&$this->get("rootnr")) //user config already setup
    $config["groupnrs"]=$users->moduleObject->getGroups($config["usernr"]);
   else //user not yet setup
   {
    $this->set("usernr", 1); $this->set("rootnr", 1); //fake su
    $config["groupnrs"]=$users->moduleObject->getGroups($config["usernr"]);
    $this->set("usernr"); $this->set("rootnr"); //clear user config
   }


   //Modules configs, multiple rows
   $config["modules"]=Array();
   $d=Data::cache("configModules", "view", "all", Array("skipCheckTables" => true));
   $customConfigModules[]=$d->module;

   if($d->tableExists()&&$d->get())
   {
    while($d->next())
    {
     $config["modules"][$d->v["module"]]=Array();
     foreach($d->check("use", "show") as $k)
     {
      if($k!="r")
      {
       if($d->v[$k]===""||(($k=="decimals"||$k=="formdecimals")&&$d->v[$k]==-1)) //empty, use config
        $config["modules"][$d->v["module"]][$k]=$config[$k];
       else
        $config["modules"][$d->v["module"]][$k]=$d->v[$k];
      }
     }
    }
   }


   //Module fields configs, multiple rows and are placed under applicable module
   $d=Data::cache("configFields", "view", "all", Array("skipCheckTables" => true));
   $customConfigModules[]=$d->module;

   if($d->tableExists()&&$d->get())
   {
    while($d->next())
    {
     if(!isset($config["modules"][$d->v["module"]]))
      $config["modules"][$d->v["module"]]=Array("field" => Array());

     foreach($d->check("use", "show") as $k)
     {
      if($k!="r")
      {
       if($d->v[$k]===""||(($k=="decimals"||$k=="formdecimals")&&$d->v[$k]==-1)) //empty, use config/moduleConfig
       {
        if(isset($config["modules"][$d->v["module"]][$k]))
         $config["modules"][$d->v["module"]]["field"][$d->v["field"]][$k]=&$config["modules"][$d->v["module"]][$k];
        else
         $config["modules"][$d->v["module"]]["field"][$d->v["field"]][$k]=&$config[$k];
       }
       else
        $config["modules"][$d->v["module"]]["field"][$d->v["field"]][$k]=$d->v[$k];
      }
     }
    }
   }


   //Repositories config, multiple rows
   $config["repositories"]=Array();
   $d=Data::cache("configRepositories", "view", "all", Array("skipCheckTables" => true));
   $customConfigModules[]=$d->module;

   if($d->tableExists()&&$d->get())
   {
    while($d->next())
    {
     $config["repositories"][$d->v["code"]]=Array();
     foreach($d->check("use", "show") as $k)
     {
      if($k!="r"&&$k!="code")
       $config["repositories"][$d->v["code"]][$k]=$d->v[$k];
     }
    }
   }

   if(!$config["repositories"]) //no repositories configured, use default
   {
    $code=$d->getDefaultValue("code");
    $config["repositories"][$code]=Array();
    foreach($d->check("use", "show") as $k)
    {
     if($k!="r"&&$k!="code")
      $config["repositories"][$code][$k]=$d->getDefaultValue($k);
    }
   }


   //JAS backend sites config, multiple rows
   if(in_array("configSites", JAS::modules()))
   {
    $config["sites"]=Array();
    $d=Data::cache("configSites", "view", "all", Array("skipCheckTables" => true));
    $customConfigModules[]=$d->module;

    if($d->tableExists()&&$d->get())
     while($d->next())
      $config["sites"][$d->v["domain"]]=$d->v["project"];
   }


   //Other "generic" configs
   foreach(JAS::modules() as $module)
   {
    if(strpos($module, "config")===0&&!in_array($module, $customConfigModules)) //should start with config prefix and shouldn't be already gathered above
    {
     $d=Data::cache($module, "view", "all", Array("skipCheckTables" => true));
     $key=strtolower(str_replace("config", "", $d->module));

     if(isset($config[$key]))
     {
      $this->e->add("Config key: \"".$key."\" for module: \"".$d->module."\" already defined by another config module", "module", "Please rename: \"".$d->module."\"");
      continue;
     }

     if($d->tableExists()&&$d->get()&&$d->next())
     {
      $config[$key]=Array();
      foreach($d->check("use", "show") as $k)
       if($k!="r")
        $config[$key][$k]=$d->v[$k];
     }
    }
   }

   return $config;
  }


  function merge($key, $config=false)
  {
   //check if config key exists
   $module="config".$key;
   if(!in_array($module, JAS::modules()))
   {
    $this->e->add($key." config merge failed: unknown config type", "system");
    return false;
   }

   if(!$config||!is_array($config)) //no /illegal config parameter
    $config=Array(); //use empty config

   $var=strtolower($key);

   //fill in the blanks in the config array
   if(!($configured=$this->get($var))||!is_array($configured)) //config key not set: use default values
   {
    $d=Data::cache($module, "view", "item", Array("skipCheckTables" => true, "skipSetRequest" => true, "skipPostInit" => true)); //don't fully init object, only default values needed

    if($d->e->any())
     $this->e->add($d->e);

    $configured=Array();
    foreach($d->check("use", "show") as $field)
     $configured[$field]=$d->getDefaultValue($field);
   }

   return array_merge($configured, $config); //merge config (configured overwritten with provided values)
  }

 }
?>
