<?

 /*****************************************************************************
 * Filename: include/modules/attempts.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."module.php");
 require_once($_path["include"]."socket.php");

 class System_Module_attempts extends Module
 {
  function __construct(&$data)
  {
   parent::__construct($data);

   $this->data->title="Attempts";

   $this->data->addField("key", "text", "all", "identifier;title", 1, 255, false, "Key");
   $this->data->addField("attempts", "nr", "all", "title", 0, false, false, "Attempts");

   $this->data->addField("last", "datetime", "all", "", 0, false, "now", "Last attempt");

   $this->data->sortOrder("key");
  }

  static function active()
  {
   $d=Data::cache("attempts", "view", "all", Array("skipCheckTables" => true));
   return $d->tableExists();
  }

  static function reset($key)
  {
   global $_config;

   if(!$key)
    return false;

   $d=Data::cache("attempts", "view", "all");

   if(strlen($key)>$d->field["key"]["max"])
    $key=substr($key, 0, $d->field["key"]["max"]);

   $d->clearValues();

   $d->setValue("key", $key);

   $d->setShowFields("attempts");

   $_config->su(true);
   if(!$d->get())
   {
    $_config->su();
    return false;
   }
   $_config->su();

   if($d->next()) //entry exists
   {
    $d->setAction("delete");
    $d->setExtra("item");

    $d->clearValues();

    $d->setValue("key", $key);

    $_config->su(true);
    if(!$d->submit())
    {
     $_config->su();
     return false;
    }
    $_config->su();
   }

   return true;
  }

  static function check($key, $max, $timeout=false)
  {
   global $_config;

   if(!$key||!$max)
    return false;

   $d=Data::cache("attempts", "view", "all");

   if(strlen($key)>$d->field["key"]["max"])
    $key=substr($key, 0, $d->field["key"]["max"]);

   $d->clearValues();

   $d->setValue("key", $key);

   $d->setShowFields("attempts", "last");

   $_config->su(true);
   if(!$d->get())
   {
    $_config->su();
    return false;
   }
   $_config->su();

   if(!$d->next()) //no entry yet
   {
    $d->setAction("add");
    $d->setExtra("item");

    $d->clearValues();

    $d->setValue("key", $key);
    $d->setValue("attempts", 1); //first attempt

    $d->setDefaultValue("last");
   }
   else //entry exists
   {
    $attempts=$d->v["attempts"]+1;
    $last=DT::dateConvertS($d->v["last"], $d->field["last"]["format"][$d->dataformat]["datetimeformat"], "U");

    if($attempts>$max) //maximum number of attempts reached
     if(!$timeout||$timeout>(mktime()-$last)) //timeout not yet exceeded
      return false;

    $d->setAction("edit");
    $d->setExtra("item");

    $d->clearValues();

    $d->setValue("key", $key);
    $d->setValue("attempts", $attempts);
    $d->setValue("last", "now");
   }

   $_config->su(true);
   if(!$d->submit())
   {
    $_config->su();
    return false;
   }
   $_config->su();

   return true;
  }

 }
?>
