<?

 /*****************************************************************************
 * Filename: include/modules/config.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."module.php");

 class System_Module_config extends Module
 {
  function __construct(&$data)
  {
   parent::__construct($data);

   $this->data->title="Configuration";

   $this->data->defaultFields("id;added;updated");
   $this->data->setShowFields(); //clear show fields

   $this->data->addField("cache", "text", "sql", "", 0, 65536); //can contain cached config
   $this->data->addField("userrights", "rights", "", "allusers", 0, false, 256); //view rights for all
   $this->data->addField("title", "text", "all", "title", 1, 255, false, "Project title", "Title of the project");

   $this->data->addField("usermodule", "module", "all", "reload", 0, 25, "users", "User module", "Which module to use for user logins");

   $this->data->addField("rootnr", "nr", "all", false, 1, false, 1, "Root number", "The user account which is considered root");

   $this->data->addField("anonnr", "nr", "all", false, 1, false, 2, "Anonymous user number", "The user number of the anonumous user, needed to check if someone is logged in");
   $this->data->addField("usernr", "nr", "all", false, 1, false, 2, "Default user number", "Should be the same as anonymous users number");
   $this->data->addField("groupnrs", "array", "show", false, 0, false, Array(), "Anonymous group numbers", "Automatically gather at initialization");

   $this->data->addField("language", "enum", "all", "reload", 1, 3, "EN", "Language code", "Default project language");
   $this->data->enumOptions("language", Array("NL" => "Dutch", "EN" => "English"));

   if(function_exists("timezone_identifiers_list")) //no timezone support for PHP < 5.1.0
   {
    $this->data->addField("sqltimezone", "list", "all", false, 1, 25, "UTC", "Database timezone", "It's wise to set this to UTC if you plan on using multiple timezones, don't change this value during usage");
    $this->data->listOptions("sqltimezone", timezone_identifiers_list());
    $this->data->addField("apitimezone", "list", "all", false, 1, 25, "UTC", "API timezone", "It's wise to set this to UTC");
    $this->data->listOptions("apitimezone", timezone_identifiers_list());

    $this->data->addField("timezone", "list", "all", false, 1, 25, date("e"), "Display timezone");
    $this->data->listOptions("timezone", timezone_identifiers_list());
    $this->data->addField("formtimezone", "list", "all", false, 1, 25, date("e"), "Input timezone");
    $this->data->listOptions("formtimezone", timezone_identifiers_list());
   }

   $this->data->addField("defaultcategory", "enum", "all", false, false, false, "Default", "Default category", "Which category to use by default when adding new items");
   $this->data->enumOptions("defaultcategory", "Default", "Last used", "First", "Last");

   $this->data->addField("dateformat", "text", "all", false, 1, 10, "d-m-Y", "Display date format");
   $this->data->addField("formdateformat", "text", "all", false, 1, 10, "d-m-Y", "Input date format");
   $this->data->addField("apidateformat", "text", "all", false, 1, 10, "Y-m-d", "API date format");
   $this->data->addField("timeformat", "text", "all", false, 1, 10, "H:i:s", "Display time format");
   $this->data->addField("formtimeformat", "text", "all", false, 1, 10, "H:i:s", "Input time format");
   $this->data->addField("apitimeformat", "text", "all", false, 1, 10, "H:i:s", "API time format");
   $this->data->addField("datetimeformat", "text", "all", false, 1, 25, "H:i:s d-m-Y", "Display date and time format");
   $this->data->addField("formdatetimeformat", "text", "all", false, 1, 25, "H:i:s d-m-Y", "Input date and time format");
   $this->data->addField("apidatetimeformat", "text", "all", false, 1, 25, "Y-m-d H:i:s", "API date and time format");

   $this->data->addField("decimals", "nr", "all", false, 0, 100, 2, "Display number of decimals");
   $this->data->addField("formdecimals", "nr", "all", false, 0, 100, 2, "Input number of decimals");
   $this->data->addField("apidecimals", "nr", "all", false, 0, 100, 10, "API number of decimals");
   $this->data->addField("decimalseparator", "text", "all", false, 1, 1, ".", "Display decimal separation character");
   $this->data->addField("formdecimalseparator", "text", "all", false, 1, 1, ".", "Input decimal separation character");
   $this->data->addField("apidecimalseparator", "text", "all", false, 1, 1, ".", "API decimal separation character");
   $this->data->addField("thousandsseparator", "text", "all", false, 0, 1, "", "Display thousands separation character");
   $this->data->addField("formthousandsseparator", "text", "all", false, 0, 1, "", "Input thousands separation character");
   $this->data->addField("apithousandsseparator", "text", "all", false, 0, 1, "", "API thousands separation character");

   $this->data->addField("logging", "boolean", "all", false, false, false, false, "Logging", "If hack attempts should be logged to the database");
   $this->data->addField("debug", "boolean", "all", false, false, false, false, "Debug information", "If debug information should be shown");
   $this->data->addField("benchmark", "boolean", "all", false, false, false, false, "Benchmark information", "If benchmark information should be gathered");
   $this->data->addField("translationinfo", "boolean", "all", false, false, false, false, "Translation info", "If translation information should be shown");
   $this->data->addField("scrambleemail", "boolean", "all", false, false, false, false, "Scramble email", "If email addresses should be scrambled");

   //XXX: move directly after defaultcategory in JAS 1.2 (only here so upgrade process doesn't rename translationerrors to prettyurls)
   $this->data->addField("prettyurls", "boolean", "all", false, false, false, false, "Pretty URLs", "Note that this requires all invalid URLs to be handed to JAS (e.g. Apache mod_rewrite will do this)");

   $this->data->addField("stp", "boolean", "all", false, false, false, false, "STP", "Use STP check to prevent CSRF");


   //crypt fields
   $cryptmethods=Array();

   if(defined("CRYPT_BLOWFISH")&&CRYPT_BLOWFISH)
    $cryptmethods[]="Blowfish";
   if(defined("CRYPT_MD5")&&CRYPT_MD5)
   {
    $cryptmethods[]="MD5";
    $cryptmethods[]="MD5 Unsalted";
   }
   if(defined("CRYPT_SHA256")&&CRYPT_SHA256)
    $cryptmethods[]="SHA256";
   if(defined("CRYPT_SHA512")&&CRYPT_SHA512)
    $cryptmethods[]="SHA512";

   if(!$cryptmethods)
   {
    $this->data->e->add("No crypt methods available", "warning");
    $cryptmethods[]="";
   }

   $this->data->addField("passwordcryptmethod", "enum", "all", "reload", false, false, $cryptmethods[0], "Crypt method", "Which encryption method to use for crypt fields");
   $this->data->enumOptions("passwordcryptmethod", $cryptmethods);
   unset($cryptmethods);

   $this->data->addField("passwordcryptcost", "nr", "all", false, 4, 31, 7, "Blowfish cost", "Number of iterations for the blowfish algorithm");
   $this->data->addField("passwordcryptrounds", "nr", "all", false, 1000, 999999999, 5000, "SHA rounds", "Number of iterations for the SHA algorithms");

   $this->data->addField("passwordlower", "nr", "all", false, 0, false, 2, "Amount of lower case characters");
   $this->data->addField("passwordupper", "nr", "all", false, 0, false, 2, "Amount of upper case characters");
   $this->data->addField("passwordnumbers", "nr", "all", false, 0, false, 2, "Amount of numbers");
   $this->data->addField("passwordspecials", "nr", "all", false, 0, false, 1, "Amount of special characters");


   $this->data->group("User configuration", "usermodule", "rootnr", "anonnr", "usernr", "groupnrs");
   $this->data->group("Display formatting", "timezone", "dateformat", "timeformat", "datetimeformat", "decimals", "decimalseparator", "thousandsseparator");
   $this->data->group("Input formatting", "formtimezone", "formdateformat", "formtimeformat", "formdatetimeformat", "formdecimals", "formdecimalseparator", "formthousandsseparator");
   $this->data->group("API formatting", "apitimezone", "apidateformat", "apitimeformat", "apidatetimeformat", "apidecimals", "apidecimalseparator", "apithousandsseparator");
   $this->data->group("Information", "logging", "debug", "benchmark", "translationinfo", "scrambleemail");
   $this->data->group("Password", "passwordcryptmethod", "passwordcryptcost", "passwordcryptrounds", "passwordlower", "passwordupper", "passwordnumbers", "passwordspecials");
  }

  function postInit()
  {
   if($this->data->action=="add"||$this->data->action=="edit")
   {
    if($module=$this->data->getValue("usermodule"))
    {
     $this->data->addReference("rootnr", $module, "nr", "*:1");
     $this->data->addReference("anonnr", $module, "nr", "*:1");
     $this->data->addReference("usernr", $module, "nr", "*:1");
    }

    foreach(Array("timezone", "dateformat", "timeformat", "datetimeformat", "decimals", "decimalseparator") as $field)
     if($this->data->getValue("form".$field)==="")
      $this->data->setValue("form".$field, $this->data->getValue($field));
   }

   if($this->data->getValue("language")=="EN") //no translations
    $this->data->field["translationinfo"]["properties"].=";hidden";
   else
    $this->data->resetField("translationinfo");

   switch($this->data->getValue("passwordcryptmethod"))
   {
    case "Blowfish":
    {
     $this->data->resetField("passwordcryptcost");
     $this->data->field["passwordcryptrounds"]["properties"].=";hidden";

     break;
    }
    case "SHA256":
    case "SHA512":
    {
     $this->data->field["passwordcryptcost"]["properties"].=";hidden";
     $this->data->resetField("passwordcryptrounds");

     break;
    }
    default:
    {
     $this->data->field["passwordcryptcost"]["properties"].=";hidden";
     $this->data->field["passwordcryptrounds"]["properties"].=";hidden";
     break;
    }
   }

   return true;
  }

  function editPostGet()
  {
   if(!$this->data->getValue("passwordcryptmethod")) //set defaults in case no proper values were retrieved for editting
   {
    $this->data->setDefaultValue("passwordcryptmethod");

    if(!$this->data->getValue("passwordcryptcost"))
     $this->data->setDefaultValue("passwordcryptcost");

    if(!$this->data->getValue("passwordcryptrounds"))
     $this->data->setDefaultValue("passwordcryptrounds");
   }

   return true;
  }


  function addPostSubmit(){return $this->updateConfig();}
  function editPostSubmit(){return $this->updateConfig();}
  function deletePostSubmit(){return $this->updateConfig();}
  static function updateConfig()
  {
   global $_config, $_CACHE_UPDATE_CONFIG;

   if($_CACHE_UPDATE_CONFIG) //ensure Config::updateCache doesn't keep caling itself
    return true;

   $_CACHE_UPDATE_CONFIG=true;

   $_config->updateCache(); //failure is not fatal
   $_config->reload();

   $_CACHE_UPDATE_CONFIG=false;
   return true;
  }
 }

?>
