<?

 /*****************************************************************************
 * Filename: include/modules/workflow.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."module.php");

 class System_Module_workflow extends Module
 {
  function __construct(&$data)
  {
   parent::__construct($data);

   $this->data->title="Workflow";

   $this->data->defaultFields();

   $this->data->addField("workflow", "nr", "all", "reload", false, false, 0, "Existing workflow");
   $this->data->addReference("workflow", "workflow", "nr", "*:0");

   $this->data->addField("name", "text", "all", "title", 1, 100, false, "Name");


   $this->data->addReference("previous", "workflowTransitions", "next", "1:*", "nr");

   if(isset($this->data->field["previous"]))
    $this->data->field["previous"]["use"]="show";

   $this->data->addReference("next", "workflowTransitions", "previous", "1:*", "nr");

   if(isset($this->data->field["next"]))
    $this->data->field["next"]["use"]="show";


   $this->data->addField("text", "text", "all", "title", 1, 100, false, "Text");
   $this->data->addField("code", "text", "all", "", 1, 25, false, "Code");

   $this->data->addField("script", "text", "", "", 0, 10000, false, "Script");


   $this->data->group("State", "text", "code");

   $this->data->addUnique("text", "workflow", "name", "text");
   $this->data->addUnique("code", "workflow", "name", "code");

   $this->data->sortOrder("name", "nr");
  }


  function postInit(){return $this->customizeFields();}
  function copyPostGet(){return $this->customizeFields();}
  function editPostGet(){return $this->customizeFields();}

  function customizeFields()
  {
   if($this->data->action!="add"&&$this->data->action!="copy"&&$this->data->action!="edit")
    return true;

   //only allow workflow (top level) entries
   $this->data->listOptions("workflow", $this->getWorkflows());

   if(!$this->data->getValue("workflow")) //new workflow
   {
    //show (start state) name
    $this->data->resetField("name");

    if(!$this->data->getValue("nr"))
     return true;
    if(!($info=$this->getWorkflowInfo($this->data->getValue("nr"))))
     return false;
   }
   else //use existing workflow
   {
    //hide the field
    $this->data->field["name"]["properties"].=";hidden";

    if(!($info=$this->getWorkflowInfo($this->data->getValue("workflow"))))
     return false;

    //force workflow name
    $this->data->setValue("name", $info["workflow"]["name"]);
   }

   $previous=$next=Array();
   $current=$this->data->getValue("nr");
   foreach($info["states"] as $state)
   {
    if($state["nr"]==$current)
     continue;

    $this->data->addField("previous_".$state["nr"], "text", "form", "", 0, 100, (isset($info["transitions"][$state["nr"]."_".$current])?$info["transitions"][$state["nr"]."_".$current]["text"]:""), $state["text"], "Transition from \"".$state["text"]."\" to \"".$this->data->getValue("text")."\"");
    $this->data->addField("next_".$state["nr"], "text", "form", "", 0, 100, (isset($info["transitions"][$current."_".$state["nr"]])?$info["transitions"][$current."_".$state["nr"]]["text"]:""), $state["text"], "Transition from \"".$this->data->getValue("text")."\" to \"".$state["text"]."\"");

    $this->data->setRequestValue("previous_".$state["nr"]);
    $this->data->setRequestValue("next_".$state["nr"]);

    $previous[]="previous_".$state["nr"];
    $next[]="next_".$state["nr"];
   }

   $this->data->group("Transition from previous state", $previous);
   $this->data->group("Transition to next state", $next);

   return true;
  }


  function addPreSubmitErrorCheck(){return $this->checkFields();}
  function copyPreSubmitErrorCheck(){return $this->checkFields();}
  function editPreSubmitErrorCheck(){return $this->checkFields();}

  function checkFields()
  {
   if($this->data->getValue("script")&&($workflow=$this->parseScript($this->data->getValue("script")))) //workflow script provided
   {
    //overwrite text+code based on first workflow script state
    $this->data->setValue("text", $workflow[0]["text"]);
    $this->data->setValue("code", $workflow[0]["code"]);
   }

   if(!$this->data->getValue("code")) //no code
    $this->data->setValue("code", strtoupper($this->data->getValue("text")));

   return true;
  }


  function addPostSubmit(){return $this->handleSubmit();}
  function copyPostSubmit(){return $this->handleSubmit();}
  function editPostSubmit(){return $this->handleSubmit();}

  function deletePreSubmit(){return $this->handleSubmit();}

  function handleSubmit()
  {
   global $_workflowInfo;

   //get catnr for current category
   if(!($catnr=$this->data->getValue("catnr")))
   {
    if($this->data->getValue("nr")) //edit
    {
     $d=Data::cache($this->data->module, "view", "item");
     $d->setValue("nr", $this->data->getValue("nr"));
     $d->setShowFields("catnr");

     if($d->get()&&$d->next())
      $catnr=$d->v["catnr"];
    }
   }

   if($this->data->getValue("script")&&($workflow=$this->parseScript($this->data->getValue("script")))) //workflow script provided
   {
    //add workflow states+transitions
    if($this->data->action!="delete")
    {
     if(!($info=$this->getWorkflowInfo($this->data->getValue("nr"))))
      return false;

     //submit remaining workflow states
     $l=sizeof($workflow);
     $d=Data::cache($this->data->module, "add", "item");
     for($c=1;$c<$l;$c++) //skip first state (already submitted by $this)
     {
      if($workflow[$c]["type"]!="state")
       continue;

      $found=false;
      foreach($info["states"] as $state)
      {
       if($workflow[$c]["code"]==$state["code"])
       {
        $found=true;
        break;
       }
      }

      if(!$found) //not yet added
      {
       $d->clearValues();
       $d->setValue("catnr", $catnr);
       $d->setValue("workflow", $info["workflow"]["nr"]);
       $d->setValue("name", $info["workflow"]["name"]);
       $d->setValue("code", $workflow[$c]["code"]);
       $d->setValue("text", $workflow[$c]["text"]);

       if(!$d->submit())
       {
        $this->data->e->add($d->e);
        return false;
       }
      }
     }

     unset($_workflowInfo[$info["workflow"]["nr"]]); //clear workflow cache
     if(!($info=$this->getWorkflowInfo($this->data->getValue("nr"))))
      return false;

     //submit workflow transitions
     $d=Data::cache("workflowTransitions", "add", "item");
     for($c=1;$c<$l;$c++) //skip first state (already submitted by $this)
     {
      if($workflow[$c]["type"]!="transition")
       continue;

      $previous=$next=false;
      foreach($info["states"] as $state)
      {
       if($workflow[$c]["previous"]==$state["code"])
       {
        $previous=$state["nr"];

        if($next)
         break;
       }

       if($workflow[$c]["next"]==$state["code"])
       {
        $next=$state["nr"];

        if($previous)
         break;
       }
      }

      if(!$previous)
      {
       $this->data->e->add("Workflow script previous state: \"".$workflow[$c]["previous"]."\" not found for transition: \"".$workflow[$c]["text"]."\" ", "error");
       return false;
      }
      else if(!$next)
      {
       $this->data->e->add("Workflow script next state: \"".$workflow[$c]["next"]."\" not found for transition: \"".$workflow[$c]["text"]."\" ", "error");
       return false;
      }

      $found=false;
      foreach($info["transitions"] as $transition)
      {
       if($previous==$transition["previous"]&&$next==$transition["next"])
       {
        $found=true;
        break;
       }
      }

      if(!$found) //not yet added
      {
       $d->clearValues();
       $d->setValue("catnr", $catnr);
       $d->setValue("workflow", $info["workflow"]["nr"]);
       $d->setValue("previous", $previous);
       $d->setValue("next", $next);
       $d->setValue("code", $workflow[$c]["code"]);
       $d->setValue("text", $workflow[$c]["text"]);

       if(!$d->submit())
       {
        $this->data->e->add($d->e);
        return false;
       }
      }
     }
    }
   }
   else //no script provided
   {
    //submit transitions
    if(!$this->data->getValue("workflow")) //new workflow
    {
     if(!($info=$this->getWorkflowInfo($this->data->getValue("nr"))))
      return false;
    }
    else //use existing workflow
    {
     if(!($info=$this->getWorkflowInfo($this->data->getValue("workflow"))))
      return false;
    }

    //submit changes transitions
    $current=$this->data->getValue("nr");
    foreach($info["states"] as $state)
    {
     if($state["nr"]==$current)
      continue;

     if($this->data->action=="delete") //workflow deleted, delete transtions
     {
      if(!$this->_submitTransition($state["nr"], $current, "", $catnr))
       return false;

      if(!$this->_submitTransition($current, $state["nr"], "", $catnr))
       return false;
     }
     else //add/edit/copy, add/edit/delete transition
     {
      if(!$this->_submitTransition($state["nr"], $current, $this->data->getValue("previous_".$state["nr"]), $catnr))
       return false;

      if(!$this->_submitTransition($current, $state["nr"], $this->data->getValue("next_".$state["nr"]), $catnr))
       return false;
     }
    }
   }

   return true;
  }

  function _submitTransition($previous, $next, $text, $catnr=false)
  {
   $d=Data::cache("workflowTransitions", "view", "all");
   $d->setShowFields("text");

   $d->clearValues();
   $d->setValue("previous", $previous);
   $d->setValue("next", $next);

   if(!$d->get())
   {
    $this->data->e->add($d->e);
    return false;
   }

   if($d->next()) //transition exists
   {
    $nr=$d->getValue("nr");

    if($text) //edit transition
    {
     if($text!=$d->v["text"]) //if text changed
     {
      $d->setAction("edit");
      $d->setExtra("item");

      $d->clearValues();
      $d->setValue("nr", $nr);
      $d->setValue("text", $text);

      if(!$d->submit())
      {
       $this->data->e->add($d->e);
       return false;
      }
     }
    }
    else //delete transition
    {
     $d->setAction("delete");
     $d->setExtra("item");

     $d->clearValues();
     $d->setValue("nr", $nr);

     if(!$d->submit())
     {
      $this->data->e->add($d->e);
      return false;
     }
    }
   }
   else //transition doesn't exist
   {
    if($text) //new transition
    {
     $d->setAction("add");
     $d->setExtra("item");

     $d->clearValues();

     if($catnr)
      $d->setValue("catnr", $catnr);

     $d->setValue("previous", $previous);
     $d->setValue("next", $next);
     $d->setValue("text", $text);

     if(!$d->submit())
     {
      $this->data->e->add($d->e);
      return false;
     }
    }
   }

   return true;
  }


  /*
   * Determine/validate available states for workflow reference
   *
   * Module should define:
   * $this->data->addField("workflow", "nr", "all", "", 1, false, $workflow, "Workflow");
   * $this->data->addReference("workflow", "workflow", "nr", "*:1");
   *
   * where $workflow is the name or the database nr of the workflow it should use, an array can be supplied for multiple values.
   *
   * $d is the Data object that initialized the module.
   * disable is a list of workflow state/transition codes that will be "disabled" if they're are valid options
   * Add action will always use the first state (workflow definition)
   * Edit allows current state and subsequent transition(s). If a transition is selected it will set its subsequent state
   */

  function determineState(&$d, $disable=false)
  {
   if(!is_object($d)||get_class($d)!="Data")
   {
    $this->data->e->add("Unable to determine workflow state, no Data object", "system");
    return false;
   }

   if(!($workflowReferences=$d->check("reference", "workflow")))
   {
    $this->data->e->add("Unable to determine workflow state, \"".$d->module."\" module doesn't contain any workflow references", "module");
    $d->e->add($this->data->e);
    return false;
   }

   if($d->action!="add"&&$d->action!="copy"&&$d->action!="edit")
    return true;

   if($disable&&!is_array($disable))
    $disable=Array($disable);

   if($d->action=="add"||$d->action=="copy") //new entry
   {
    foreach($workflowReferences as $workflowReference)
    {
     $workflowField=$d->reference[$workflowReference]["from"];

     //get states as defined in field's default value (either nr, workflow name or array)
     if(!($states=$d->getDefaultValue($workflowField)))
     {
      $this->data->e->add("Unable to determine workflow state, \"".$d->module."\" module workflow reference \"".$workflowReference."\" doesn't have a default value", "module");
      $d->e->add($this->data->e);
      return false;
     }

     if(!is_array($states))
      $states=Array($states);

     //list all select workflows
     $options=Array();
     foreach($states as $state)
     {
      if(!($info=$this->getWorkflowInfo($state)))
      {
       $d->e->add($this->data->e);
       return false;
      }

      $id=$info["workflow"]["nr"];

      //disable options
      if($disable)
      {
       foreach($info["states"] as $tmp)
       {
        if(!$tmp["workflow"]) //main workflow state
        {
         if(in_array($tmp["code"], $disable)) //check if main state's code is in disabled options
          $id=false; //don't show state id

         break;
        }
       }
      }

      if(!$id) //XXX: ensure cms doesn't show empty options
       continue;

      $options[$id]=$info["workflow"]["name"];
     }

     $d->listOptions($workflowReference, $options);
    }
   }
   else if($d->getValue("nr")) //existing entry
   {
    $d2=clone $d;
    $d2->setAction("view");
    $d2->setExtra("item");

    $d2->setShowFields($workflowReferences);

    if(!$d2->get()||!$d2->next())
    {
     $this->data->e->add("Unable to determine workflow state, \"".$d->module."\" module entry doesn't exist", "module");
     $d->e->add($this->data->e);
     return false;
    }

    foreach($workflowReferences as $workflowReference)
    {
     $workflowField=$d->reference[$workflowReference]["from"];

     if(!($state=$d2->getValue($workflowField)))
      continue;

     $options=$this->getOptions($state, true, $disable); //current state and relevant transitions
     $d->listOptions($workflowReference, $options);
    }
   }

   return true;
  }

  function validateState(&$d, $disable=false)
  {
   if(!$this->determineState($d, $disable))
    return false;

   if($d->action!="add"&&$d->action!="copy"&&($d->action!="edit"||!$d->getValue("nr")))
    return true;

   foreach($d->check("reference", "workflow") as $workflowReference)
   {
    if(!isset($d->listOptions[$workflowReference][$d->getValue($workflowReference)]))
    {
     $workflowField=$d->reference[$workflowReference]["from"];

     $this->data->e->add($d->field[$workflowField]["title"]." option invalid", "error", "Entry: 1");
     $d->e->add($this->data->e);
     return false;
    }
   }

   return true;
  }



  /*
   * Workflows info
   */

  function getWorkflows()
  {
   $d=Data::cache($this->data->module, "view", "all");
   $d->clearValues();

   $d->setValue("workflow", 0);

   $d->setShowFields("name");
   $d->sortOrder("name");

   if(!$d->get())
   {
    $this->data->e->add($d->e);
    return false;
   }

   $info=Array("" => "");
   while($d->next())
    $info[$d->v["nr"]]=$d->v["name"];

   return $info;
  }


  /*
   * Workflow:
   * - workflow name
   * - workflow number (i.e. main state number)
   * - state number
   */

  function getWorkflowInfo($workflow)
  {
   global $_workflowInfo;

   if(!isset($_workflowInfo))
    $_workflowInfo=Array();

   //workflow name
   if(!preg_match("/^\d+$/", $workflow))
   {
    //get from workflow from cache
    foreach($_workflowInfo as $nr => $info)
     if($info["workflow"]["name"]==$workflow)
      return $info;

    //get workflow nr
    $d=Data::cache($this->data->module, "view", "all");
    $d->clearValues();

    $d->setValue("name", $workflow);
    $d->setValue("workflow", 0);

    $d->setShowFields();

    if(!$d->get())
    {
     $this->data->e->add($d->e);
     return false;
    }

    if(!$d->next())
    {
     $this->data->e->add("Unknown workflow: \"".$workflow."\"", "error");
     return false;
    }

    $workflow=$d->v["nr"];
   }

   //check if workflow (nr) is already cached
   if(isset($_workflowInfo[$workflow]))
    return $_workflowInfo[$workflow];

   //get workflow info based on workflow number
   $info=Array(
    "workflow" => Array(),
    "states" => Array(),
    "transitions" => Array(),
   );

   $d=Data::cache($this->data->module, "view", "all");

   $d->clearValues();
   $d->setValue("nr", $workflow);
   $d->setOperant("nr", "OR");
   $d->setValue("workflow", $workflow);

   $d->setShowFields("text", "code", "name", "workflow");
   $d->sortOrder("nr");

   if(!$d->get())
   {
    $this->data->e->add($d->e);
    return false;
   }

   if(!$d->vmax)
   {
    $this->data->e->add("Unknown workflow: \"".$workflow."\"", "error");
    return false;
   }

   if(!$d->search("workflow", 0)) //no main state
   {
    $workflow=$d->getValue("workflow"); //get main state

    $d->clearValues();
    $d->setValue("nr", $workflow);
    $d->setOperant("nr", "OR");
    $d->setValue("workflow", $workflow);

    if(!$d->get())
    {
     $this->data->e->add($d->e);
     return false;
    }

    if(!$d->vmax)
    {
     $this->data->e->add("Unknown workflow: \"".$workflow."\"", "error");
     return false;
    }
   }

   $d->reset();
   while($d->next())
   {
    if($d->v["nr"]==$workflow) //main workflow state
     $info["workflow"]=Array("nr" => $d->v["nr"], "name" => $d->v["name"]); //get workflow name

    $info["states"][$d->v["nr"]]=$d->v;
    unset($info["states"][$d->v["nr"]]["name"]);
   }

   //get transitions
   $d=Data::cache("workflowTransitions", "view", "all");

   $d->clearValues();
   $d->setValue("workflow", $info["workflow"]["nr"]);

   $d->setShowFields("text", "code", "previous", "next", "users", "groups");
   $d->sortOrder("nr");

   if(!$d->get())
   {
    $this->data->e->add($d->e);
    return false;
   }

   while($d->next())
    $info["transitions"][$d->v["previous"]."_".$d->v["next"]]=$d->v;

   $_workflowInfo[$workflow]=$info; //cache info
   return $info;
  }


  function getWorkflowNR($workflow)
  {
   if(!($info=$this->getWorkflowInfo($workflow)))
    return false;

   return $info["workflow"]["nr"];
  }

  function getStatesCode($workflow)
  {
   if(!($info=$this->getWorkflowInfo($workflow)))
    return false;

   $codes=Array();
   foreach($info["states"] as $state => $value)
    $codes[$state]=$value["code"];

   return $codes;
  }

  function getStatesText($workflow)
  {
   if(!($info=$this->getWorkflowInfo($workflow)))
    return false;

   $texts=Array();
   foreach($info["states"] as $state => $value)
    $texts[$state]=$value["text"];

   return $codes;
  }


  function getStateCode($state)
  {
   if(!($info=$this->getWorkflowInfo($state)))
    return false;

   return $info["states"][$state]["code"];
  }

  function getStateText($state)
  {
   if(!($info=$this->getWorkflowInfo($state)))
    return false;

   return $info["states"][$state]["text"];
  }


  function getOptions($state, $includeState=false, $disable=false)
  {
   global $_config;

   if(!($info=$this->getWorkflowInfo($state)))
    return false;

   $options=Array();

   if($includeState)
    $options[$state]=$info["states"][$state]["text"];

   if($disable&&!is_array($disable))
    $disable=Array($disable);

   foreach($info["transitions"] as $transition)
   {
    if($transition["previous"]!=$state) //doesn't transit from current state
     continue;

    $id=$transition["next"];

    //check restrictions
    if($transition["users"]) //users
     if(!in_array($_config->get("usernr"), $transition["users"])) //current user not in allowed users
      $id=false; //don't show state id

    if($transition["groups"]) //user groups
     if(!array_intersect($_config->get("groupnrs"), $transition["groups"])) //current user's groups not in allowed user groups
      $id=false; //don't show state id

    if($disable)
    {
     if(in_array($transition["code"], $disable)) //check if transition's code is in disabled options
      $id=false; //don't show state id
     if(isset($info["states"][$transition["next"]])&&in_array($info["states"][$transition["next"]]["code"], $disable)) //check if next state's code is in disabled options
      $id=false; //don't show state id
    }

    if(!$id)
    {
     if($includeState) //XXX: ensure cms doesn't show empty options (only work because includeState is typically only enabled in the CMS)
      continue;

     while(isset($options[$id]))
      $id.=" ";
    }

    $options[$id]=($includeState?"&nbsp; ":"").$transition["text"];
   }

   return $options;
  }


  /*
   * Script:
   * State Text
   * State Text (STATE CODE)
   * Transition Text > NEXT STATE CODE
   * Transition Text (TRANSITION CODE) > NEXT STATE CODE
   */

  static function parseScript($script)
  {
   if(!($script=explode("\n", $script)))
    return false;

   $workflow=Array();
   foreach($script as $line)
   {
    if(preg_match("/^(.+?)(\s+\((.+?)\))?(\s+>\s*(.+?))?\s*$/", trim($line), $m))
    {
     if(isset($m[5])) //transition
     {
      if(!$workflow) //can't start with a transition
       return false;

      $workflow[]=Array(
       "type" => "transition",
       "text" => $m[1],
       "code" => (isset($m[3])&&$m[3]?$m[3]:strtoupper($m[1])),
       "previous" => $previous,
       "next" => $m[5],
      );
     }
     else //state
     {
      $workflow[]=Array(
       "type" => "state",
       "text" => $m[1],
       "code" => (isset($m[3])&&$m[3]?$m[3]:strtoupper($m[1])),
      );

      $previous=end($workflow)["code"];
     }
    }
   }

   if(!$workflow)
    return false;

   return $workflow;
  }
 }

?>
