<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/config.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."config.php");

 class UT_Config extends UT_Classes
 {
  function Constructor()
  {
   $object=new Config();

   if(!is_object($object)||get_class($object)!="Config")
   {
    $this->e->add("Constructor doesn't return a Config object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new Config();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Config object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("Config object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Set()
  {
   global $_session;

   $config=new Config();

   foreach(Array("value", 123, 0, 0.1, false, true, Array()) as $value)
   {
    if(!$config->set("key", $value))
    {
     $this->e->add("Set shouldn't have failed", "error", $value);
     $this->e->add($config->e);
     $_session->set("config.key");
     return UT_FAILED;
    }

    if($_session->get("config.key")===NULL)
    {
     $this->e->add("Values list doesn't have a key: key", "error", $value);
     $_session->set("config.key");
     return UT_FAILED;
    }

    if($_session->get("config.key")!==$value)
    {
     $this->e->add("Values list value doesn't match for key: key", "error", $value);
     $_session->set("config.key");
     return UT_FAILED;
    }
   }

   if(!$config->set("key"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($config->e);
    $_session->set("config.key");
    return UT_FAILED;
   }

   if($_session->get("config.key")!==NULL)
   {
    $this->e->add("Values list shouldn't have a key: key", "error");
    $_session->set("config.key");
    return UT_FAILED;
   }


   //array
   unset($config->values["key"]);
   if(!$config->set("key.0", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($config->e);
    $_session->set("config.key");
    return UT_FAILED;
   }

   if($_session->get("config.key")===NULL)
   {
    $this->e->add("Values list doesn't have a key: key", "error");
    $_session->set("config.key");
    return UT_FAILED;
   }

   if(!is_array($_session->get("config.key")))
   {
    $this->e->add("Values list value isn't an array for key: key", "error");
    $_session->set("config.key");
    return UT_FAILED;
   }

   if($_session->get("config.key.0")===NULL)
   {
    $this->e->add("Values list doesn't have a key: key.sub", "error");
    $_session->set("config.key");
    return UT_FAILED;
   }

   if($_session->get("config.key.0")!=="value")
   {
    $this->e->add("Values list value doesn't match for key: key.sub", "error");
    $_session->set("config.key");
    return UT_FAILED;
   }

   unset($config->values["key"]);
   if(!$config->set("key.sub", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($config->e);
    $_session->set("config.key");
    return UT_FAILED;
   }

   if($_session->get("config.key")===NULL)
   {
    $this->e->add("Values list doesn't have a key: key", "error");
    $_session->set("config.key");
    return UT_FAILED;
   }

   if(!is_array($_session->get("config.key")))
   {
    $this->e->add("Values list value isn't an array for key: key", "error");
    $_session->set("config.key");
    return UT_FAILED;
   }

   if($_session->get("config.key.sub")===NULL)
   {
    $this->e->add("Values list doesn't have a key: key.sub", "error");
    $_session->set("config.key");
    return UT_FAILED;
   }

   if($_session->get("config.key.sub")!=="value")
   {
    $this->e->add("Values list value doesn't match for key: key.sub", "error");
    $_session->set("config.key");
    return UT_FAILED;
   }


   //object
   unset($config->values["key"]);
   if(!$config->set("key", $this))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($config->e);
    $_session->set("config.key");
    return UT_FAILED;
   }

   if($_session->get("config.key")===NULL)
   {
    $this->e->add("Values list doesn't have a key: key", "error");
    $_session->set("config.key");
    return UT_FAILED;
   }

   if(!is_object($_session->get("config.key")))
   {
    $this->e->add("Values list value isn't an object for key: key", "error");
    $_session->set("config.key");
    return UT_FAILED;
   }

   if(!$config->set("key.sub", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($config->e);
    $_session->set("config.key");
    return UT_FAILED;
   }

   if($_session->get("config.key.sub")===NULL)
   {
    $this->e->add("Values list doesn't have a key: key.sub", "error");
    $_session->set("config.key");
    return UT_FAILED;
   }

   if($_session->get("config.key.sub")!="value")
   {
    $this->e->add("Values list value doesn't match for key: key.sub", "error");
    $_session->set("config.key");
    return UT_FAILED;
   }

   if(!$config->set("key.e.sub", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($config->e);
    $_session->set("config.key");
    return UT_FAILED;
   }

   if($_session->get("config.key.e.sub")===NULL)
   {
    $this->e->add("Values list doesn't have a key: key.e.sub", "error");
    $_session->set("config.key");
    return UT_FAILED;
   }

   if($_session->get("config.key.e.sub")!="value")
   {
    $this->e->add("Values list value doesn't match for key: key.e.sub", "error");
    $_session->set("config.key");
    return UT_FAILED;
   }


   //failures
   unset($config->values["key"]);
   if(!$config->set("key", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($config->e);
    $_session->set("config.key");
    return UT_FAILED;
   }

   if($config->set("key.sub", "value")) //try to use string as object/array
   {
    $this->e->add("Set should have failed", "error");
    $_session->set("config.key");
    return UT_FAILED;
   }

   $_session->set("config.key");
   return UT_OK;
  }

  function Get()
  {
   $config=new Config();

   foreach(Array("value", 123, 0, 0.1, false, true, Array()) as $value)
   {
    if(!$config->set("key", $value))
    {
     $this->e->add("Set shouldn't have failed", "error", $value);
     $this->e->add($config->e);
     return UT_FAILED;
    }

    if($config->get("key")!==$value)
    {
     $this->e->add("Values list value doesn't match for key: key", "error", $value);
     return UT_FAILED;
    }
   }


   //array
   unset($config->values["key"]);
   if(!$config->set("key.sub", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($config->e);
    return UT_FAILED;
   }

   if($config->get("key")!=Array("sub" => "value"))
   {
    $this->e->add("Values list value doesn't match for key: key", "error");
    return UT_FAILED;
   }

   if($config->get("key.sub")!="value")
   {
    $this->e->add("Values list value doesn't match for key: key.sub", "error");
    return UT_FAILED;
   }


   //object
   unset($config->values["key"]);
   if(!$config->set("key", $this))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($config->e);
    return UT_FAILED;
   }

   if($config->get("key")!=$this)
   {
    $this->e->add("Values list value doesn't match for key: key", "error");
    return UT_FAILED;
   }

   if(!$config->set("key.sub", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($config->e);
    return UT_FAILED;
   }

   if($config->get("key.sub")!="value")
   {
    $this->e->add("Values list value doesn't match for key: key.sub", "error");
    return UT_FAILED;
   }

   if(!$config->set("key.e.sub", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($config->e);
    return UT_FAILED;
   }

   if($config->get("key.e.sub")!="value")
   {
    $this->e->add("Values list value doesn't match for key: key.e.sub", "error");
    return UT_FAILED;
   }

   //failures
   if($config->get("asd")!==NULL)
   {
    $this->e->add("Values list value shouldn't be set for key: asd", "error");
    return UT_FAILED;
   }

   unset($config->values["key"]);
   if(!$config->set("key", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($config->e);
    return UT_FAILED;
   }

   if($config->get("key.sub")!==NULL)
   {
    $this->e->add("Values list value shouldn't be set for key: key.sub", "error");
    return UT_FAILED;
   }


   unset($config->values["key"]);
   if(!$config->set("key", Array()))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($config->e);
    return UT_FAILED;
   }

   if($config->get("key.sub")!==NULL)
   {
    $this->e->add("Values list value shouldn't be set for key: key.sub", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function UserAnonymous()
  {
   $config=new Config();

   $usernr=$config->get("usernr");

   if($usernr!=$config->get("anonnr")) //not anonymous
   {
    if($config->userAnonymous())
    {
     $this->e->add("Config user shouldn't be anonymous", "error");
     return UT_FAILED;
    }

    $config->set("usernr", $config->get("anonnr")); //make anonymous
    if(!$config->userAnonymous())
    {
     $this->e->add("Config user should be anonymous", "error");
     $config->set("usernr", $usernr); //restore user
     return UT_FAILED;
    }
   }
   else //anonymous
   {
    if(!$config->userAnonymous())
    {
     $this->e->add("Config user should be anonymous", "error");
     return UT_FAILED;
    }

    $config->set("usernr", 345783045); //make not anonymous
    if($config->userAnonymous())
    {
     $this->e->add("Config user shouldn't be anonymous", "error");
     $config->set("usernr", $usernr); //restore user
     return UT_FAILED;
    }
   }

   $config->set("usernr", $usernr); //restore user
   return UT_OK;
  }

  function UserRoot()
  {
   $config=new Config();

   $usernr=$config->get("usernr");

   if($usernr!=$config->get("rootnr")) //not root
   {
    if($config->userRoot())
    {
     $this->e->add("Config user shouldn't be root", "error");
     return UT_FAILED;
    }

    $config->set("usernr", $config->get("rootnr")); //make root
    if(!$config->userRoot())
    {
     $this->e->add("Config user should be root", "error");
     $config->set("usernr", $usernr); //restore user
     return UT_FAILED;
    }
   }
   else //root
   {
    if(!$config->userRoot())
    {
     $this->e->add("Config user should be root", "error");
     return UT_FAILED;
    }

    $config->set("usernr", 345783045); //make not root
    if($config->userRoot())
    {
     $this->e->add("Config user shouldn't be root", "error");
     $config->set("usernr", $usernr); //restore user
     return UT_FAILED;
    }
   }

   $config->set("usernr", $usernr); //restore user
   return UT_OK;
  }


  function UserBackend()
  {
   $config=new Config();

   $usernr=$config->get("usernr");
   $userbackend=$config->get("userbackend");

   if($usernr!=$config->get("rootnr")) //not root
   {
    if(!$userbackend) //not a backend user
    {
     if($config->userBackend())
     {
      $this->e->add("Config user shouldn't be backend user", "error");
      return UT_FAILED;
     }

     $config->set("userbackend", true); //make backend user
     if(!$config->userBackend())
     {
      $this->e->add("Config user should be backend user", "error");
      $config->set("userbackend", $userbackend); //restore user backend
      $config->set("usernr", $usernr); //restore user
      return UT_FAILED;
     }
    }
    else //backend user
    {
     if(!$config->userBackend())
     {
      $this->e->add("Config user shouldn't be backend user", "error");
      return UT_FAILED;
     }

     $config->set("userbackend", false); //make non-backend user
     if($config->userBackend())
     {
      $this->e->add("Config user shouldn't be backend user", "error");
      $config->set("userbackend", $userbackend); //restore user backend
      $config->set("usernr", $usernr); //restore user
      return UT_FAILED;
     }
    }

    $config->set("usernr", $config->get("rootnr")); //make root
    if(!$config->userBackend())
    {
     $this->e->add("Config user should be backend user", "error");
     $config->set("userbackend", $userbackend); //restore user backend
     $config->set("usernr", $usernr); //restore user
     return UT_FAILED;
    }
   }
   else //root
   {
    if(!$config->userBackend())
    {
     $this->e->add("Config user should be backend user", "error");
     return UT_FAILED;
    }

    $config->set("usernr", 345783045); //make not root
    if(!$userbackend) //not a backend user
    {
     if($config->userBackend())
     {
      $this->e->add("Config user shouldn't be backend user", "error");
      return UT_FAILED;
     }

     $config->set("userbackend", true); //make backend user
     if(!$config->userBackend())
     {
      $this->e->add("Config user should be backend user", "error");
      $config->set("userbackend", $userbackend); //restore user backend
      $config->set("usernr", $usernr); //restore user
      return UT_FAILED;
     }
    }
    else //backend user
    {
     if(!$config->userBackend())
     {
      $this->e->add("Config user shouldn't be backend user", "error");
      return UT_FAILED;
     }

     $config->set("userbackend", false); //make non-backend user
     if($config->userBackend())
     {
      $this->e->add("Config user shouldn't be backend user", "error");
      $config->set("userbackend", $userbackend); //restore user backend
      $config->set("usernr", $usernr); //restore user
      return UT_FAILED;
     }
    }

   }

   $config->set("userbackend", $userbackend); //restore user backend
   $config->set("usernr", $usernr); //restore user
   return UT_OK;
  }


  function GetConfig()
  {
   global $_config;

   $fields=Array(
    "title",
    "usermodule",
    "rootnr",
    "anonnr",
    "usernr",
    "groupnrs",
    "language",
    "sqltimezone",
    "timezone",
   );

   if(!($config=$_config->getConfig()))
   {
    $this->e->add("Config get shouldn't fail", "error");
    return UT_FAILED;
   }

   foreach($fields as $field)
   {
    if(!isset($config[$field])||!$config[$field])
    {
     $this->e->add("Config get should have set a ".$field, "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }

  function GetCached()
  {
   global $_config;

   $d=Data::cache("config", "view", "all", Array("format" => "text"));
   $d->setShowFields("nr", "cache");

   if(!$d->get()||!$d->next())
   {
    $this->e->add("Unable to get config cache", "warning");
    $this->e->add($d->e);
    return UT_SKIPPED;
   }

   $du=Data::cache("config", "edit", "item", Array("format" => "text"));
   $du->clearValues();
   $du->setValue("nr", $d->v["nr"]);

   $test=Array("value", 123, 0, 0.1, false, true, Array());

   //change cache
   $du->setValue("cache", serialize($test));
   $_config->su(true); $success=$du->submit(); $_config->su();

   //prep for restore cache
   $du->setValue("cache", $d->v["cache"]);

   if(!$success) //unable to change cache, subsequent test will probably fail as well
   {
    $this->e->add("Unable to change config cache", "error");
    $this->e->add($du->e);
    return UT_FAILED;
   }

   if(!($newcache=$_config->getCached()))
   {
    $this->e->add("Config get cached shouldn't fail", "error");
    $_config->su(true); $du->submit(); $_config->su(); //restore original cache
    return UT_FAILED;
   }

   if(!$this->_compare($test, $newcache))
   {
    $this->e->add("Config get cached doesn't match expected outcome", "error");
    $_config->su(true); $du->submit(); $_config->su(); //restore original cache
    return UT_FAILED;
   }

   $_config->su(true); $du->submit(); $_config->su(); //restore original cache

   return UT_OK;
  }

  function UpdateCache()
  {
   global $_config;

   $d=Data::cache("config", "view", "all", Array("format" => "text"));
   $d->setShowFields("nr", "cache");

   if(!$d->get()||!$d->next())
   {
    $this->e->add("Unable to get config cache", "warning");
    $this->e->add($d->e);
    return UT_SKIPPED;
   }

   $du=Data::cache("config", "edit", "item", Array("format" => "text"));
   $du->clearValues();
   $du->setValue("nr", $d->v["nr"]);

   //clear cache
   $du->setValue("cache", "");
   $_config->su(true); $success=$du->submit(); $_config->su();

   //prep for restore cache
   $du->setValue("cache", $d->v["cache"]);

   if(!$success) //unable to change cache, subsequent test will probably fail as well
   {
    $this->e->add("Unable to clear config cache", "error");
    $this->e->add($du->e);
    return UT_FAILED;
   }

   if(!$_config->updateCache())
   {
    $this->e->add("Config update cached shouldn't fail", "error");
    $_config->su(true); $du->submit(); $_config->su(); //restore original cache
    return UT_FAILED;
   }

   if(!($newcache=$_config->getCached()))
   {
    $this->e->add("Config get cached shouldn't fail", "error");
    $_config->su(true); $du->submit(); $_config->su(); //restore original cache
    return UT_FAILED;
   }

   if(!$this->_compare($_config->getConfig(), $newcache))
   {
    $this->e->add("Config updated cache doesn't match expected outcome", "error");
    $_config->su(true); $du->submit(); $_config->su(); //restore original cache
    return UT_FAILED;
   }

   $_config->su(true); $du->submit(); $_config->su(); //restore original cache

   return UT_OK;
  }


  function Load()
  {
   global $_config, $_session;

   $fields=Array(
    "title",
    "usermodule",
    "rootnr",
    "anonnr",
    "usernr",
    "groupnrs",
    "language",
    "sqltimezone",
    "timezone",
   );

   $config=$_session->get("config");

//no config
   $_session->set("config");

   if(!$_config->load())
   {
    $this->e->add("Config load shouldn't fail", "error");
    $_session->set("config", $config); //restore config
    return UT_FAILED;
   }

   foreach($fields as $field)
   {
    if(!$_config->get($field))
    {
     $this->e->add("Config load should have set a ".$field, "error");
     $_session->set("config", $config); //restore config
     return UT_FAILED;
    }
   }

//don't reload config
   $_config->set("title", "ASDADASDAS");
   $_config->set("usernr", "345783045");

   if(!$_config->load())
   {
    $this->e->add("Config load shouldn't fail", "error");
    $_session->set("config", $config); //restore config
    return UT_FAILED;
   }

   foreach($fields as $field)
   {
    if(!$_config->get($field))
    {
     $this->e->add("Config load should have set a ".$field, "error");
     $_session->set("config", $config); //restore config
     return UT_FAILED;
    }
   }

   if($_config->get("title")!="ASDADASDAS")
   {
    $this->e->add("Config load shouldn't have changed: title", "error");
    $_session->set("config", $config); //restore config
    return UT_FAILED;
   }

   if($_config->get("usernr")!="345783045")
   {
    $this->e->add("Config load shouldn't have changed: usernr", "error");
    $_session->set("config", $config); //restore config
    return UT_FAILED;
   }

//reload config
   $_config->set("reload", true);
   $_config->set("title", "ASDADASDAS");
   $_config->set("usernr", "345783045");

   if(!$_config->load())
   {
    $this->e->add("Config load shouldn't fail", "error");
    $_session->set("config", $config); //restore config
    return UT_FAILED;
   }

   foreach($fields as $field)
   {
    if(!$_config->get($field))
    {
     $this->e->add("Config load should have set a ".$field, "error");
     $_session->set("config", $config); //restore config
     return UT_FAILED;
    }
   }

   if($_config->get("title")=="ASDADASDAS")
   {
    $this->e->add("Config load should have changed: title", "error");
    $_session->set("config", $config); //restore config
    return UT_FAILED;
   }

   if($_config->get("usernr")!="345783045")
   {
    $this->e->add("Config load shouldn't have changed: usernr", "error");
    $_session->set("config", $config); //restore config
    return UT_FAILED;
   }


   $_session->set("config", $config); //restore config
   return UT_OK;
  }

  function Reload()
  {
   global $_config, $_session;

   $fields=Array(
    "title",
    "usermodule",
    "rootnr",
    "anonnr",
    "usernr",
    "groupnrs",
    "language",
    "sqltimezone",
    "timezone",
   );

   $config=$_session->get("config");

   $_config->set("title", "ASDADASDAS");
   $_config->set("usernr", "345783045");

   if(!$_config->reload())
   {
    $this->e->add("Config reload shouldn't fail", "error");
    $_session->set("config", $config); //restore config
    return UT_FAILED;
   }

   foreach($fields as $field)
   {
    if(!$_config->get($field))
    {
     $this->e->add("Config reload should have set a ".$field, "error");
     $_session->set("config", $config); //restore config
     return UT_FAILED;
    }
   }

   if($_config->get("title")=="ASDADASDAS")
   {
    $this->e->add("Config reload should have changed: title", "error");
    $_session->set("config", $config); //restore config
    return UT_FAILED;
   }

   if($_config->get("usernr")!="345783045")
   {
    $this->e->add("Config reload shouldn't have changed: usernr", "error");
    $_session->set("config", $config); //restore config
    return UT_FAILED;
   }


   $_session->set("config", $config); //restore config
   return UT_OK;
  }


  function Merge()
  {
   global $_config, $_session;

   $config=$_session->get("config");

//clear config cache
   $_config->set("api");

   if(!($merged=$_config->merge("API", Array("data" => true))))
   {
    $this->e->add("Config merge shouldn't fail", "error");
    $_session->set("config", $config); //restore config
    return UT_FAILED;
   }

   if($merged["csv"]||$merged["xml"])
   {
    $this->e->add("Config merge should have used the default values for CSV/XML API", "error");
    $_session->set("config", $config); //restore config
    return UT_FAILED;
   }

   if(!$merged["data"])
   {
    $this->e->add("Config merge should have used the overwrite value for Data API", "error");
    $_session->set("config", $config); //restore config
    return UT_FAILED;
   }

//defined config cache
   $_config->set("api.csv", true);
   $_config->set("api.data", true);
   $_config->set("api.xml", true);

   if(!($merged=$_config->merge("API", Array("data" => false))))
   {
    $this->e->add("Config merge shouldn't fail", "error");
    $_session->set("config", $config); //restore config
    return UT_FAILED;
   }

   if(!$merged["csv"]||!$merged["xml"])
   {
    $this->e->add("Config merge should have used the configured values for CSV/XML API", "error");
    $_session->set("config", $config); //restore config
    return UT_FAILED;
   }

   if($merged["data"])
   {
    $this->e->add("Config merge should have used the overwrite value for Data API", "error");
    $_session->set("config", $config); //restore config
    return UT_FAILED;
   }

   $_session->set("config", $config); //restore config
   return UT_OK;
  }
 }


 class UT_SU_Function extends UT_Config
 {
  function _getUsers()
  {
   global $_config;

   $d=Data::cache($_config->get("usermodule"), "view", "item");

   if(!($idField=$d->getID())) //no id field
   {
    $this->e->add("Get users failed, no ID field found");
    return false;
   }

   if(!($userField=$d->check("properties", "username"))) //no username field
   {
    $this->e->add("Get users failed, no username field found");
    return false;
   }

   if(sizeof($userField)!=1) //multiple username fields, not supported
   {
    $this->e->add("Get users failed, multiple username fields not supported");
    return false;
   }

   $userField=$userField[0];
   $d->setShowFields($userField);

   $d->clearValues();
   $d->setValue($idField, $_config->get("anonnr"), 0);
   $d->setValue($idField, $_config->get("rootnr"), 1);

   if(!$d->get())
   {
    $this->e->add("Get users failed, unable to gather users");
    $this->e->add($d->e);
    return false;
   }

   $users=Array();
   while($d->next())
   {
    if($d->v[$idField]==$_config->get("rootnr"))
     $users["root"]=Array("id" => $d->v[$idField], "username" => $d->v[$userField]);
    else if($d->v[$idField]==$_config->get("anonnr"))
     $users["anon"]=Array("id" => $d->v[$idField], "username" => $d->v[$userField]);
   }

   if(!isset($users["anon"])||!isset($users["root"]))
   {
    $this->e->add("Get users failed, anon/root doesn't exist");
    return false;
   }

   return $users;
  }

  function SU()
  {
   global $_config;

   $usernr=$_config->get("usernr"); //store usernr
   $_config->set("usernr", $_config->get("anonnr")); //become anon


   //normal scenario
   if(!$_config->su(true)) //become root
   {
    $this->e->add("Su failed, unable to become root");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("rootnr"))
   {
    $this->e->add("Su failed, user didn't become root");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if(!$_config->su()) //revert
   {
    $this->e->add("Su failed, unable to revert to original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("anonnr"))
   {
    $this->e->add("Su failed, user didn't revert to original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }


   //become root twice fail
   if(!$_config->su(true)) //become root
   {
    $this->e->add("Su failed, unable to become root");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("rootnr"))
   {
    $this->e->add("Su failed, user didn't become root");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->su(true)) //become root again
   {
    $this->e->add("Su should fail to become root twice");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("rootnr"))
   {
    $this->e->add("Su failed, user should have remained root");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if(!$_config->su()) //revert
   {
    $this->e->add("Su failed, unable to revert to original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("anonnr"))
   {
    $this->e->add("Su failed, user didn't revert to original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }


   //revert twice fail
   if($_config->su()) //revert
   {
    $this->e->add("Su should fail to revert to original user when already original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("anonnr"))
   {
    $this->e->add("Su failed, user should have remained original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }


   $_config->set("usernr", $usernr); //restore usernr
   $_config->set("suusernr"); //ensure no leftovers
   return UT_OK;
  }

  function SUUser()
  {
   global $_config;

   if(!($users=$this->_getUsers()))
    return UT_SKIPPED;

   $usernr=$_config->get("usernr"); //store usernr

   //normal scenario
   $_config->set("usernr", $_config->get("rootnr")); //become root

   if(!$_config->su($users["root"]["username"])) //become root
   {
    $this->e->add("Su failed, unable to become root");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("rootnr"))
   {
    $this->e->add("Su failed, user didn't become root");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if(!$_config->su()) //revert
   {
    $this->e->add("Su failed, unable to revert to original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("rootnr"))
   {
    $this->e->add("Su failed, user didn't revert to original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if(!$_config->su($users["anon"]["username"])) //become anon
   {
    $this->e->add("Su failed, unable to become anon");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("anonnr"))
   {
    $this->e->add("Su failed, user didn't become anon");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if(!$_config->su()) //revert
   {
    $this->e->add("Su failed, unable to revert to original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("rootnr"))
   {
    $this->e->add("Su failed, user didn't revert to original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }


   //normal scenario using ID
   $_config->set("usernr", $_config->get("rootnr")); //become root

   if(!$_config->su($users["root"]["id"])) //become root
   {
    $this->e->add("Su failed, unable to become root");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("rootnr"))
   {
    $this->e->add("Su failed, user didn't become root");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if(!$_config->su()) //revert
   {
    $this->e->add("Su failed, unable to revert to original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("rootnr"))
   {
    $this->e->add("Su failed, user didn't revert to original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if(!$_config->su($users["anon"]["id"])) //become anon
   {
    $this->e->add("Su failed, unable to become anon");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("anonnr"))
   {
    $this->e->add("Su failed, user didn't become anon");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if(!$_config->su()) //revert
   {
    $this->e->add("Su failed, unable to revert to original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("rootnr"))
   {
    $this->e->add("Su failed, user didn't revert to original user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }


   //not root fail
   $_config->set("usernr", $_config->get("anonnr")); //become anon

   if($_config->su($users["root"]["username"])) //become root
   {
    $this->e->add("Su should fail, non root users aren't allowed to change to a specific user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("anonnr"))
   {
    $this->e->add("Su failed, user changed");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->su($users["anon"]["username"])) //become anon
   {
    $this->e->add("Su should fail, non root users aren't allowed to change to a specific user");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("anonnr"))
   {
    $this->e->add("Su failed, user changed");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }


   //su twice fail
   if(!$_config->su(true)) //become root
   {
    $this->e->add("Su failed, unable to become root");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("rootnr"))
   {
    $this->e->add("Su failed, user didn't become root");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->su($users["root"]["username"])) //become root
   {
    $this->e->add("Su should fail, su already used");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("rootnr"))
   {
    $this->e->add("Su failed, user changed");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->su($users["anon"]["username"])) //become anon
   {
    $this->e->add("Su should fail, su already used");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }

   if($_config->get("usernr")!=$_config->get("rootnr"))
   {
    $this->e->add("Su failed, user changed");
    $_config->set("usernr", $usernr); //restore usernr
    $_config->set("suusernr"); //ensure no leftovers
    return UT_FAILED;
   }


   $_config->set("usernr", $usernr); //restore usernr
   $_config->set("suusernr"); //ensure no leftovers
   return UT_OK;
  }
 }


?>
