<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/dt.sro.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["plugins"]."unit_testing/ut/dt.php");
 require_once($_path["include"]."dt.php");

 class UT_DT_Scope_Resolution_Operator extends UT_DT
 {
  function CallStatic()
  {
   $format="Y-m-d H:i:s";
   $time="2012-09-04 23:15:10";

   if(!DT::dateExtractS($time, $format))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   /*
   if(($nonexistingexecutable=DT::nonexistingexecutableSS()))
   {
    $this->e->add("nonexistingexecutableS should fail", "error");
    return UT_FAILED;
   }
   */

   return UT_OK;
  }


  function GetTimezone()
  {
   $current=date_default_timezone_get();

   if(DT::getTimezoneS()!=$current)
   {
    $this->e->add("Get timezone doesn't match: ".$current, "error");
    return UT_FAILED;
   }

   date_default_timezone_set("UTC");

   if(DT::getTimezoneS()!="UTC")
   {
    date_default_timezone_set($current);
    $this->e->add("Get timezone doesn't match: UTC", "error");
    return UT_FAILED;
   }

   date_default_timezone_set("Europe/Amsterdam");

   if(DT::getTimezoneS()!="Europe/Amsterdam")
   {
    date_default_timezone_set($current);
    $this->e->add("Get timezone doesn't match: Europe/Amsterdam", "error");
    return UT_FAILED;
   }

   date_default_timezone_set("Asia/Tokyo");

   if(DT::getTimezoneS()!="Asia/Tokyo")
   {
    date_default_timezone_set($current);
    $this->e->add("Get timezone doesn't match: Asia/Tokyo", "error");
    return UT_FAILED;
   }

   date_default_timezone_set($current);

   //system timezone
   if(!($system=exec("date +%Z")))
   {
    $this->e->add("Get system timezone failed", "error");
    return UT_FAILED;
   }
   $system=timezone_name_from_abbr($system);

   if(DT::getTimezoneS(true)!=$system)
   {
    $this->e->add("Get system timezone doesn't match: ".$system, "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function SetTimezone()
  {
   global $_config;

   $current=date_default_timezone_get();

   if(!DT::setTimezoneS("UTC"))
   {
    date_default_timezone_set($current);
    $this->e->add("Set timezone shouldn't have failed", "error");
    return UT_FAILED;
   }

   if(DT::getTimezoneS()!="UTC")
   {
    date_default_timezone_set($current);
    $this->e->add("Set timezone didn't set timezone to: UTC", "error");
    return UT_FAILED;
   }

   if(!DT::setTimezoneS("Europe/Amsterdam"))
   {
    date_default_timezone_set($current);
    $this->e->add("Set timezone shouldn't have failed", "error");
    return UT_FAILED;
   }

   if(DT::getTimezoneS()!="Europe/Amsterdam")
   {
    date_default_timezone_set($current);
    $this->e->add("Set timezone didn't set timezone to: Europe/Amsterdam", "error");
    return UT_FAILED;
   }

   if(!DT::setTimezoneS("Asia/Tokyo"))
   {
    date_default_timezone_set($current);
    $this->e->add("Set timezone shouldn't have failed", "error");
    return UT_FAILED;
   }

   if(DT::getTimezoneS()!="Asia/Tokyo")
   {
    date_default_timezone_set($current);
    $this->e->add("Set timezone didn't set timezone to: Asia/Tokyo", "error");
    return UT_FAILED;
   }

   if(!DT::setTimezoneS($current))
   {
    date_default_timezone_set($current);
    $this->e->add("Set timezone shouldn't have failed", "error");
    return UT_FAILED;
   }

   if(DT::getTimezoneS()!=$current)
   {
    date_default_timezone_set($current);
    $this->e->add("Set timezone didn't set timezone to: ".$current, "error");
    return UT_FAILED;
   }

   if(!DT::setTimezoneS())
   {
    date_default_timezone_set($current);
    $this->e->add("Set timezone shouldn't have failed", "error");
    return UT_FAILED;
   }

   if(DT::getTimezoneS()!=$_config->get("timezone"))
   {
    date_default_timezone_set($current);
    $this->e->add("Set timezone didn't set timezone to: ".$_config->get("timezone"), "error");
    return UT_FAILED;
   }

   if(DT::setTimezoneS("asdasdasd"))
   {
    date_default_timezone_set($current);
    $this->e->add("Set timezone should have failed", "error");
    return UT_FAILED;
   }

   date_default_timezone_set($current);

   return UT_OK;
  }


  function Date()
  {
   $format="Y-m-d H:i:s";

   $date=date($format);
   if(!($output=DT::dateS($format)))
   {
    $this->e->add("Date shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=$output)
   {
    $this->e->add("Date output incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateExtract()
  {
   $format="Y-m-d H:i:s";
   $time="2012-09-04 23:15:10";

   if(!($date=DT::dateExtractS($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 9,
    "day" => 4,
    "hour" => 23,
    "minute" => 15,
    "second" => 10,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateExtractArray()
  {
   $format="Y-m-d H:i:s";
   $time1="2012-09-04 23:15:10";
   $time2="1920-10-23 01:02:03";

   if(!($date=DT::dateExtractS(Array($time1, $time2), $format)))
   {
    $this->e->add("Date extract array shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    Array(
     "year" => 2012,
     "month" => 9,
     "day" => 4,
     "hour" => 23,
     "minute" => 15,
     "second" => 10,
    ),
    Array(
     "year" => 1920,
     "month" => 10,
     "day" => 23,
     "hour" => 1,
     "minute" => 2,
     "second" => 3,
    ),
   ))
   {
    $this->e->add("Date extract array output incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateConvert()
  {
//date
   $tstamp=time();
   $inf="Y-m-d";
   $outf="d-m-Y";

   $in=date($inf, $tstamp);
   $out=date($outf, $tstamp);

   if(DT::dateConvertS($in, $inf, $outf)!==$out)
   {
    $this->e->add("Converting datetime from ".$inf." to ".$outf." failed", "error");
    return UT_FAILED;
   }

//date time
   $inf="Y-m-d H:i:s";
   $outf="H:i d-m-Y";

   $in=date($inf, $tstamp);
   $out=date($outf, $tstamp);

   if(DT::dateConvertS($in, $inf, $outf)!==$out)
   {
    $this->e->add("Converting datetime from ".$inf." to ".$outf." failed", "error");
    return UT_FAILED;
   }

//to unix timestamp
   $inf="d m Y H i s";
   $outf=false;

   $in=date($inf, $tstamp);
   $out=$tstamp;

   if(DT::dateConvertS($in, $inf, $outf)!==$out)
   {
    $this->e->add("Converting datetime from ".$inf." to ".$outf." failed", "error");
    return UT_FAILED;
   }

//one big input string
   $inf="YmdHis";
   $outf="j n y G i s";

   $in=date($inf, $tstamp);
   $out=date($outf, $tstamp);

   if(DT::dateConvertS($in, $inf, $outf)!==$out)
   {
    $this->e->add("Converting datetime from ".$inf." to ".$outf." failed", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function DateConvertArray()
  {
   $tstamp=time();
   $inf="Y-m-d H:i:s";
   $outf="H:i d-m-Y";

   $in1=date($inf, $tstamp);
   $out1=date($outf, $tstamp);

   $in2=date($inf, $tstamp+73945);
   $out2=date($outf, $tstamp+73945);

   if(!($values=DT::dateConvertS(Array($in1, $in2), $inf, $outf)))
   {
    $this->e->add("Date time array convert failed", "error");
    return UT_FAILED;
   }
   if($values[0]!=$out1)
   {
    $this->e->add("Date time array convert argument 1 incorrect", "error");
    return UT_FAILED;
   }
   if($values[1]!=$out2)
   {
    $this->e->add("Date time array convert argument 2 incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateDelta()
  {
   $format="Y-m-d H:i:s";
   $time="2012-09-04 23:15:10";

   if(DT::dateDeltaS($time, 5, "second", $format)!="2012-09-04 23:15:15")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if(DT::dateDeltaS($time, -15, "seconds", $format)!="2012-09-04 23:14:55")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if(DT::dateDeltaS($time, 55, "minute", $format)!="2012-09-05 00:10:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if(DT::dateDeltaS($time, -16, "minutes", $format)!="2012-09-04 22:59:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if(DT::dateDeltaS($time, 2, "hour", $format)!="2012-09-05 01:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if(DT::dateDeltaS($time, -28, "hours", $format)!="2012-09-03 19:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if(DT::dateDeltaS($time, 13, "day", $format)!="2012-09-17 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if(DT::dateDeltaS($time, -15, "days", $format)!="2012-08-20 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if(DT::dateDeltaS($time, 5, "week", $format)!="2012-10-09 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if(DT::dateDeltaS($time, -2, "weeks", $format)!="2012-08-21 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if(DT::dateDeltaS($time, 5, "month", $format)!="2013-02-04 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if(DT::dateDeltaS($time, -15, "months", $format)!="2011-06-04 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if(DT::dateDeltaS($time, 24, "year", $format)!="2036-09-04 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if(DT::dateDeltaS($time, -15, "years", $format)!="1997-09-04 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateDeltaArray()
  {
   $format="Y-m-d H:i:s";
   $time1="2012-09-04 23:15:10";
   $time2="1920-10-23 01:02:03";

   if(DT::dateDeltaS(Array($time1, $time2), -70, "second", $format)!=Array("2012-09-04 23:14:00", "1920-10-23 01:00:53"))
   {
    $this->e->add("Date delta array output incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function TimezoneConvert()
  {
   $format="Y-m-d H:i:s";

   $intime="2012-05-27 13:39:29";
   $intz="Europe/Amsterdam";

   $outtime="2012-05-27 20:39:29";
   $outtz="Asia/Tokyo";

   if(DT::timezoneConvertS($intime, $intz, $outtz, $format)!==$outtime)
   {
    $this->e->add("Converting timezone from ".$intz." to ".$outtz." failed", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function TimezoneConvertArray()
  {
   $format="Y-m-d H:i:s";

   $intime1="2012-01-15 23:15:10";
   $intime2="2012-05-27 13:39:29";
   $intz="Europe/Amsterdam";

   $outtime1="2012-01-16 07:15:10";
   $outtime2="2012-05-27 20:39:29";
   $outtz="Asia/Tokyo";

   if(!($values=DT::timezoneConvertS(Array($intime1, $intime2), $intz, $outtz, $format)))
   {
    $this->e->add("Converting timezone from ".$intz." to ".$outtz." failed", "error");
    return UT_FAILED;
   }
   if($values[0]!=$outtime1)
   {
    $this->e->add("Timezone array convert argument 1 incorrect", "error");
    return UT_FAILED;
   }
   if($values[1]!=$outtime2)
   {
    $this->e->add("Timezone array convert argument 2 incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function TimeAgo()
  {
   $format="Y-m-d H:i:s";

   //seconds
   if(DT::timeAgoS(date($format, time()-86400), $format, "seconds")!="86400 seconds ago")
   {
    $this->e->add("Time 86400 seconds ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-2), $format, "seconds")!="2 seconds ago")
   {
    $this->e->add("Time 2 seconds ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-1), $format, "seconds")!="1 second ago")
   {
    $this->e->add("Time 1 second ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-0), $format, "seconds")!="just now")
   {
    $this->e->add("Time 0 seconds ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+1), $format, "seconds")!="in 1 second")
   {
    $this->e->add("Time in 1 second doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+2), $format, "seconds")!="in 2 seconds")
   {
    $this->e->add("Time in 2 seconds doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+86400), $format, "seconds")!="in 86400 seconds")
   {
    $this->e->add("Time in 86400 seconds doesn't match", "error");
    return UT_FAILED;
   }

   //minutes
   if(DT::timeAgoS(date($format, time()-86400), $format, "minutes")!="1440 minutes ago")
   {
    $this->e->add("Time 1440 minutes ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-2*60), $format, "minutes")!="2 minutes ago")
   {
    $this->e->add("Time 2 minutes ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-60), $format, "minutes")!="1 minute ago")
   {
    $this->e->add("Time 1 minute ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-30), $format, "minutes")!="less than a minute ago")
   {
    $this->e->add("Time 0 minutes ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-0), $format, "minutes")!="less than a minute ago")
   {
    $this->e->add("Time 0 minutes ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+30), $format, "minutes")!="in less than a minute")
   {
    $this->e->add("Time in 0 minutes doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+60), $format, "minutes")!="in 1 minute")
   {
    $this->e->add("Time in 1 minute doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+2*60), $format, "minutes")!="in 2 minutes")
   {
    $this->e->add("Time in 2 minutes doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+86400), $format, "minutes")!="in 1440 minutes")
   {
    $this->e->add("Time in 1440 minutes doesn't match", "error");
    return UT_FAILED;
   }

   //hours
   if(DT::timeAgoS(date($format, time()-86400), $format, "hours")!="24 hours ago")
   {
    $this->e->add("Time 24 hours ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-2*3600), $format, "hours")!="2 hours ago")
   {
    $this->e->add("Time 2 hours ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-3600), $format, "hours")!="1 hour ago")
   {
    $this->e->add("Time 1 hour ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-1800), $format, "hours")!="less than an hour ago")
   {
    $this->e->add("Time 0 hours ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-0), $format, "hours")!="less than an hour ago")
   {
    $this->e->add("Time 0 hours ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+1800), $format, "hours")!="in less than an hour")
   {
    $this->e->add("Time in 0 hours doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+3600), $format, "hours")!="in 1 hour")
   {
    $this->e->add("Time in 1 hour doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+2*3600), $format, "hours")!="in 2 hours")
   {
    $this->e->add("Time in 2 hours doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+86400), $format, "hours")!="in 24 hours")
   {
    $this->e->add("Time in 24 hours doesn't match", "error");
    return UT_FAILED;
   }

   //days
   if(DT::timeAgoS(date($format, time()-24*86400), $format, "days")!="24 days ago")
   {
    $this->e->add("Time 24 days ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-2*86400), $format, "days")!="2 days ago")
   {
    $this->e->add("Time 2 days ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-86400), $format, "days")!="1 day ago")
   {
    $this->e->add("Time 1 day ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-43200), $format, "days")!="today")
   {
    $this->e->add("Time 0 days ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-0), $format, "days")!="today")
   {
    $this->e->add("Time 0 days ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+43200), $format, "days")!="today")
   {
    $this->e->add("Time in 0 days doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+86400), $format, "days")!="in 1 day")
   {
    $this->e->add("Time in 1 day doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+2*86400), $format, "days")!="in 2 days")
   {
    $this->e->add("Time in 2 days doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+24*86400), $format, "days")!="in 24 days")
   {
    $this->e->add("Time in 24 days doesn't match", "error");
    return UT_FAILED;
   }

   //weeks
   if(DT::timeAgoS(date($format, time()-24*7*86400), $format, "weeks")!="24 weeks ago")
   {
    $this->e->add("Time 24 weeks ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-2*7*86400), $format, "weeks")!="2 weeks ago")
   {
    $this->e->add("Time 2 weeks ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-7*86400), $format, "weeks")!="1 week ago")
   {
    $this->e->add("Time 1 week ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-4*86400), $format, "weeks")!="this week")
   {
    $this->e->add("Time 0 weeks ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-0), $format, "weeks")!="this week")
   {
    $this->e->add("Time 0 weeks ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+4*86400), $format, "weeks")!="coming week")
   {
    $this->e->add("Time in 0 weeks doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+7*86400), $format, "weeks")!="in 1 week")
   {
    $this->e->add("Time in 1 week doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+2*7*86400), $format, "weeks")!="in 2 weeks")
   {
    $this->e->add("Time in 2 weeks doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+24*7*86400), $format, "weeks")!="in 24 weeks")
   {
    $this->e->add("Time in 24 weeks doesn't match", "error");
    return UT_FAILED;
   }

   //months
   if(DT::timeAgoS(date($format, mktime(0, 0, 0, date("m")-24)), $format, "months")!="24 months ago")
   {
    $this->e->add("Time 24 months ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, mktime(0, 0, 0, date("m")-2)), $format, "months")!="2 months ago")
   {
    $this->e->add("Time 2 months ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, mktime(0, 0, 0, date("m")-1)), $format, "months")!="1 month ago")
   {
    $this->e->add("Time 1 month ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-10), $format, "months")!="this month")
   {
    $this->e->add("Time 0 months ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-0), $format, "months")!="this month")
   {
    $this->e->add("Time 0 months ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+10), $format, "months")!="coming month")
   {
    $this->e->add("Time in 0 months doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, mktime(0, 0, 0, date("m")+1)), $format, "months")!="in 1 month")
   {
    $this->e->add("Time in 1 month doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, mktime(0, 0, 0, date("m")+2)), $format, "months")!="in 2 months")
   {
    $this->e->add("Time in 2 months doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, mktime(0, 0, 0, date("m")+24)), $format, "months")!="in 24 months")
   {
    $this->e->add("Time in 24 months doesn't match", "error");
    return UT_FAILED;
   }

   //years
   if(DT::timeAgoS(date($format, time()-24*365*86400), $format, "years")!="24 years ago")
   {
    $this->e->add("Time 24 years ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-2*365*86400), $format, "years")!="2 years ago")
   {
    $this->e->add("Time 2 years ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-365*86400), $format, "years")!="1 year ago")
   {
    $this->e->add("Time 1 year ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-180*86400), $format, "years")!="this year")
   {
    $this->e->add("Time 0 years ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-0), $format, "years")!="this year")
   {
    $this->e->add("Time 0 years ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+180*86400), $format, "years")!="coming year")
   {
    $this->e->add("Time in 0 years doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+365*86400), $format, "years")!="in 1 year")
   {
    $this->e->add("Time in 1 year doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+2*365*86400), $format, "years")!="in 2 years")
   {
    $this->e->add("Time in 2 years doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+10*365*86400), $format, "years")!="in 10 years")
   {
    $this->e->add("Time in 10 years doesn't match", "error");
    return UT_FAILED;
   }

   //autodetect
   if(DT::timeAgoS(date($format, time()-24*365*86400), $format)!="24 years ago")
   {
    $this->e->add("Time 24 years ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-365*86400), $format)!="1 year ago")
   {
    $this->e->add("Time 1 year ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, mktime(0, 0, 0, date("m")-11)), $format)!="11 months ago")
   {
    $this->e->add("Time 11 months ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, mktime(0, 0, 0, date("m")-1)), $format)!="1 month ago")
   {
    $this->e->add("Time 1 months ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-3*7*86400), $format)!="3 weeks ago")
   {
    $this->e->add("Time 3 weeks ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-7*86400), $format)!="1 week ago")
   {
    $this->e->add("Time 1 week ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-6*86400), $format)!="6 days ago")
   {
    $this->e->add("Time 6 days ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-86400), $format)!="1 day ago")
   {
    $this->e->add("Time 1 day ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-23*3600), $format)!="23 hours ago")
   {
    $this->e->add("Time 23 hours ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-3600), $format)!="1 hour ago")
   {
    $this->e->add("Time 1 hour ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-59*60), $format)!="59 minutes ago")
   {
    $this->e->add("Time 59 minutes ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-60), $format)!="1 minute ago")
   {
    $this->e->add("Time 1 minute ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-59), $format)!="59 seconds ago")
   {
    $this->e->add("Time 59 seconds ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()-1), $format)!="1 second ago")
   {
    $this->e->add("Time 1 second ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()), $format)!="just now")
   {
    $this->e->add("Time 0 second ago doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+1), $format)!="in 1 second")
   {
    $this->e->add("Time in 1 second doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+59), $format)!="in 59 seconds")
   {
    $this->e->add("Time in 59 seconds doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+60), $format)!="in 1 minute")
   {
    $this->e->add("Time in 1 minute doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+59*60), $format)!="in 59 minutes")
   {
    $this->e->add("Time in 59 minutes doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+3600), $format)!="in 1 hour")
   {
    $this->e->add("Time in 1 hour doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+23*3600), $format)!="in 23 hours")
   {
    $this->e->add("Time in 23 hours doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+86400), $format)!="in 1 day")
   {
    $this->e->add("Time in 1 day doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+6*86400), $format)!="in 6 days")
   {
    $this->e->add("Time in 6 days doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+7*86400), $format)!="in 1 week")
   {
    $this->e->add("Time in 1 week doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+3*7*86400), $format)!="in 3 weeks")
   {
    $this->e->add("Time in 3 weeks doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, mktime(0, 0, 0, date("m")+1)), $format)!="in 1 month")
   {
    $this->e->add("Time in 1 months doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, mktime(0, 0, 0, date("m")+11)), $format)!="in 11 months")
   {
    $this->e->add("Time in 11 months doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+365*86400), $format)!="in 1 year")
   {
    $this->e->add("Time in 1 year doesn't match", "error");
    return UT_FAILED;
   }
   if(DT::timeAgoS(date($format, time()+10*365*86400), $format)!="in 10 years")
   {
    $this->e->add("Time in 10 years doesn't match", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

 }
?>
