<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/filter.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."filter.php");

 class UT_Filter extends UT_Classes
 {
  function Constructor()
  {
   $object=new Filter();

   if(!is_object($object)||get_class($object)!="Filter")
   {
    $this->e->add("Constructor doesn't return a Filter object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->cms)||get_class($object->cms)!="CMS")
   {
    $this->e->add("Object: \"cms\" isn't an CMS object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   //config merge
   $object=new Filter(Array("showTitles" => false, "showFields" => false));

   if($object->config["showTitles"])
   {
    $this->e->add("Filter shouldn't show titles");
    return UT_FAILED;
   }

   if($object->config["showFields"])
   {
    $this->e->add("Filter shouldn't show fields");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Copy_Constructor()
  {
   $orig=new Filter();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Filter object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->cms->vars=Array("test");
   if($orig->cms->vars==$copy->cms->vars)
   {
    $this->e->add("Filter object copy also modified original \"cms->vars\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("Filter object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   $copy->l->sets=Array("test");
   if($orig->l->sets==$copy->l->sets)
   {
    $this->e->add("Filter object copy also modified original \"Language\" object", "error");
    return UT_FAILED;
   }

   $orig->l->sets=Array("test");
   if($orig->l->sets!=$orig->e->l->sets)
   {
    $this->e->add("Filter object original didn't modify original \"Language\" object", "error");
    return UT_FAILED;
   }

   $copy->l->sets=Array("test2");
   if($copy->l->sets!=$copy->e->l->sets)
   {
    $this->e->add("Filter object copy didn't modify copy \"Language\" object", "error");
    return UT_FAILED;
   }

   //error
   $orig=new Filter();
   $orig->e->add("test 1");
   $orig->e->add("test 2");
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Filter object copy doesn't match original", "error");
    return UT_FAILED;
   }

   //data / form
   $orig=new Filter();
   $orig->setDataForm(new Data(), new Form());
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Filter object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->cms->data->format=Array();
   if($orig->cms->data->format!=$copy->cms->data->format)
   {
    $this->e->add("Filter object copy didn't modified referenced \"Data\" variable", "error");
    return UT_FAILED;
   }

   $copy->cms->form->form=-1;
   if($orig->cms->form->form!=$copy->cms->form->form)
   {
    $this->e->add("Filter object copy didn't modified referenced \"Form\" variable", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

 }

?>
