<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/fs.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."fs.php");

 class FS_EXTENDS_TEST extends FS
 {
 }

 class UT_FS extends UT_Classes
 {
  function _init()
  {
   global $_path;

   $this->testfile=$_path["plugins"]."unit_testing/misc/fs.tst";
   $this->testdir=$_path["plugins"]."unit_testing/misc/fsdir/";

   $this->uid=posix_geteuid()+1;
   $this->gid=posix_getegid()+1;

   $this->difffile1=$_path["plugins"]."unit_testing/misc/fs_diff1.tst";
   $this->difffile2=$_path["plugins"]."unit_testing/misc/fs_diff2.tst";

   $this->diff1=Array(
    3 => "!In nibh sodales at, sollicitudin ac tellus.",
    8 => "-",
    9 => "-",
    14 => "-",
    15 => "-",
   );

   $this->diff2=Array(
    3 => "!In nibh erat, pellentesque at sodales at, sollicitudin ac tellus.",
    8 => "+Suspendisse potenti.",
    9 => "+Vivamus turpis ut interdum.",
    14 => "+Vivamus turpis ut interdum.",
    15 => "+In fringilla massa turpis fermentum odio, nec bibendum elit neque et lectus.",
   );

  }


  function Constructor()
  {
   $object=new FS();

   if(!is_object($object)||get_class($object)!="FS")
   {
    $this->e->add("Constructor doesn't return a FS object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new FS();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("FS object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->phpversion=-1;
   if($orig->phpversion==$copy->phpversion)
   {
    $this->e->add("FS object copy also modified original \"phpversion\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("FS object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   $copy->cli->ssh=Array();
   if($orig->cli->ssh==$copy->cli->ssh)
   {
    $this->e->add("FS object copy also modified original \"CLI\" object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Which()
  {
   $this->_init();

   $fs=new FS();

   if(!($bash=$fs->which("bash")))
   {
    $this->e->add("Which should find executable: \"bash\" in path", "error");
    return UT_FAILED;
   }
   if(exec("which bash")!=$bash)
   {
    $this->e->add("Which output doesn't match for executable: \"bash\"", "error");
    return UT_FAILED;
   }

   if(!($gcc=$fs->which("gcc")))
   {
    $this->e->add("Which should find executable: \"gcc\" in path", "error");
    return UT_FAILED;
   }
   if(exec("which gcc")!=$gcc)
   {
    $this->e->add("Which output doesn't match for executable: \"gcc\"", "error");
    return UT_FAILED;
   }

   if(($nonexistingexecutable=$fs->which("nonexistingexecutable")))
   {
    $this->e->add("Which should not find executable: \"nonexistingexecutable\" in path", "error");
    return UT_FAILED;
   }
   if(exec("which nonexistingexecutable")!=$nonexistingexecutable)
   {
    $this->e->add("Which output should be empty for executable: \"nonexistingexecutable\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function InDir()
  {
   global $_path;
   $this->_init();

   $fs=new FS();

   if(!$fs->inDir($_path["plugins"]."unit_testing/", $this->testfile))
   {
    $this->e->add("File: \"".$this->testfile."\" should be in directory: \"".$_path["plugins"]."unit_testing/\"", "error");
    return UT_FAILED;
   }

   if($fs->inDir($_path["plugins"]."unit_testing/somedir/", $this->testfile))
   {
    $this->e->add("File: \"".$this->testfile."\" shouldn't be in directory: \"".$_path["plugins"]."unit_testing/somedir/\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function RealPath()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $fs=new FS();

   $real=realpath($this->testfile); //the file should exist for the php function

   if(($realpath=$fs->realPath($this->testfile))!=$real)
   {
    $this->e->add("Real file path: \"".$real."\" doesn't match: \"".$realpath."\"", "error");
    return UT_FAILED;
   }

   @unlink($this->testfile);

   if(($realpath=$fs->realPath($this->testfile))!=$real)
   {
    $this->e->add("Real file path: \"".$real."\" doesn't match: \"".$realpath."\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Filename()
  {
   $fs=new FS();

   if($fs->filename("include/plugins/unit_testing/misc/fs.tst")!="fs.tst")
   {
    $this->e->add("Filename doesn't match expected output", "error");
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if($fs->filename("fs.tst")!="fs.tst")
   {
    $this->e->add("Filename doesn't match expected output", "error");
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Filepath()
  {
   $fs=new FS();

   if($fs->filepath("include/plugins/unit_testing/misc/fs.tst")!="include/plugins/unit_testing/misc/")
   {
    $this->e->add("Filepath doesn't match expected output", "error");
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if($fs->filepath("fs.tst")!="")
   {
    $this->e->add("Filepath doesn't match expected output", "error");
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Fileext()
  {
   $fs=new FS();

   if($fs->fileext("include/plugins/unit_testing/misc/fs.tst")!="tst")
   {
    $this->e->add("Fileext doesn't match expected output", "error");
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if($fs->fileext("fs.tst")!="tst")
   {
    $this->e->add("Fileext doesn't match expected output", "error");
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if($fs->fileext("include/../plugins/unit_testing/misc/fs.tst")!="tst")
   {
    $this->e->add("Fileext doesn't match expected output", "error");
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if($fs->fileext("include/../plugins/unit_testing/misc/fs")!="")
   {
    $this->e->add("Fileext doesn't match expected output", "error");
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if($fs->fileext("fs")!="")
   {
    $this->e->add("Fileext doesn't match expected output", "error");
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function TMPFileName()
  {
   $fs=new FS();

   if(!preg_match("!^/tmp/JAS_[A-Z0-9]+!i", $fs->tmpfilename()))
   {
    $this->e->add("TMPFileName doesn't match expected output", "error");
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if(!preg_match("!^/tmp/JAS_asd_[A-Z0-9]+!i", $fs->tmpfilename("asd")))
   {
    $this->e->add("TMPFileName doesn't match expected output", "error");
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if(!preg_match("!^/tmp/JAS_asd2_[A-Z0-9]+!i", $fs->tmpfilename("asd2_")))
   {
    $this->e->add("TMPFileName doesn't match expected output", "error");
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Files()
  {
   global $_path;

   $fs=new FS();

   if(($files=$fs->files($_path["root"]."include/"))===false)
   {
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if(!in_array($_path["root"]."include/fs.php", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!in_array($_path["root"]."include/format/", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function FilesRecursive()
  {
   global $_path;

   $fs=new FS();

   if(($files=$fs->files($_path["root"]."include/", true))===false)
   {
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if(!in_array($_path["root"]."include/fs.php", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!in_array($_path["root"]."include/language/nl/fs.xml", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function FilesStripRoot()
  {
   global $_path;

   $fs=new FS();

   if(($files=$fs->files($_path["root"]."include/", true, true))===false)
   {
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if(!in_array("fs.php", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!in_array("language/nl/fs.xml", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function FilesGetDotFiles()
  {
   global $_path;

   $fs=new FS();

   if(($files=$fs->files($_path["root"]."include/", false, true, true))===false)
   {
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if(!in_array("fs.php", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!in_array(".htaccess", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }


   if(($files=$fs->files($_path["root"]."include/", true, true, true))===false)
   {
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if(!in_array("fs.php", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!in_array(".htaccess", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!in_array("plugins/unit_testing/images/.htaccess", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function FilesIncludeFilter()
  {
   global $_path;

   $fs=new FS();

   if(($files=$fs->files($_path["include"], false, true, true, "^[^/]+\.php$"))===false)
   {
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if(!in_array("fs.php", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(in_array(".htaccess", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(in_array("mime.xml", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }


   if(($files=$fs->files($_path["include"], true, false, false, "^(api|format)/.*\.xml$"))===false)
   {
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if(in_array($_path["include"]."fs.php", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!in_array($_path["include"]."api/xml/1.0/jas.xml", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(in_array($_path["include"]."format/xml.php", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!in_array($_path["include"]."format/rss/atom-1.0.xml", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(in_array($_path["include"]."protocol/http.php", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(in_array($_path["include"].".htaccess", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function FilesExcludeFilter()
  {
   global $_path;

   $fs=new FS();

   if(($files=$fs->files($_path["include"], false, true, true, false, "^[^/]+\.php$"))===false)
   {
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if(in_array("fs.php", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!in_array(".htaccess", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!in_array("mime.xml", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }


   if(($files=$fs->files($_path["include"], true, false, false, "^(api|format)/.*\.xml$", "2\.0"))===false)
   {
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if(in_array($_path["include"]."fs.php", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!in_array($_path["include"]."api/xml/1.0/jas.xml", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(in_array($_path["include"]."format/xml.php", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!in_array($_path["include"]."format/rss/atom-1.0.xml", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(in_array($_path["include"]."format/rss/rss-2.0.xml", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(in_array($_path["include"]."protocol/http.php", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(in_array($_path["include"].".htaccess", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Directories()
  {
   global $_path;

   $fs=new FS();

   if(($directories=$fs->directories($_path["root"]."include/", true, false, false))===false)
   {
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if(!in_array($_path["root"]."include/plugins/", $directories))
   {
    $this->e->add("Directories doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!in_array($_path["root"]."include/language/nl/", $directories))
   {
    $this->e->add("Directories doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function DirectoriesStripRoot()
  {
   global $_path;

   $fs=new FS();

   if(($directories=$fs->directories($_path["root"]."include/", true, true, false))===false)
   {
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if(!in_array("plugins/", $directories))
   {
    $this->e->add("Directories doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!in_array("language/nl/", $directories))
   {
    $this->e->add("Directories doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function DirectoriesGetDotFiles()
  {
   global $_path;

   $fs=new FS();

   if(($directories=$fs->directories($_path["root"]."include/", true, true, true))===false)
   {
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if(!in_array("plugins/", $directories))
   {
    $this->e->add("Directories doesn't match expected output", "error");
    return UT_FAILED;
   }

   $plugdir=$_path["plugins"]."unit_testing/";
   $miscdir=$plugdir."misc/";
   $testdir=$miscdir.".fstest/";
   if(!@mkdir($testdir))
   {
    $this->e->add("Unable to create test directory: \"".$testdir."\"", "error");
    @rmdir($testdir);
    return UT_FAILED;
   }

   if(($directories=$fs->directories($miscdir, true, true, true))===false)
   {
    $this->e->add($fs->e);
    @rmdir($testdir);
    return UT_FAILED;
   }

   if(!in_array(".fstest/", $directories))
   {
    $this->e->add("Files doesn't match expected output", "error");
    @rmdir($testdir);
    return UT_FAILED;
   }

   if(($directories=$fs->directories($plugdir, true, true, true))===false)
   {
    $this->e->add($fs->e);
    @rmdir($testdir);
    return UT_FAILED;
   }

   if(!in_array("misc/.fstest/", $directories))
   {
    $this->e->add("Files doesn't match expected output", "error");
    @rmdir($testdir);
    return UT_FAILED;
   }

   @rmdir($testdir);
   return UT_OK;
  }

  function DirectoriesIncludeFilter()
  {
   global $_path;

   $fs=new FS();

   if(($directories=$fs->directories($_path["include"], false, true, false, "^api/$"))===false)
   {
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if(in_array("storage/", $directories))
   {
    $this->e->add("Directories doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!in_array("api/", $directories))
   {
    $this->e->add("Directories doesn't match expected output", "error");
    return UT_FAILED;
   }


   if(($directories=$fs->directories($_path["include"], true, false, true, "^(format|protocol)/"))===false)
   {
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if(in_array($_path["include"]."storage/", $directories))
   {
    $this->e->add("Directories doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!in_array($_path["include"]."format/", $directories))
   {
    $this->e->add("Directories doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!in_array($_path["include"]."format/rss/", $directories))
   {
    $this->e->add("Directories doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!in_array($_path["include"]."protocol/", $directories))
   {
    $this->e->add("Directories doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(in_array($_path["include"]."plugins/pager/", $directories))
   {
    $this->e->add("Directories doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function DirectoriesExcludeFilter()
  {
   global $_path;

   $fs=new FS();

   if(($directories=$fs->directories($_path["include"], false, true, false, false, "^api/$"))===false)
   {
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if(!in_array("storage/", $directories))
   {
    $this->e->add("Directories doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(in_array("api/", $directories))
   {
    $this->e->add("Directories doesn't match expected output", "error");
    return UT_FAILED;
   }


   if(($directories=$fs->directories($_path["include"], true, false, true, "^(api|format)/", "1\.0"))===false)
   {
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if(in_array($_path["include"]."storage/", $directories))
   {
    $this->e->add("Directories doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!in_array($_path["include"]."api/", $directories))
   {
    $this->e->add("Directories doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!in_array($_path["include"]."api/xml/", $directories))
   {
    $this->e->add("Directories doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(in_array($_path["include"]."api/xml/1.0/", $directories))
   {
    $this->e->add("Directories doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!in_array($_path["include"]."format/", $directories))
   {
    $this->e->add("Directories doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!in_array($_path["include"]."format/rss/", $directories))
   {
    $this->e->add("Directories doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(in_array($_path["include"]."plugins/pager/", $directories))
   {
    $this->e->add("Directories doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Size()
  {
   $this->_init();

   $fs=new FS();

   if($fs->write($this->testfile, "12345")===false)
   {
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if(($size=$fs->size($this->testfile))===false)
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   if($size!=5)
   {
    $this->e->add("File size doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   $fs->delete($this->testfile);


   if($fs->size($this->testfile)!==false)
   {
    $this->e->add("File size should fail on non-existing file: \"".$this->testfile."\"", "error");
    $fs->delete($this->testdir);
    return UT_FAILED;
   }


   if(($files=$fs->mkdir($this->testdir))===false)
   {
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if($fs->size($this->testdir)!==false)
   {
    $this->e->add("File size should fail on directory: \"".$this->testdir."\"", "error");
    $fs->delete($this->testdir);
    return UT_FAILED;
   }

   $fs->delete($this->testdir);
   return UT_OK;
  }


  function Exists()
  {
   $this->_init();

   $fs=new FS();

   if($fs->exists($this->testfile)!==false)
   {
    $this->e->add("File: \"".$this->testfile."\" should not exist", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   if($fs->write($this->testfile, "")===false)
   {
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if($fs->exists($this->testfile)===false)
   {
    $this->e->add("File: \"".$this->testfile."\" should exist", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   if($fs->exists($this->testfile, "file")===false)
   {
    $this->e->add("File: \"".$this->testfile."\" should exist", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   if($fs->exists($this->testfile, "directory")!==false)
   {
    $this->e->add("File: \"".$this->testfile."\" should not exist as directory", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   $fs->delete($this->testfile);


   if($fs->exists($this->testdir)!==false)
   {
    $this->e->add("Directory: \"".$this->testdir."\" should not exist", "error");
    $fs->delete($this->testdir);
    return UT_FAILED;
   }

   if(($files=$fs->mkdir($this->testdir))===false)
   {
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if($fs->exists($this->testdir)===false)
   {
    $this->e->add("Directory: \"".$this->testdir."\" should exist", "error");
    $fs->delete($this->testdir);
    return UT_FAILED;
   }

   if($fs->exists($this->testdir, "directory")===false)
   {
    $this->e->add("Directory: \"".$this->testdir."\" should exist", "error");
    $fs->delete($this->testdir);
    return UT_FAILED;
   }

   if($fs->exists($this->testdir, "file")!==false)
   {
    $this->e->add("Directory: \"".$this->testdir."\" should not exist as file", "error");
    $fs->delete($this->testdir);
    return UT_FAILED;
   }

   $fs->delete($this->testdir);
   return UT_OK;
  }


  function Mkdir()
  {
   $this->_init();

   $fs=new FS();

   if(!$fs->mkdir($this->testdir))
   {
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if(!file_exists($this->testdir))
   {
    $this->e->add("Test directory: \"".$this->testdir."\" doesn't exist", "error");
    return UT_FAILED;
   }

   if(!is_dir($this->testdir))
   {
    $this->e->add("Test directory: \"".$this->testdir."\" isn't a directory", "error");
    @unlink($this->testdir);
    return UT_FAILED;
   }

   if(decoct(fileperms($this->testdir))!=40755)
   {
    $this->e->add("Test directory: \"".$this->testdir."\" file permissions incorrect", "error");
    @rmdir($this->testdir);
    return UT_FAILED;
   }

   @rmdir($this->testdir);

   //recursive
   $testdir=$this->testdir."sub/";
   if(!$fs->mkdir($testdir, true))
   {
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if(!file_exists($testdir))
   {
    $this->e->add("Test directory: \"".$testdir."\" doesn't exist", "error");
    return UT_FAILED;
   }

   if(!is_dir($testdir))
   {
    $this->e->add("Test directory: \"".$testdir."\" isn't a directory", "error");
    @unlink($testdir);
    @rmdir($this->testdir);
    return UT_FAILED;
   }

   if(decoct(fileperms($this->testdir))!=40755)
   {
    $this->e->add("Test directory: \"".$this->testdir."\" file permissions incorrect", "error");
    @rmdir($testdir);
    @rmdir($this->testdir);
    return UT_FAILED;
   }

   @rmdir($testdir);
   @rmdir($this->testdir);

   //non recursive with multiple directories
   if($fs->mkdir($testdir))
   {
    $this->e->add("Test directory: \"".$testdir."\" shouldn't be created", "error");
    @rmdir($testdir);
    @rmdir($this->testdir);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Delete()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $fs=new FS();

   if(!$fs->delete($this->testfile))
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(file_exists($this->testfile))
   {
    $this->e->add("Test file: \"".$this->testfile."\" should have been deleted", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }


   @rmdir($this->testdir);
   if(!@mkdir($this->testdir))
   {
    $this->e->add("Unable to create test directory: \"".$this->testdir."\"", "error");
    return UT_FAILED;
   }

   if(!$fs->delete($this->testdir))
   {
    $this->e->add($fs->e);
    @rmdir($this->testdir);
    return UT_FAILED;
   }

   if(file_exists($this->testdir))
   {
    $this->e->add("Test directory: \"".$this->testdir."\" should have been deleted", "error");
    @rmdir($this->testdir);
    return UT_FAILED;
   }


   if(!@mkdir($this->testdir))
   {
    $this->e->add("Unable to create test directory: \"".$this->testdir."\"", "error");
    return UT_FAILED;
   }

   if(!@touch($this->testdir."asd"))
   {
    $this->e->add("Unable to touch test file: \"".$this->testdir."asd\"", "error");
    @rmdir($this->testdir);
    return UT_FAILED;
   }

   if($fs->delete($this->testdir))
   {
    $this->e->add("Non-empty directory: \"".$this->testdir."\" should not have been deleted", "error");
    @unlink($this->testdir."asd");
    @rmdir($this->testdir);
    return UT_FAILED;
   }
   $fs->e->clear();

   if(!$fs->delete($this->testdir, true))
   {
    $this->e->add($fs->e);
    @unlink($this->testdir."asd");
    @rmdir($this->testdir);
    return UT_FAILED;
   }

   if(file_exists($this->testdir))
   {
    $this->e->add("Test directory: \"".$this->testdir."\" should have been deleted", "error");
    @unlink($this->testdir."asd");
    @rmdir($this->testfile);
    return UT_FAILED;
   }


   if(!@mkdir($this->testdir))
   {
    $this->e->add("Unable to create test directory: \"".$this->testdir."\"", "error");
    return UT_FAILED;
   }

   if(!@mkdir($this->testdir."asd/"))
   {
    $this->e->add("Unable to create test directory: \"".$this->testdir."asd/\"", "error");
    return UT_FAILED;
   }

   if(!@touch($this->testdir."asd/asd"))
   {
    $this->e->add("Unable to touch test file: \"".$this->testdir."asd/asd\"", "error");
    @rmdir($this->testdir);
    return UT_FAILED;
   }

   if($fs->delete($this->testdir))
   {
    $this->e->add("Non-empty directory: \"".$this->testdir."\" should not have been deleted", "error");
    @unlink($this->testdir."asd/asd");
    @unlink($this->testdir."asd/");
    @rmdir($this->testdir);
    return UT_FAILED;
   }
   $fs->e->clear();

   if(!$fs->delete($this->testdir, true))
   {
    $this->e->add($fs->e);
    @unlink($this->testdir."asd/asd");
    @unlink($this->testdir."asd/");
    @rmdir($this->testdir);
    return UT_FAILED;
   }

   if(file_exists($this->testdir))
   {
    $this->e->add("Test directory: \"".$this->testdir."\" should have been deleted", "error");
    @unlink($this->testdir."asd/asd");
    @unlink($this->testdir."asd/");
    @rmdir($this->testdir);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Read()
  {
   $this->_init();

   $testdata="testdata";
   if(!@file_put_contents($this->testfile, $testdata))
   {
    $this->e->add("Unable to create test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $fs=new FS();

   if(($data=$fs->read($this->testfile))===false)
   {
    $this->e->add("Unable to read data from test file: \"".$this->testfile."\"", "error");
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($data!=$testdata)
   {
    $this->e->add("Data from test file: \"".$this->testfile."\" doesn't match input", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }

  function ReadLines()
  {
   $this->_init();

   $testdata="test
data";
   if(!@file_put_contents($this->testfile, $testdata))
   {
    $this->e->add("Unable to create test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $fs=new FS();

   if(($data=$fs->readLines($this->testfile))===false)
   {
    $this->e->add("Unable to read data from test file: \"".$this->testfile."\"", "error");
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($data!=explode("\n", $testdata))
   {
    $this->e->add("Data from test file: \"".$this->testfile."\" doesn't match input", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function Grep()
  {
   $this->_init();

   $testdata="testdata";
   if(!@file_put_contents($this->testfile, $testdata))
   {
    $this->e->add("Unable to create test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $fs=new FS();

   if(!$fs->grep("/test/", $this->testfile))
   {
    $this->e->add("Grep failed for test file: \"".$this->testfile."\"", "error");
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($fs->grep("/asdasd/", $this->testfile))
   {
    $this->e->add("Grep should fail for test file: \"".$this->testfile."\"", "error");
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(!($m=$fs->grep("/(test)(\w+)/", $this->testfile)))
   {
    $this->e->add("Grep failed for test file: \"".$this->testfile."\"", "error");
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("test"), Array("data")), $m))
   {
    $this->e->add("Grep matches for test file: \"".$this->testfile."\" doesn't match expected output", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }

  function Replace()
  {
   $this->_init();

   $testdata="testdata";
   if(!@file_put_contents($this->testfile, $testdata))
   {
    $this->e->add("Unable to create test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $fs=new FS();

   if(!$fs->replace("/test/", "asd", $this->testfile))
   {
    $this->e->add("Replace failed for test file: \"".$this->testfile."\"", "error");
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($fs->read($this->testfile)!="asddata")
   {
    $this->e->add("Replace doesn't match expected output for test file: \"".$this->testfile."\"", "error");
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(!$fs->replace("/(asd)(\w+)/", "\\1\\1", $this->testfile))
   {
    $this->e->add("Replace failed for test file: \"".$this->testfile."\"", "error");
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($fs->read($this->testfile)!="asdasd")
   {
    $this->e->add("Replace doesn't match expected output for test file: \"".$this->testfile."\"", "error");
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($fs->replace("/asdasdasd/", "data", $this->testfile))
   {
    $this->e->add("Replace should fail for test file: \"".$this->testfile."\"", "error");
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }


   @unlink($this->testfile);
   return UT_OK;
  }


  function Write()
  {
   $this->_init();

   $testdata="testdata";

   $fs=new FS();

   if($fs->write($this->testfile, $testdata)===false)
   {
    $this->e->add("Unable to write data to test file: \"".$this->testfile."\"", "error");
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if(@file_get_contents($this->testfile)!=$testdata)
   {
    $this->e->add("Data in test file: \"".$this->testfile."\" doesn't match input", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($fs->write($this->testfile, $testdata)!==false)
   {
    $this->e->add("Shouldn't be able to write data to existing test file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($fs->write($this->testfile, $testdata, "overwrite")===false)
   {
    $this->e->add("Unable to overwrite data in test file: \"".$this->testfile."\"", "error");
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(@file_get_contents($this->testfile)!=$testdata)
   {
    $this->e->add("Data in test file: \"".$this->testfile."\" doesn't match input", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($fs->write($this->testfile, $testdata, "append")===false)
   {
    $this->e->add("Unable to append data in test file: \"".$this->testfile."\"", "error");
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(@file_get_contents($this->testfile)!=$testdata.$testdata)
   {
    $this->e->add("Data in test file: \"".$this->testfile."\" doesn't match input", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);

   if($fs->write($this->testfile, $testdata, false, true)===false)
   {
    $this->e->add("Unable to lock file: \"".$this->testfile."\" for writing", "error");
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if(@file_get_contents($this->testfile)!=$testdata)
   {
    $this->e->add("Data in test file: \"".$this->testfile."\" doesn't match input", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function Copy()
  {
   global $_path;
   $this->_init();

   $testfile=$_path["include"]."fs.php";

   $fs=new FS();

   if(!$fs->mkdir($this->testdir))
   {
    $this->e->add("Unable to create test dir: \"".$this->testdir."\"", "error");
    $this->e->add($fs->e);
    $fs->delete($this->testdir, true);
    return UT_SKIPPED;
   }

   if(!$fs->copy($testfile, $this->testdir))
   {
    $this->e->add("Unable to copy test file: \"".$testfile."\" to: \"".$this->testdir."\"", "error");
    $this->e->add($fs->e);
    $fs->delete($this->testdir, true);
    return UT_FAILED;
   }

   if(!$fs->exists($testfile))
   {
    $this->e->add("Copy test failed, file: \"".$testfile."\" doesn't exist", "error");
    $fs->delete($this->testdir, true);
    return UT_FAILED;
   }

   if(!$fs->exists($this->testdir.preg_replace("!^.*/([^/]+)$!", "\\1", $testfile)))
   {
    $this->e->add("Copy test failed, file: \"".$this->testdir.preg_replace("!^.*/([^/]+)$!", "\\1", $testfile)."\" doesn't exist", "error");
    $fs->delete($this->testdir, true);
    return UT_FAILED;
   }

   if(!$fs->copy($testfile, $this->testdir."lala.php"))
   {
    $this->e->add("Unable to copy test file: \"".$testfile."\" to: \"".$this->testdir."lala.php\"", "error");
    $this->e->add($fs->e);
    $fs->delete($this->testdir, true);
    return UT_FAILED;
   }

   if(!$fs->exists($testfile))
   {
    $this->e->add("Copy test failed, file: \"".$testfile."\" doesn't exist", "error");
    $fs->delete($this->testdir, true);
    return UT_FAILED;
   }

   if(!$fs->exists($this->testdir."lala.php"))
   {
    $this->e->add("Copy test failed, file: \"".$this->testdir."lala.php"."\" doesn't exist", "error");
    $fs->delete($this->testdir, true);
    return UT_FAILED;
   }

   if($fs->copy("nonexistingfile", $this->testdir."lala.php"))
   {
    $this->e->add("Should not be able to copy test file: \"nonexistingfile\" to: \"".$this->testdir."lala.php\"", "error");
    $fs->delete($this->testdir, true);
    return UT_FAILED;
   }

   $fs->delete($this->testdir, true);
   return UT_OK;
  }


  function Rename()
  {
   global $_path;
   $this->_init();

   $testfile=$_path["include"]."fs.php";

   $fs=new FS();

   if(!$fs->mkdir($this->testdir))
   {
    $this->e->add("Unable to create test dir: \"".$this->testdir."\"", "error");
    $this->e->add($fs->e);
    $fs->delete($this->testdir, true);
    return UT_SKIPPED;
   }

   if(!$fs->copy($testfile, $this->testdir))
   {
    $this->e->add("Unable to copy test file: \"".$testfile."\" to: \"".$this->testdir."\"", "error");
    $this->e->add($fs->e);
    $fs->delete($this->testdir, true);
    return UT_FAILED;
   }

   if(!$fs->rename($this->testdir.preg_replace("!^.*/([^/]+)$!", "\\1", $testfile), $this->testdir."lala.php"))
   {
    $this->e->add("Unable to rename test file: \"".$this->testdir.preg_replace("!^.*/([^/]+)$!", "\\1", $testfile)."\" to: \"".$this->testdir."lala.php\"", "error");
    $this->e->add($fs->e);
    $fs->delete($this->testdir, true);
    return UT_FAILED;
   }

   if($fs->exists($this->testdir.preg_replace("!^.*/([^/]+)$!", "\\1", $testfile)))
   {
    $this->e->add("Rename test failed, file: \"".$this->testdir.preg_replace("!^.*/([^/]+)$!", "\\1", $testfile)."\" still exists", "error");
    $fs->delete($this->testdir, true);
    return UT_FAILED;
   }

   if(!$fs->exists($this->testdir."lala.php"))
   {
    $this->e->add("Rename test failed, file: \"".$this->testdir."lala.php"."\" doesn't exist", "error");
    $fs->delete($this->testdir, true);
    return UT_FAILED;
   }

   if($fs->rename("nonexistingfile", $this->testdir."lala.php"))
   {
    $this->e->add("Should not be able to rename test file: \"nonexistingfile\" to: \"".$this->testdir."lala.php\"", "error");
    $fs->delete($this->testdir, true);
    return UT_FAILED;
   }

   $fs->delete($this->testdir, true);
   return UT_OK;
  }


  function FileModeConvert()
  {
   $fs=new FS();

   //both way conversions
   $tests=Array(
    Array("inputType" => "int", "outputType" => "int", "input" => "1", "output" => "1"),
    Array("inputType" => "int", "outputType" => "int", "input" => "2", "output" => "2"),
    Array("inputType" => "int", "outputType" => "int", "input" => "4", "output" => "4"),

    Array("inputType" => "int", "outputType" => "int", "input" => "123", "output" => "123"),
    Array("inputType" => "int", "outputType" => "int", "input" => "345", "output" => "345"),
    Array("inputType" => "int", "outputType" => "int", "input" => "567", "output" => "567"),

    Array("inputType" => "string", "outputType" => "int", "input" => "r", "input2" => "r--", "output" => "4"),
    Array("inputType" => "string", "outputType" => "int", "input" => "rw", "input2" => "rw-", "output" => "6"),
    Array("inputType" => "string", "outputType" => "int", "input" => "wx", "input2" => "-wx", "output" => "3"),
    Array("inputType" => "string", "outputType" => "int", "input" => "--x", "output" => "1"),
    Array("inputType" => "string", "outputType" => "int", "input" => "r-x", "output" => "5"),
    Array("inputType" => "string", "outputType" => "int", "input" => "rwx", "output" => "7"),

    Array("inputType" => "string", "outputType" => "int", "input" => "r--rw-rwx", "output" => "467"),
    Array("inputType" => "string", "outputType" => "int", "input" => "---r-x--x", "output" => "051"),
    Array("inputType" => "string", "outputType" => "int", "input" => "rwxrwxrwx", "output" => "777"),

    Array("inputType" => "array", "outputType" => "int", "input" => Array("read" => true, "write" => false, "execute" => true), "output" => "5"),
    Array("inputType" => "array", "outputType" => "int", "input" => Array("read" => false, "write" => false, "execute" => false), "output" => "0"),
    Array("inputType" => "array", "outputType" => "int", "input" => Array("read" => true, "write" => true, "execute" => true), "output" => "7"),

    Array("inputType" => "array", "outputType" => "int", "input" => Array(
     "user" => Array("read" => true, "write" => true, "execute" => true),
     "group" => Array("read" => false, "write" => true, "execute" => false),
     "other" => Array("read" => true, "write" => true, "execute" => false),
    ), "output" => "726"),
    Array("inputType" => "array", "outputType" => "int", "input" => Array(
     "user" => Array("read" => false, "write" => false, "execute" => true),
     "group" => Array("read" => true, "write" => false, "execute" => false),
     "other" => Array("read" => true, "write" => false, "execute" => true),
    ), "output" => "145"),
    Array("inputType" => "array", "outputType" => "int", "input" => Array(
     "user" => Array("read" => false, "write" => true, "execute" => true),
     "group" => Array("read" => false, "write" => false, "execute" => false),
     "other" => Array("read" => false, "write" => true, "execute" => true),
    ), "output" => "303"),
   );

   foreach($tests as $test)
   {
    if(($converted=$fs->fileModeConvert($test["input"], $test["outputType"]))===false)
    {
     $this->e->add($fs->e);
     return UT_FAILED;
    }

    if($converted!=$test["output"])
    {
     $this->e->add("File mode conversion failed".(!is_array($test["output"])?", expected: \"".$test["output"]."\"":"").(!is_array($converted)?", got: \"".$converted."\"":""), "error");
     return UT_FAILED;
    }

    //reverse the tests
    if(($converted=$fs->fileModeConvert($test["output"], $test["inputType"]))===false)
    {
     $this->e->add($fs->e);
     return UT_FAILED;
    }

    if(isset($test["input2"]))
    {
     if($converted!=$test["input2"])
     {
      $this->e->add("File mode conversion failed".(!is_array($test["input2"])?", expected: \"".$test["input2"]."\"":"").(!is_array($converted)?", got: \"".$converted."\"":""), "error");
      return UT_FAILED;
     }
    }
    else
    {
     if($converted!=$test["input"])
     {
      $this->e->add("File mode conversion failed".(!is_array($test["input"])?", expected: \"".$test["input"]."\"":"").(!is_array($converted)?", got: \"".$converted."\"":""), "error");
      return UT_FAILED;
     }
    }
   }

   //fail tests
   $tests=Array(
    Array("outputType" => "asd", "input" => "1"),
    Array("outputType" => "int", "input" => "rax"),
    Array("outputType" => "int", "input" => "fail"),
    Array("outputType" => "int", "input" => Array("fail" => false, "read" => true, "write" => true)),
    Array("outputType" => "int", "input" => Array(
     "user" => Array("read" => false, "write" => true, "execute" => true),
     "group" => Array("read" => false, "write" => false, "execute" => false),
     "asda" => Array("read" => false, "write" => true, "execute" => true),
    )),
    Array("outputType" => "int", "input" => Array(
     "user" => Array("read" => false, "write" => true, "execute" => true),
     "group" => Array("fail" => false, "write" => false, "execute" => false),
     "other" => Array("read" => false, "write" => true, "execute" => true),
    )),
   );

   foreach($tests as $test)
   {
    if(($converted=$fs->fileModeConvert($test["input"], $test["outputType"]))!==false)
    {
     $this->e->add("File mode conversion failure test didn't fail".(!is_array($converted)?", got: \"".$converted."\"":""), "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }


  function GetUser()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $fs=new FS();

   //test user id
   if(($user=$fs->getUser($this->testfile, true))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(($realUser=fileowner($this->testfile))!==$user)
   {
    $this->e->add("User id doesn't match real user id for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   //test user name
   if(($user=$fs->getUser($this->testfile, false))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   $realUser=posix_getpwuid($realUser);
   if($realUser["name"]!==$user)
   {
    $this->e->add("User name doesn't match real user name for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function GetGroup()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $fs=new FS();

   //test group id
   if(($group=$fs->getGroup($this->testfile, true))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(($realGroup=filegroup($this->testfile))!==$group)
   {
    $this->e->add("Group id doesn't match real group id for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   //test group name
   if(($group=$fs->getGroup($this->testfile, false))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   $realGroup=posix_getgrgid($realGroup);
   if($realGroup["name"]!==$group)
   {
    $this->e->add("Group name doesn't match real group name for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function SetUser()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $tmp=fileowner($this->testfile);
   if(!@chown($this->testfile, $this->uid))
   {
    $this->e->add("Chown not supported", "warning");
    @unlink($this->testfile);
    return UT_SKIPPED;
   }
   @chown($this->testfile, $tmp);
   unset($tmp);

   $fs=new FS();

   if($fs->setUser($this->testfile, $this->uid)===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($fs->getUser($this->testfile, true)!==$this->uid)
   {
    $this->e->add("New user id doesn't match real user id for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function SetGroup()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $tmp=filegroup($this->testfile);
   if(!@chgrp($this->testfile, $this->gid))
   {
    $this->e->add("Chgrp not supported", "warning");
    @unlink($this->testfile);
    return UT_SKIPPED;
   }
   @chgrp($this->testfile, $tmp);

   unset($tmp);

   $fs=new FS();

   if($fs->setGroup($this->testfile, $this->gid)===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($fs->getGroup($this->testfile, true)!==$this->gid)
   {
    $this->e->add("New group id doesn't match real group id for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function GetMode()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $fs=new FS();

   chmod($this->testfile, 0123);
   clearstatcache();

   if(($mode=$fs->getMode($this->testfile))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!=123)
   {
    $this->e->add("File mode should have been: \"123\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   chmod($this->testfile, 0456);
   clearstatcache();

   if(($mode=$fs->getMode($this->testfile, "string"))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!="r--r-xrw-")
   {
    $this->e->add("File mode should have been: \"r--r-xrw-\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   chmod($this->testfile, 0147);
   clearstatcache();

   if(($mode=$fs->getMode($this->testfile, "array"))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(!(
    !$mode["user"]["read"]&&
    !$mode["user"]["write"]&&
    $mode["user"]["execute"]&&
    $mode["group"]["read"]&&
    !$mode["group"]["write"]&&
    !$mode["group"]["execute"]&&
    $mode["other"]["read"]&&
    $mode["other"]["write"]&&
    $mode["other"]["execute"]
   ))
   {
    $this->e->add("File mode incorrect for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function GetUserMode()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $fs=new FS();

   chmod($this->testfile, 0765);
   clearstatcache();

   if(($mode=$fs->getUserMode($this->testfile))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!=7)
   {
    $this->e->add("File mode should have been: \"7\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   chmod($this->testfile, 0432);
   clearstatcache();

   if(($mode=$fs->getUserMode($this->testfile, "string"))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!="r--")
   {
    $this->e->add("File mode should have been: \"r--\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   chmod($this->testfile, 0371);
   clearstatcache();

   if(($mode=$fs->getUserMode($this->testfile, "array"))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(!(
    !$mode["read"]&&
    $mode["write"]&&
    $mode["execute"]
   ))
   {
    $this->e->add("File mode incorrect for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function GetGroupMode()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $fs=new FS();

   chmod($this->testfile, 0765);
   clearstatcache();

   if(($mode=$fs->getGroupMode($this->testfile))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!=6)
   {
    $this->e->add("File mode should have been: \"6\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   chmod($this->testfile, 0432);
   clearstatcache();

   if(($mode=$fs->getGroupMode($this->testfile, "string"))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!="-wx")
   {
    $this->e->add("File mode should have been: \"-wx\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   chmod($this->testfile, 0371);
   clearstatcache();

   if(($mode=$fs->getGroupMode($this->testfile, "array"))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(!(
    $mode["read"]&&
    $mode["write"]&&
    $mode["execute"]
   ))
   {
    $this->e->add("File mode incorrect for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function GetOtherMode()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $fs=new FS();

   chmod($this->testfile, 0760);
   clearstatcache();

   if(($mode=$fs->getOtherMode($this->testfile))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!=0)
   {
    $this->e->add("File mode should have been: \"0\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   chmod($this->testfile, 0432);
   clearstatcache();

   if(($mode=$fs->getOtherMode($this->testfile, "string"))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!="-w-")
   {
    $this->e->add("File mode should have been: \"-w-\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   chmod($this->testfile, 0371);
   clearstatcache();

   if(($mode=$fs->getOtherMode($this->testfile, "array"))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(!(
    !$mode["read"]&&
    !$mode["write"]&&
    $mode["execute"]
   ))
   {
    $this->e->add("File mode incorrect for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function SetMode()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $fs=new FS();

   if($fs->setMode($this->testfile, 2, 3, 5)===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }
   //set mode should also clear cache

   if(($mode=$fs->getMode($this->testfile))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!=235)
   {
    $this->e->add("File mode should have been: \"235\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($fs->setMode($this->testfile, "rw", "r", "x")===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(($mode=$fs->getMode($this->testfile))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!=641)
   {
    $this->e->add("File mode should have been: \"641\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($fs->setMode($this->testfile,
    Array("read" => false, "write" => true, "execute" => true),
    Array("read" => true, "write" => true, "execute" => true),
    Array("read" => false, "write" => false, "execute" => false)
    )===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(($mode=$fs->getMode($this->testfile))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!="370")
   {
    $this->e->add("File mode should have been: \"370\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($fs->setMode($this->testfile, 2)===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(($mode=$fs->getMode($this->testfile))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!=270)
   {
    $this->e->add("File mode should have been: \"270\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($fs->setMode($this->testfile, false, 2)===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(($mode=$fs->getMode($this->testfile))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!=220)
   {
    $this->e->add("File mode should have been: \"220\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($fs->setMode($this->testfile, false, false, 2)===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(($mode=$fs->getMode($this->testfile))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!=222)
   {
    $this->e->add("File mode should have been: \"222\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function SetUserMode()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $fs=new FS();

   if($fs->setMode($this->testfile, 0, 0, 0)===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($fs->setUserMode($this->testfile, 2)===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(($mode=$fs->getMode($this->testfile))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!=200)
   {
    $this->e->add("File mode should have been: \"200\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($fs->setUserMode($this->testfile, "rw")===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(($mode=$fs->getMode($this->testfile))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!=600)
   {
    $this->e->add("File mode should have been: \"600\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($fs->setUserMode($this->testfile,
    Array("read" => false, "write" => true, "execute" => true)
    )===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(($mode=$fs->getMode($this->testfile))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!="300")
   {
    $this->e->add("File mode should have been: \"300\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function SetGroupMode()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $fs=new FS();

   if($fs->setMode($this->testfile, 0, 0, 0)===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($fs->setGroupMode($this->testfile, 1)===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(($mode=$fs->getMode($this->testfile))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!="010")
   {
    $this->e->add("File mode should have been: \"010\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($fs->setGroupMode($this->testfile, "rx")===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(($mode=$fs->getMode($this->testfile))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!="050")
   {
    $this->e->add("File mode should have been: \"050\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($fs->setGroupMode($this->testfile,
    Array("read" => true, "write" => true, "execute" => true)
    )===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(($mode=$fs->getMode($this->testfile))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!="070")
   {
    $this->e->add("File mode should have been: \"070\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function SetOtherMode()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $fs=new FS();

   if($fs->setMode($this->testfile, 0, 0, 0)===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($fs->setOtherMode($this->testfile, 6)===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(($mode=$fs->getMode($this->testfile))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!="006")
   {
    $this->e->add("File mode should have been: \"006\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($fs->setOtherMode($this->testfile, "rx")===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(($mode=$fs->getMode($this->testfile))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!="005")
   {
    $this->e->add("File mode should have been: \"005\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($fs->setOtherMode($this->testfile,
    Array("read" => true, "write" => false, "execute" => false)
    )===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(($mode=$fs->getMode($this->testfile))===false)
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!="004")
   {
    $this->e->add("File mode should have been: \"004\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function GetTime()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $times=Array(
    "created" => @filectime($this->testfile),
    "modified" => @filemtime($this->testfile),
    "accessed" => @fileatime($this->testfile),
   );

   $fs=new FS();

   if($times!=$fs->getTime($this->testfile))
   {
    $this->e->add("Get time doesn't match expected output", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   foreach(array_keys($times) as $type)
   {
    if($times[$type]!=$fs->getTime($this->testfile, $type))
    {
     $this->e->add("Get ".$type." time doesn't match expected output", "error");
     @unlink($this->testfile);
     return UT_FAILED;
    }
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function SetTime()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $fs=new FS();

   if(!$fs->setTime($this->testfile, 2, "modified"))
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($fs->getTime($this->testfile, "modified")!=2)
   {
    $this->e->add("Set time didn't change modified time", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }
   if($fs->getTime($this->testfile, "created")==2)
   {
    $this->e->add("Set time shouldn't have changed created time", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }
   if($fs->getTime($this->testfile, "accessed")==2)
   {
    $this->e->add("Set time shouldn't have changed accessed time", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(!$fs->setTime($this->testfile, 3, "accessed"))
   {
    $this->e->add($fs->e);
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($fs->getTime($this->testfile, "accessed")!=3)
   {
    $this->e->add("Set time didn't change accessed time", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }
   if($fs->getTime($this->testfile, "created")==3)
   {
    $this->e->add("Set time shouldn't have changed created time", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }
   if($fs->getTime($this->testfile, "modified")==3)
   {
    $this->e->add("Set time shouldn't have changed modified time", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($fs->setTime($this->testfile, 3, "created"))
   {
    $this->e->add("Set time shouldn't be able to change created time", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function Diff()
  {
   $this->_init();

   $fs=new FS();

   if(($diff=$fs->diff($this->difffile1, $this->difffile2))===false)
   {
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if($diff!=$this->diff2)
   {
    $this->e->add("Diff doesn't match expected output", "error");
    return UT_FAILED;
   }

   //reversed diff
   if(($diff=$fs->diff($this->difffile2, $this->difffile1))===false)
   {
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if($diff!=$this->diff1)
   {
    $this->e->add("Diff doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function GetACL()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $acl=Array(
    "userName" => "apache",
    "groupName" => "apache",
    "user" => "rw-",
    "group" => "r--",
    "other" => "r--",
    "mask" => false,
    "users" => Array(),
    "groups" => Array(),
   );

   $fs=new FS();

   if(($output=$fs->getACL($this->testfile))===false)
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   if($output!=$acl)
   {
    $this->e->add("Get ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }


   $acl["users"]["nobody"]="r-x";
   $acl["mask"]="r-x";
   exec("setfacl --modify user:nobody:r-x ".$this->testfile, $stdout, $ret);

   if($ret!=0)
   {
    $this->e->add("Unable to run setfacl on: \"".$this->testfile."\", make sure ACL is enabled on this mount", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   if(($output=$fs->getACL($this->testfile))===false)
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   if($output!=$acl)
   {
    $this->e->add("Get ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }


   $acl["groups"]["nogroup"]="r--";
   $acl["mask"]="r-x";
   exec("setfacl --modify group:nogroup:r-- ".$this->testfile, $stdout, $ret);

   if($ret!=0)
   {
    $this->e->add("Unable to run setfacl on: \"".$this->testfile."\", make sure ACL is enabled on this mount", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   if(($output=$fs->getACL($this->testfile))===false)
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   if($output!=$acl)
   {
    $this->e->add("Get ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   $fs->delete($this->testfile);


   //int file modes
   $acl=Array(
    "userName" => "apache",
    "groupName" => "apache",
    "user" => "6",
    "group" => "4",
    "other" => "4",
    "mask" => "5",
    "users" => Array("nobody" => "5"),
    "groups" => Array("nogroup" => "4"),
   );

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   exec("setfacl --modify user:nobody:r-x ".$this->testfile, $stdout, $ret);
   if($ret!=0)
   {
    $this->e->add("Unable to run setfacl on: \"".$this->testfile."\", make sure ACL is enabled on this mount", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   exec("setfacl --modify group:nogroup:r-- ".$this->testfile, $stdout, $ret);
   if($ret!=0)
   {
    $this->e->add("Unable to run setfacl on: \"".$this->testfile."\", make sure ACL is enabled on this mount", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   if(($output=$fs->getACL($this->testfile, "int"))===false)
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   if($output!=$acl)
   {
    $this->e->add("Get ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   $fs->delete($this->testfile);


   //array file modes
   $acl=Array(
    "userName" => "apache",
    "groupName" => "apache",
    "user" => Array("read" => true, "write" => true, "execute" => false),
    "group" => Array("read" => true, "write" => false, "execute" => false),
    "other" => Array("read" => true, "write" => false, "execute" => false),
    "mask" => Array("read" => true, "write" => false, "execute" => true),
    "users" => Array("nobody" => Array("read" => true, "write" => false, "execute" => true)),
    "groups" => Array("nogroup" => Array("read" => true, "write" => false, "execute" => false)),
   );

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   exec("setfacl --modify user:nobody:r-x ".$this->testfile, $stdout, $ret);
   if($ret!=0)
   {
    $this->e->add("Unable to run setfacl on: \"".$this->testfile."\", make sure ACL is enabled on this mount", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   exec("setfacl --modify group:nogroup:r-- ".$this->testfile, $stdout, $ret);
   if($ret!=0)
   {
    $this->e->add("Unable to run setfacl on: \"".$this->testfile."\", make sure ACL is enabled on this mount", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   if(($output=$fs->getACL($this->testfile, "array"))===false)
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   if($output!=$acl)
   {
    $this->e->add("Get ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   $fs->delete($this->testfile);


   //multiple files
   $acl=Array(
    "userName" => "apache",
    "groupName" => "apache",
    "user" => "rw-",
    "group" => "rw-",
    "other" => "r--",
    "mask" => false,
    "users" => Array(),
    "groups" => Array(),
   );

   $acl2=Array(
    $this->difffile1 => $acl,
    $this->difffile2 => $acl,
   );

   if(($output=$fs->getACL(Array($this->difffile1, $this->difffile2)))===false)
   {
    $this->e->add($fs->e);
    return UT_FAILED;
   }

   if($output!=$acl2)
   {
    $this->e->add("Get ACL doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function GetACLUser()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $fs=new FS();

   if(($output=$fs->getACLUser($this->testfile, "nobody"))===false)
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl="---";
   if($output!=$acl)
   {
    $this->e->add("Get ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }


   exec("setfacl --modify user:nobody:r-x ".$this->testfile, $stdout, $ret);
   if($ret!=0)
   {
    $this->e->add("Unable to run setfacl on: \"".$this->testfile."\", make sure ACL is enabled on this mount", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   if(($output=$fs->getACLUser($this->testfile, "nobody"))===false)
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl="r-x";
   if($output!=$acl)
   {
    $this->e->add("Get ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   $fs->delete($this->testfile);


   //multiple files
   $acl=Array("read" => false, "write" => false, "execute" => false);

   $acl2=Array(
    $this->difffile1 => $acl,
    $this->difffile2 => $acl,
   );

   if(($output=$fs->getACLUser(Array($this->difffile1, $this->difffile2), "nobody", "array"))===false)
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   if($output!=$acl2)
   {
    $this->e->add("Get ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function GetACLGroup()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $fs=new FS();

   if(($output=$fs->getACLGroup($this->testfile, "nogroup"))===false)
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl="---";
   if($output!=$acl)
   {
    $this->e->add("Get ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }


   exec("setfacl --modify group:nogroup:r-x ".$this->testfile, $stdout, $ret);
   if($ret!=0)
   {
    $this->e->add("Unable to run setfacl on: \"".$this->testfile."\", make sure ACL is enabled on this mount", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   if(($output=$fs->getACLGroup($this->testfile, "nogroup"))===false)
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl="r-x";
   if($output!=$acl)
   {
    $this->e->add("Get ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   $fs->delete($this->testfile);


   //multiple files
   $acl="0";

   $acl2=Array(
    $this->difffile1 => $acl,
    $this->difffile2 => $acl,
   );

   if(($output=$fs->getACLGroup(Array($this->difffile1, $this->difffile2), "nogroup", "int"))===false)
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   if($output!=$acl2)
   {
    $this->e->add("Get ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function SetACL()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $fs=new FS();

   if(($output=$fs->getACL($this->testfile))===false)
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl=$output;

   //no changes test
   if(!$fs->setACL($this->testfile, $acl))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   if($fs->getACL($this->testfile)!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   //add nobody rights
   if(!$fs->setACL($this->testfile, Array("users" => Array("nobody" => "r-x"))))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl["users"]["nobody"]="r-x";
   $acl["mask"]="r-x";
   if($fs->getACL($this->testfile)!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   //change nobody rights
   if(!$fs->setACL($this->testfile, Array("users" => Array("nobody" => "r--"))))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl["users"]["nobody"]="r--";
   $acl["mask"]="r--";
   if($fs->getACL($this->testfile)!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }


   //add nogroup rights
   if(!$fs->setACL($this->testfile, Array("groups" => Array("nogroup" => "r-x"))))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl["groups"]["nogroup"]="r-x";
   $acl["mask"]="r-x";
   if($fs->getACL($this->testfile)!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   //change nogroup rights
   if(!$fs->setACL($this->testfile, Array("groups" => Array("nogroup" => "r--"))))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl["groups"]["nogroup"]="r--";
   $acl["mask"]="r--";
   if($fs->getACL($this->testfile)!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }


   //add apache rights
   if(!$fs->setACL($this->testfile, Array("users" => Array("apache" => "r-x"))))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl["users"]["apache"]="r-x";
   $acl["mask"]="r-x";
   if($fs->getACL($this->testfile)!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   //change apache rights
   if(!$fs->setACL($this->testfile, Array("users" => Array("apache" => "r--"))))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl["users"]["apache"]="r--";
   $acl["mask"]="r--";
   if($fs->getACL($this->testfile)!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }


   //add apache rights
   if(!$fs->setACL($this->testfile, Array("groups" => Array("apache" => "r-x"))))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl["groups"]["apache"]="r-x";
   $acl["mask"]="r-x";
   if($fs->getACL($this->testfile)!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   //change apache rights
   if(!$fs->setACL($this->testfile, Array("groups" => Array("apache" => "r--"))))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl["groups"]["apache"]="r--";
   $acl["mask"]="r--";
   if($fs->getACL($this->testfile)!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }


   //change user rights
   if(!$fs->setACL($this->testfile, Array("user" => "rwx")))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl["user"]="rwx";
   $acl["mask"]="r--";
   if($fs->getACL($this->testfile)!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }


   //change group rights
   if(!$fs->setACL($this->testfile, Array("group" => "r-x")))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl["group"]="r-x";
   $acl["mask"]="r-x";
   if($fs->getACL($this->testfile)!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }


   //change other rights
   if(!$fs->setACL($this->testfile, Array("other" => "--x")))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl["other"]="--x";
   $acl["mask"]="r-x";
   if($fs->getACL($this->testfile)!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }


   //remove other rights
   if(!$fs->setACL($this->testfile, Array("other" => false)))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl["other"]="---";
   $acl["mask"]="r-x";
   if($fs->getACL($this->testfile)!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }


   //remove nobody rights
   if(!$fs->setACL($this->testfile, Array("users" => Array("nobody" => "---"))))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   unset($acl["users"]["nobody"]);
   $acl["mask"]="r-x";
   if($fs->getACL($this->testfile)!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   //remove nogroup rights
   if(!$fs->setACL($this->testfile, Array("groups" => Array("nogroup" => "---"))))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   unset($acl["groups"]["nogroup"]);
   $acl["mask"]="r-x";
   if($fs->getACL($this->testfile)!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }


   //remove all acl rights
   if(!$fs->setACL($this->testfile, false))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   unset($acl["users"]["apache"]);
   unset($acl["groups"]["apache"]);
   $acl["mask"]=false;
   if($fs->getACL($this->testfile)!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   $fs->delete($this->testfile);


   //directory+recurse test
   if(!@mkdir($this->testdir))
   {
    $this->e->add("Unable to create test directory: \"".$this->testdir."\"", "error");
    return UT_FAILED;
   }
   if(!@touch($this->testdir."asd"))
   {
    $this->e->add("Unable to touch test file: \"".$this->testdir."asd\"", "error");
    return UT_FAILED;
   }

   if(($output=$fs->getACL($this->testdir))===false)
   {
    $this->e->add($fs->e);
    $fs->delete($this->testdir, true);
    return UT_FAILED;
   }
   $acl=$output;
   if(($output=$fs->getACL($this->testdir."asd"))===false)
   {
    $this->e->add($fs->e);
    $fs->delete($this->testdir, true);
    return UT_FAILED;
   }
   $acl2=$output;

   //no changes test
   if(!$fs->setACL($this->testdir, $acl, true))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testdir, true);
    return UT_FAILED;
   }
   if($fs->getACL($this->testdir)!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testdir, true);
    return UT_FAILED;
   }
   if($fs->getACL($this->testdir."asd")!=$acl2)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testdir, true);
    return UT_FAILED;
   }

   //add nobody rights
   if(!$fs->setACL($this->testdir, Array("users" => Array("nobody" => "r-x")), true))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testdir, true);
    return UT_FAILED;
   }
   $acl["users"]["nobody"]="r-x";
   $acl["mask"]="r-x";
   $acl2["users"]["nobody"]="r-x";
   $acl2["mask"]="r-x";
   if($fs->getACL($this->testdir)!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testdir, true);
    return UT_FAILED;
   }
   if($fs->getACL($this->testdir."asd")!=$acl2)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testdir, true);
    return UT_FAILED;
   }

   //change nobody rights
   if(!$fs->setACL($this->testdir, Array("users" => Array("nobody" => "r--")), true))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testdir, true);
    return UT_FAILED;
   }
   $acl["users"]["nobody"]="r--";
   $acl["mask"]="r-x";
   $acl2["users"]["nobody"]="r--";
   $acl2["mask"]="r--";
   if($fs->getACL($this->testdir)!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testdir, true);
    return UT_FAILED;
   }
   if($fs->getACL($this->testdir."asd")!=$acl2)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testdir, true);
    return UT_FAILED;
   }


   //remove rights
   if(!$fs->setACL($this->testdir, false, true))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testdir, true);
    return UT_FAILED;
   }
   unset($acl["users"]["nobody"]);
   unset($acl2["users"]["nobody"]);
   $acl["mask"]=false;
   $acl2["mask"]=false;
   if($fs->getACL($this->testdir)!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testdir);
    return UT_FAILED;
   }
   if($fs->getACL($this->testdir."asd")!=$acl2)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testdir, true);
    return UT_FAILED;
   }

   $fs->delete($this->testdir, true);
   return UT_OK;
  }


  function SetACLUser()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $fs=new FS();

   if(!$fs->setACLUser($this->testfile, "nobody", "r-x"))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl="r-x";
   if($fs->getACLUser($this->testfile, "nobody")!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }


   if(!$fs->setACLUser($this->testfile, "nobody", "4"))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl="r--";
   if($fs->getACLUser($this->testfile, "nobody")!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }


   if(!$fs->setACLUser($this->testfile, "apache", "5"))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl="r--";
   if($fs->getACLUser($this->testfile, "nobody")!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl="r-x";
   if($fs->getACLUser($this->testfile, "apache")!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }


   //remove rights
   if(!$fs->setACLUser($this->testfile, "nobody", false))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   if(!$fs->setACLUser($this->testfile, "apache", false))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl="---";
   if($fs->getACLUser($this->testfile, "nobody")!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl="---";
   if($fs->getACLUser($this->testfile, "apache")!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   $fs->delete($this->testfile);
   return UT_OK;
  }


  function SetACLGroup()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $fs=new FS();

   if(!$fs->setACLGroup($this->testfile, "nogroup", "r-x"))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl="r-x";
   if($fs->getACLGroup($this->testfile, "nogroup")!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }


   if(!$fs->setACLGroup($this->testfile, "nogroup", "4"))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl="r--";
   if($fs->getACLGroup($this->testfile, "nogroup")!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }


   if(!$fs->setACLGroup($this->testfile, "apache", Array("read" => true, "write" => false, "execute" => true)))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl="r--";
   if($fs->getACLGroup($this->testfile, "nogroup")!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl="r-x";
   if($fs->getACLGroup($this->testfile, "apache")!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }


   //remove rights
   if(!$fs->setACLGroup($this->testfile, "nogroup", false))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   if(!$fs->setACLGroup($this->testfile, "apache", false))
   {
    $this->e->add($fs->e);
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl="---";
   if($fs->getACLGroup($this->testfile, "nogroup")!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }
   $acl="---";
   if($fs->getACLGroup($this->testfile, "apache")!=$acl)
   {
    $this->e->add("Set ACL doesn't match expected output", "error");
    $fs->delete($this->testfile);
    return UT_FAILED;
   }

   $fs->delete($this->testfile);
   return UT_OK;
  }

 }

?>
