<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/fs.sro.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["plugins"]."unit_testing/ut/fs.php");
 require_once($_path["include"]."fs.php");

 class UT_FS_Scope_Resolution_Operator extends UT_FS
 {
  function CallStatic()
  {
   $this->_init();

   if(!FS::whichS("bash"))
   {
    $this->e->add("WhichS shouldn't fail", "error");
    return UT_FAILED;
   }

   /*
   if(($nonexistingexecutable=FS::nonexistingexecutableS()))
   {
    $this->e->add("nonexistingexecutableS should fail", "error");
    return UT_FAILED;
   }
   */

   return UT_OK;
  }


  function Which()
  {
   $this->_init();

   if(!($bash=FS::whichS("bash")))
   {
    $this->e->add("Which should find executable: \"bash\" in path", "error");
    return UT_FAILED;
   }
   if(exec("which bash")!=$bash)
   {
    $this->e->add("Which output doesn't match for executable: \"bash\"", "error");
    return UT_FAILED;
   }

   if(!($gcc=FS::whichS("gcc")))
   {
    $this->e->add("Which should find executable: \"gcc\" in path", "error");
    return UT_FAILED;
   }
   if(exec("which gcc")!=$gcc)
   {
    $this->e->add("Which output doesn't match for executable: \"gcc\"", "error");
    return UT_FAILED;
   }

   if(($nonexistingexecutable=FS::whichS("nonexistingexecutable")))
   {
    $this->e->add("Which should not find executable: \"nonexistingexecutable\" in path", "error");
    return UT_FAILED;
   }
   if(exec("which nonexistingexecutable")!=$nonexistingexecutable)
   {
    $this->e->add("Which output should be empty for executable: \"nonexistingexecutable\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function InDir()
  {
   global $_path;
   $this->_init();

   if(!FS::inDirS($_path["plugins"]."unit_testing/", $this->testfile))
   {
    $this->e->add("File: \"".$this->testfile."\" should be in directory: \"".$_path["plugins"]."unit_testing/\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function RealPath()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $real=realpath($this->testfile); //the file should exist for the php function

   if(($realpath=FS::realPathS($this->testfile))!=$real)
   {
    $this->e->add("Real file path: \"".$real."\" doesn't match: \"".$realpath."\"", "error");
    return UT_FAILED;
   }

   @unlink($this->testfile);

   if(($realpath=FS::realPathS($this->testfile))!=$real)
   {
    $this->e->add("Real file path: \"".$real."\" doesn't match: \"".$realpath."\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Filename()
  {
   if(FS::filenameS("include/plugins/unit_testing/misc/fs.tst")!="fs.tst")
   {
    $this->e->add("Filename doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(FS::filenameS("fs.tst")!="fs.tst")
   {
    $this->e->add("Filename doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Filepath()
  {
   if(FS::filepathS("include/plugins/unit_testing/misc/fs.tst")!="include/plugins/unit_testing/misc/")
   {
    $this->e->add("Filepath doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(FS::filepathS("fs.tst")!="")
   {
    $this->e->add("Filepath doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Fileext()
  {
   if(FS::fileextS("include/plugins/unit_testing/misc/fs.tst")!="tst")
   {
    $this->e->add("Fileext doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(FS::fileextS("fs.tst")!="tst")
   {
    $this->e->add("Fileext doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(FS::fileextS("include/../plugins/unit_testing/misc/fs.tst")!="tst")
   {
    $this->e->add("Fileext doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(FS::fileextS("include/../plugins/unit_testing/misc/fs")!="")
   {
    $this->e->add("Fileext doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(FS::fileextS("fs")!="")
   {
    $this->e->add("Fileext doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function TMPFileName()
  {
   if(!preg_match("!^/tmp/JAS_[A-Z0-9]+!i", FS::tmpfilenameS()))
   {
    $this->e->add("TMPFileName doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!preg_match("!^/tmp/JAS_asd_[A-Z0-9]+!i", FS::tmpfilenameS("asd")))
   {
    $this->e->add("TMPFileName doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!preg_match("!^/tmp/JAS_asd2_[A-Z0-9]+!i", FS::tmpfilenameS("asd2_")))
   {
    $this->e->add("TMPFileName doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Files()
  {
   global $_path;

   if(($files=FS::filesS($_path["root"]."include/"))===false)
   {
    $this->e->add("SRO files call failed", "error");
    return UT_FAILED;
   }

   if(!in_array($_path["root"]."include/fs.php", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!in_array($_path["root"]."include/format/", $files))
   {
    $this->e->add("Files doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Directories()
  {
   global $_path;

   if(($files=FS::directoriesS($_path["root"]."include/", true, false, false))===false)
   {
    $this->e->add("SRO Directories call failed", "error");
    return UT_FAILED;
   }

   if(!in_array($_path["root"]."include/plugins/", $files))
   {
    $this->e->add("Directories doesn't match expected output", "error");
    return UT_FAILED;
   }

   if(!in_array($_path["root"]."include/language/nl/", $files))
   {
    $this->e->add("Directories doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Size()
  {
   $this->_init();

   if(FS::writeS($this->testfile, "12345")===false)
   {
    $this->e->add("Unable to write test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   if(($size=FS::sizeS($this->testfile))===false)
   {
    $this->e->add("SRO size call failed", "error");
    FS::deleteS($this->testfile);
    return UT_FAILED;
   }

   if($size!=5)
   {
    $this->e->add("File size doesn't match expected output", "error");
    FS::deleteS($this->testfile);
    return UT_FAILED;
   }

   FS::deleteS($this->testfile);


   if(FS::sizeS($this->testfile)!==false)
   {
    $this->e->add("File size should fail on non-existing file: \"".$this->testfile."\"", "error");
    FS::deleteS($this->testdir);
    return UT_FAILED;
   }


   if(($files=FS::mkdirS($this->testdir))===false)
   {
    $this->e->add("Unable to create test directory: \"".$this->testdir."\"", "error");
    return UT_FAILED;
   }

   if(FS::sizeS($this->testdir)!==false)
   {
    $this->e->add("File size should fail on directory: \"".$this->testdir."\"", "error");
    FS::deleteS($this->testdir);
    return UT_FAILED;
   }

   FS::deleteS($this->testdir);
   return UT_OK;
  }


  function Exists()
  {
   $this->_init();

   if(FS::existsS($this->testfile)!==false)
   {
    $this->e->add("File: \"".$this->testfile."\" should not exist", "error");
    FS::deleteS($this->testfile);
    return UT_FAILED;
   }

   if(FS::writeS($this->testfile, "")===false)
   {
    $this->e->add("Unable to write test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   if(FS::existsS($this->testfile)===false)
   {
    $this->e->add("File: \"".$this->testfile."\" should exist", "error");
    FS::deleteS($this->testfile);
    return UT_FAILED;
   }

   if(FS::existsS($this->testfile, "file")===false)
   {
    $this->e->add("File: \"".$this->testfile."\" should exist", "error");
    FS::deleteS($this->testfile);
    return UT_FAILED;
   }

   if(FS::existsS($this->testfile, "directory")!==false)
   {
    $this->e->add("File: \"".$this->testfile."\" should not exist as directory", "error");
    FS::deleteS($this->testfile);
    return UT_FAILED;
   }

   FS::deleteS($this->testfile);


   if(FS::existsS($this->testdir)!==false)
   {
    $this->e->add("Directory: \"".$this->testdir."\" should not exist", "error");
    FS::deleteS($this->testdir);
    return UT_FAILED;
   }

   if(($files=FS::mkdirS($this->testdir))===false)
   {
    $this->e->add("Unable to create test directory: \"".$this->testdir."\"", "error");
    return UT_FAILED;
   }

   if(FS::existsS($this->testdir)===false)
   {
    $this->e->add("Directory: \"".$this->testdir."\" should exist", "error");
    FS::deleteS($this->testdir);
    return UT_FAILED;
   }

   if(FS::existsS($this->testdir, "directory")===false)
   {
    $this->e->add("Directory: \"".$this->testdir."\" should exist", "error");
    FS::deleteS($this->testdir);
    return UT_FAILED;
   }

   if(FS::existsS($this->testdir, "file")!==false)
   {
    $this->e->add("Directory: \"".$this->testdir."\" should not exist as file", "error");
    FS::deleteS($this->testdir);
    return UT_FAILED;
   }

   FS::deleteS($this->testdir);
   return UT_OK;
  }



  function Mkdir()
  {
   $this->_init();

   if(!FS::mkdirS($this->testdir))
   {
    $this->e->add("SRO mkdir call failed", "error");
    return UT_FAILED;
   }

   if(!file_exists($this->testdir))
   {
    $this->e->add("Test directory: \"".$this->testdir."\" doesn't exist", "error");
    return UT_FAILED;
   }

   if(!is_dir($this->testdir))
   {
    $this->e->add("Test directory: \"".$this->testdir."\" isn't a directory", "error");
    @unlink($this->testdir);
    return UT_FAILED;
   }

   if(decoct(fileperms($this->testdir))!=40755)
   {
    $this->e->add("Test directory: \"".$this->testdir."\" file permissions incorrect", "error");
    @rmdir($this->testdir);
    return UT_FAILED;
   }

   @rmdir($this->testdir);

   //recursive
   $testdir=$this->testdir."sub/";
   if(!FS::mkdirS($testdir, true))
   {
    $this->e->add("SRO mkdir call failed", "error");
    return UT_FAILED;
   }

   if(!file_exists($testdir))
   {
    $this->e->add("Test directory: \"".$testdir."\" doesn't exist", "error");
    return UT_FAILED;
   }

   if(!is_dir($testdir))
   {
    $this->e->add("Test directory: \"".$testdir."\" isn't a directory", "error");
    @unlink($testdir);
    @rmdir($this->testdir);
    return UT_FAILED;
   }

   if(decoct(fileperms($this->testdir))!=40755)
   {
    $this->e->add("Test directory: \"".$this->testdir."\" file permissions incorrect", "error");
    @rmdir($testdir);
    @rmdir($this->testdir);
    return UT_FAILED;
   }

   @rmdir($testdir);
   @rmdir($this->testdir);

   //non recursive with multiple directories
   if(FS::mkdirS($testdir))
   {
    $this->e->add("Test directory: \"".$testdir."\" shouldn't be created", "error");
    @rmdir($testdir);
    @rmdir($this->testdir);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Delete()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   if(!FS::deleteS($this->testfile))
   {
    $this->e->add("SRO delete call failed", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(file_exists($this->testfile))
   {
    $this->e->add("Test file: \"".$this->testfile."\" should have been deleted", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }


   @rmdir($this->testdir);
   if(!@mkdir($this->testdir))
   {
    $this->e->add("Unable to create test directory: \"".$this->testdir."\"", "error");
    return UT_FAILED;
   }

   if(!FS::deleteS($this->testdir))
   {
    $this->e->add("SRO delete call failed", "error");
    @rmdir($this->testdir);
    return UT_FAILED;
   }

   if(file_exists($this->testdir))
   {
    $this->e->add("Test directory: \"".$this->testdir."\" should have been deleted", "error");
    @rmdir($this->testdir);
    return UT_FAILED;
   }


   if(!@mkdir($this->testdir))
   {
    $this->e->add("Unable to create test directory: \"".$this->testdir."\"", "error");
    return UT_FAILED;
   }

   if(!@touch($this->testdir."asd"))
   {
    $this->e->add("Unable to touch test file: \"".$this->testdir."asd\"", "error");
    @rmdir($this->testdir);
    return UT_FAILED;
   }

   if(FS::deleteS($this->testdir))
   {
    $this->e->add("Non-empty directory: \"".$this->testdir."\" should not have been deleted", "error");
    @unlink($this->testdir."asd");
    @rmdir($this->testdir);
    return UT_FAILED;
   }

   if(!FS::deleteS($this->testdir, true))
   {
    $this->e->add("SRO delete call failed", "error");
    @unlink($this->testdir."asd");
    @rmdir($this->testdir);
    return UT_FAILED;
   }

   if(file_exists($this->testdir))
   {
    $this->e->add("Test directory: \"".$this->testdir."\" should have been deleted", "error");
    @unlink($this->testdir."asd");
    @rmdir($this->testfile);
    return UT_FAILED;
   }


   if(!@mkdir($this->testdir))
   {
    $this->e->add("Unable to create test directory: \"".$this->testdir."\"", "error");
    return UT_FAILED;
   }

   if(!@mkdir($this->testdir."asd/"))
   {
    $this->e->add("Unable to create test directory: \"".$this->testdir."asd/\"", "error");
    return UT_FAILED;
   }

   if(!@touch($this->testdir."asd/asd"))
   {
    $this->e->add("Unable to touch test file: \"".$this->testdir."asd/asd\"", "error");
    @rmdir($this->testdir);
    return UT_FAILED;
   }

   if(FS::deleteS($this->testdir))
   {
    $this->e->add("Non-empty directory: \"".$this->testdir."\" should not have been deleted", "error");
    @unlink($this->testdir."asd/asd");
    @unlink($this->testdir."asd/");
    @rmdir($this->testdir);
    return UT_FAILED;
   }

   if(!FS::deleteS($this->testdir, true))
   {
    $this->e->add("SRO delete call failed", "error");
    @unlink($this->testdir."asd/asd");
    @unlink($this->testdir."asd/");
    @rmdir($this->testdir);
    return UT_FAILED;
   }

   if(file_exists($this->testdir))
   {
    $this->e->add("Test directory: \"".$this->testdir."\" should have been deleted", "error");
    @unlink($this->testdir."asd/asd");
    @unlink($this->testdir."asd/");
    @rmdir($this->testdir);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Read()
  {
   $this->_init();

   $testdata="testdata";
   if(!@file_put_contents($this->testfile, $testdata))
   {
    $this->e->add("Unable to create test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   if(($data=FS::readS($this->testfile))===false)
   {
    $this->e->add("Unable to read data from test file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($data!=$testdata)
   {
    $this->e->add("Data from test file: \"".$this->testfile."\" doesn't match input", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function ReadLines()
  {
   $this->_init();

   $testdata="test
data";
   if(!@file_put_contents($this->testfile, $testdata))
   {
    $this->e->add("Unable to create test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   if(($data=FS::readLinesS($this->testfile))===false)
   {
    $this->e->add("Unable to read data from test file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($data!=explode("\n", $testdata))
   {
    $this->e->add("Data from test file: \"".$this->testfile."\" doesn't match input", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function Grep()
  {
   $this->_init();

   $testdata="testdata";
   if(!@file_put_contents($this->testfile, $testdata))
   {
    $this->e->add("Unable to create test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   if(!FS::grepS("/test/", $this->testfile))
   {
    $this->e->add("Grep failed for test file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(FS::grepS("/asdasd/", $this->testfile))
   {
    $this->e->add("Grep should fail for test file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(!($m=FS::grepS("/(test)(\w+)/", $this->testfile)))
   {
    $this->e->add("Grep failed for test file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(!$this->_compare(Array(Array("test"), Array("data")), $m))
   {
    $this->e->add("Grep matches for test file: \"".$this->testfile."\" doesn't match expected output", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }

  function Replace()
  {
   $this->_init();

   $testdata="testdata";
   if(!@file_put_contents($this->testfile, $testdata))
   {
    $this->e->add("Unable to create test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   if(!FS::replaceS("/test/", "asd", $this->testfile))
   {
    $this->e->add("Replace failed for test file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(FS::readS($this->testfile)!="asddata")
   {
    $this->e->add("Replace doesn't match expected output for test file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(!FS::replaceS("/(asd)(\w+)/", "\\1\\1", $this->testfile))
   {
    $this->e->add("Replace failed for test file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(FS::readS($this->testfile)!="asdasd")
   {
    $this->e->add("Replace doesn't match expected output for test file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(FS::replaceS("/asdasdasd/", "data", $this->testfile))
   {
    $this->e->add("Replace should fail for test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }


   @unlink($this->testfile);
   return UT_OK;
  }


  function Write()
  {
   $this->_init();

   $testdata="testdata";

   if(FS::writeS($this->testfile, $testdata)===false)
   {
    $this->e->add("Unable to write data to test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   if(@file_get_contents($this->testfile)!=$testdata)
   {
    $this->e->add("Data in test file: \"".$this->testfile."\" doesn't match input", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(FS::writeS($this->testfile, $testdata)!==false)
   {
    $this->e->add("Shouldn't be able to write data to existing test file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(FS::writeS($this->testfile, $testdata, "overwrite")===false)
   {
    $this->e->add("Unable to overwrite data in test file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(@file_get_contents($this->testfile)!=$testdata)
   {
    $this->e->add("Data in test file: \"".$this->testfile."\" doesn't match input", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(FS::writeS($this->testfile, $testdata, "append")===false)
   {
    $this->e->add("Unable to append data in test file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(@file_get_contents($this->testfile)!=$testdata.$testdata)
   {
    $this->e->add("Data in test file: \"".$this->testfile."\" doesn't match input", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);

   if(FS::writeS($this->testfile, $testdata, false, true)===false)
   {
    $this->e->add("Unable to lock file: \"".$this->testfile."\" for writing", "error");
    return UT_FAILED;
   }

   if(@file_get_contents($this->testfile)!=$testdata)
   {
    $this->e->add("Data in test file: \"".$this->testfile."\" doesn't match input", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function Copy()
  {
   global $_path;
   $this->_init();

   $testfile=$_path["include"]."fs.php";

   if(!FS::mkdirS($this->testdir))
   {
    $this->e->add("Unable to create test dir: \"".$this->testdir."\"", "error");
    FS::deleteS($this->testdir, true);
    return UT_SKIPPED;
   }

   if(!FS::copyS($testfile, $this->testdir))
   {
    $this->e->add("Unable to copy test file: \"".$testfile."\" to: \"".$this->testdir."\"", "error");
    FS::deleteS($this->testdir, true);
    return UT_FAILED;
   }

   if(!FS::existsS($testfile))
   {
    $this->e->add("Copy test failed, file: \"".$testfile."\" doesn't exist", "error");
    FS::deleteS($this->testdir, true);
    return UT_FAILED;
   }

   if(!FS::existsS($this->testdir.preg_replace("!^.*/([^/]+)$!", "\\1", $testfile)))
   {
    $this->e->add("Copy test failed, file: \"".$this->testdir.preg_replace("!^.*/([^/]+)$!", "\\1", $testfile)."\" doesn't exist", "error");
    FS::deleteS($this->testdir, true);
    return UT_FAILED;
   }

   if(FS::copyS("nonexistingfile", $this->testdir."lala.php"))
   {
    $this->e->add("Should not be able to copy test file: \"nonexistingfile\" to: \"".$this->testfile."lala.php\"", "error");
    FS::deleteS($this->testdir, true);
    return UT_FAILED;
   }

   FS::deleteS($this->testdir, true);

   return UT_OK;
  }


  function Rename()
  {
   global $_path;
   $this->_init();

   $testfile=$_path["include"]."fs.php";

   if(!FS::mkdirS($this->testdir))
   {
    $this->e->add("Unable to create test dir: \"".$this->testdir."\"", "error");
    FS::deleteS($this->testdir, true);
    return UT_SKIPPED;
   }

   if(!FS::copyS($testfile, $this->testdir))
   {
    $this->e->add("Unable to copy test file: \"".$testfile."\" to: \"".$this->testdir."\"", "error");
    FS::deleteS($this->testdir, true);
    return UT_FAILED;
   }

   if(!FS::renameS($this->testdir.preg_replace("!^.*/([^/]+)$!", "\\1", $testfile), $this->testdir."lala.php"))
   {
    $this->e->add("Unable to rename test file: \"".$this->testdir.preg_replace("!^.*/([^/]+)$!", "\\1", $testfile)."\" to: \"".$this->testdir."lala.php\"", "error");
    FS::deleteS($this->testdir, true);
    return UT_FAILED;
   }

   if(FS::existsS($this->testdir.preg_replace("!^.*/([^/]+)$!", "\\1", $testfile)))
   {
    $this->e->add("Rename test failed, file: \"".$this->testdir.preg_replace("!^.*/([^/]+)$!", "\\1", $testfile)."\" still exists", "error");
    FS::deleteS($this->testdir, true);
    return UT_FAILED;
   }

   if(!FS::existsS($this->testdir."lala.php"))
   {
    $this->e->add("Rename test failed, file: \"".$this->testdir."lala.php"."\" doesn't exist", "error");
    FS::deleteS($this->testdir, true);
    return UT_FAILED;
   }

   if(FS::renameS("nonexistingfile", $this->testdir."lala.php"))
   {
    $this->e->add("Should not be able to rename test file: \"nonexistingfile\" to: \"".$this->testdir."lala.php\"", "error");
    FS::deleteS($this->testdir, true);
    return UT_FAILED;
   }

   FS::deleteS($this->testdir, true);
   return UT_OK;
  }




  function FileModeConvert()
  {
   if(($converted=FS::fileModeConvertS(755, "string"))===false)
   {
    $this->e->add("File mode convertion failed", "error");
    return UT_FAILED;
   }

   if($converted!="rwxr-xr-x")
   {
    $this->e->add("File mode convertion failed, expected \"rwxr-xr-x\"", "error");
    return UT_FAILED;
   }

   if(($converted=FS::fileModeConvertS(775, "asd"))!==false)
   {
    $this->e->add("File mode convertion failure test didn't fail", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function GetUser()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   //test user id
   if(($user=FS::getUserS($this->testfile, true))===false)
   {
    $this->e->add("Get user failed", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(($realUser=fileowner($this->testfile))!==$user)
   {
    $this->e->add("User id doesn't match real user id for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function GetGroup()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   //test user id
   if(($user=FS::getGroupS($this->testfile, true))===false)
   {
    $this->e->add("Get group failed", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(($realGroup=filegroup($this->testfile))!==$user)
   {
    $this->e->add("Group id doesn't match real group id for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function SetUser()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $tmp=fileowner($this->testfile);
   if(!@chown($this->testfile, $this->uid))
   {
    $this->e->add("Chown not supported", "warning");
    @unlink($this->testfile);
    return UT_SKIPPED;
   }
   @chown($this->testfile, $tmp);
   unset($tmp);

   if(FS::setUserS($this->testfile, $this->uid)===false)
   {
    $this->e->add("Set user failed", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(FS::getUserS($this->testfile, true)!==$this->uid)
   {
    $this->e->add("New user id doesn't match real user id for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function SetGroup()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $tmp=filegroup($this->testfile);
   if(!@chgrp($this->testfile, $this->uid))
   {
    $this->e->add("Chgrp not supported", "warning");
    @unlink($this->testfile);
    return UT_SKIPPED;
   }
   @chgrp($this->testfile, $tmp);
   unset($tmp);

   if(FS::setGroupS($this->testfile, $this->uid)===false)
   {
    $this->e->add("Set group failed", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(FS::getGroupS($this->testfile, true)!==$this->uid)
   {
    $this->e->add("New group id doesn't match real group id for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function GetMode()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   chmod($this->testfile, 0123);
   clearstatcache();

   if(($mode=FS::getModeS($this->testfile))===false)
   {
    $this->e->add("Get mode failed", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!=123)
   {
    $this->e->add("File mode should have been: \"123\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function GetUserMode()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   chmod($this->testfile, 0765);
   clearstatcache();

   if(($mode=FS::getUserModeS($this->testfile))===false)
   {
    $this->e->add("Get user mode failed", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!=7)
   {
    $this->e->add("File mode should have been: \"7\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function GetGroupMode()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   chmod($this->testfile, 0765);
   clearstatcache();

   if(($mode=FS::getGroupModeS($this->testfile))===false)
   {
    $this->e->add("Get group mode failed", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!=6)
   {
    $this->e->add("File mode should have been: \"6\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function GetOtherMode()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   chmod($this->testfile, 0765);
   clearstatcache();

   if(($mode=FS::getOtherModeS($this->testfile))===false)
   {
    $this->e->add("Get other mode failed", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!=5)
   {
    $this->e->add("File mode should have been: \"5\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function SetMode()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   if(FS::setModeS($this->testfile, 2, 3, 5)===false)
   {
    $this->e->add("Set mode failed", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }
   //set mode should also clear cache

   if(($mode=FS::getModeS($this->testfile))===false)
   {
    $this->e->add("Get mode failed", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!=235)
   {
    $this->e->add("File mode should have been: \"235\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function SetUserMode()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   if(FS::setModeS($this->testfile, 0, 0, 0)===false)
   {
    $this->e->add("Set mode failed", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(FS::setUserModeS($this->testfile, 2)===false)
   {
    $this->e->add("Set user mode failed", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(($mode=FS::getModeS($this->testfile))===false)
   {
    $this->e->add("Get mode failed", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!=200)
   {
    $this->e->add("File mode should have been: \"200\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function SetGroupMode()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   if(FS::setModeS($this->testfile, 0, 0, 0)===false)
   {
    $this->e->add("Set mode failed", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(FS::setGroupModeS($this->testfile, 1)===false)
   {
    $this->e->add("Set group mode failed", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(($mode=FS::getModeS($this->testfile))===false)
   {
    $this->e->add("Get mode failed", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!="010")
   {
    $this->e->add("File mode should have been: \"010\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function SetOtherMode()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   if(FS::setModeS($this->testfile, 0, 0, 0)===false)
   {
    $this->e->add("Set mode failed", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(FS::setOtherModeS($this->testfile, 3)===false)
   {
    $this->e->add("Set other mode failed", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(($mode=FS::getModeS($this->testfile))===false)
   {
    $this->e->add("Get mode failed", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if($mode!="003")
   {
    $this->e->add("File mode should have been: \"003\" for file: \"".$this->testfile."\"", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function GetTime()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   $times=Array(
    "created" => @filectime($this->testfile),
    "modified" => @filemtime($this->testfile),
    "accessed" => @fileatime($this->testfile),
   );

   if($times!=FS::getTimeS($this->testfile))
   {
    $this->e->add("Get time doesn't match expected output", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   foreach(array_keys($times) as $type)
   {
    if($times[$type]!=FS::getTimeS($this->testfile, $type))
    {
     $this->e->add("Get ".$type." time doesn't match expected output", "error");
     @unlink($this->testfile);
     return UT_FAILED;
    }
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function SetTime()
  {
   $this->_init();

   if(!@touch($this->testfile))
   {
    $this->e->add("Unable to touch test file: \"".$this->testfile."\"", "error");
    return UT_FAILED;
   }

   if(!FS::setTimeS($this->testfile, 2, "modified"))
   {
    $this->e->add("Set time failed", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   if(FS::getTimeS($this->testfile, "modified")!=2)
   {
    $this->e->add("Set time didn't change modified time", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }
   if(FS::getTimeS($this->testfile, "created")==2)
   {
    $this->e->add("Set time shouldn't have changed created time", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }
   if(FS::getTimeS($this->testfile, "accessed")==2)
   {
    $this->e->add("Set time shouldn't have changed accessed time", "error");
    @unlink($this->testfile);
    return UT_FAILED;
   }

   @unlink($this->testfile);
   return UT_OK;
  }


  function Diff()
  {
   $this->_init();

   if(($diff=FS::diffS($this->difffile1, $this->difffile2))===false)
   {
    $this->e->add("Diff failed", "error");
    return UT_FAILED;
   }

   if($diff!=$this->diff2)
   {
    $this->e->add("Diff doesn't match expected output", "error");
    return UT_FAILED;
   }

   //reversed diff
   if(($diff=FS::diffS($this->difffile2, $this->difffile1))===false)
    return UT_FAILED;

   if($diff!=$this->diff1)
   {
    $this->e->add("Diff doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function GetACL()
  {
   ob_start();
   if(FS::getACLS($this->testfile))
   {
    ob_end_clean();
    $this->e->add("SRO getACL call should fail", "error");
    return UT_FAILED;
   }

   ob_end_clean();
   return UT_OK;
  }

  function GetACLUser()
  {
   ob_start();
   if(FS::getACLUserS($this->testfile, "nobody"))
   {
    ob_end_clean();
    $this->e->add("SRO getACLUser call should fail", "error");
    return UT_FAILED;
   }

   ob_end_clean();
   return UT_OK;
  }

  function GetACLGroup()
  {
   ob_start();
   if(FS::getACLGroupS($this->testfile, "nogroup"))
   {
    ob_end_clean();
    $this->e->add("SRO getACLGroup call should fail", "error");
    return UT_FAILED;
   }

   ob_end_clean();
   return UT_OK;
  }


  function SetACL()
  {
   ob_start();
   if(FS::setACLS($this->testfile, false))
   {
    ob_end_clean();
    $this->e->add("SRO setACL call should fail", "error");
    return UT_FAILED;
   }

   ob_end_clean();
   return UT_OK;
  }

  function SetACLUser()
  {
   ob_start();
   if(FS::setACLUserS($this->testfile, "nobody", false))
   {
    ob_end_clean();
    $this->e->add("SRO setACLUser call should fail", "error");
    return UT_FAILED;
   }

   ob_end_clean();
   return UT_OK;
  }

  function SetACLGroup()
  {
   ob_start();
   if(FS::setACLGroupS($this->testfile, "nogroup", false))
   {
    ob_end_clean();
    $this->e->add("SRO setACLGroup call should fail", "error");
    return UT_FAILED;
   }

   ob_end_clean();
   return UT_OK;
  }

 }
?>
