<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/headers.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."headers.php");

 class HEADERS_EXTENDS_TEST extends Headers
 {
 }

 class UT_Headers extends UT_Classes
 {
  function Constructor()
  {
   $object=new Headers();

   if(!is_object($object)||get_class($object)!="Headers")
   {
    $this->e->add("Constructor doesn't return a Headers object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   if($object->mime!==false)
   {
    $this->e->add("Mime value isn't false", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new Headers();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Headers object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->mime=true;
   if($orig->mime==$copy->mime)
   {
    $this->e->add("Headers object copy also modified original \"mime\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("Headers object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Request()
  {
   $h=new Headers();

   if(function_exists("getallexpected"))
    $expected=getallexpected();
   else
   {
    $expected=Array();
    foreach($_SERVER as $name => $value)
     if(substr($name, 0, 5) == "HTTP_")
      $expected[str_replace(" ", "-", ucwords(strtolower(str_replace("_", " ", substr($name, 5)))))]=$value;
   }

   if(!($headers=$h->request()))
   {
    $this->e->add("No request headers found", "error");
    $this->e->add($h->e);
    return UT_FAILED;
   }

   if(!$this->_compare($expected, $headers))
   {
    $this->e->add("Request header don't match expected output", "error");
    return UT_FAILED;
   }

   //use headers in _SERVER
   if(!($headers=$h->request(true)))
   {
    $this->e->add("No request headers found", "error");
    $this->e->add($h->e);
    return UT_FAILED;
   }

   if(!$this->_compare($expected, $headers))
   {
    $this->e->add("Request header don't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Set()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   $h=new Headers();

   if(!$h->set("key", "value"))
   {
    $this->e->add("Adding new header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!in_array("key: value", headers_list()))
   {
    $this->e->add("Adding new header failed", "error", "Header not found");

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   $this->_restoreHeaders();
   return UT_OK;
  }


  function Exists()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   $h=new Headers();

   if(!$h->set("key", "value"))
   {
    $this->e->add("Adding new header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!$h->exists("key"))
   {
    $this->e->add("Exists new header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!$h->exists("KEY"))
   {
    $this->e->add("Exists new header upper case failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->exists("ASDASDASDASD"))
   {
    $this->e->add("Exists new header should have failed failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   $this->_restoreHeaders();
   return UT_OK;
  }


  function Get()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   $h=new Headers();

   if(!$h->set("key", "value"))
   {
    $this->e->add("Adding new header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("key")===false)
   {
    $this->e->add("Get new header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("key")!="value")
   {
    $this->e->add("Get new header failed", "error", "Value doesn't match");

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //array
   if(($headers=$h->get())===false)
   {
    $this->e->add("Get headers failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   foreach(headers_list() as $header)
   {
    list($key, $value)=explode(": ", $header, 2);

    if(!isset($headers[$key]))
    {
     $this->e->add("Get headers failed", "error", "Key not found");

     $this->_restoreHeaders();
     return UT_FAILED;
    }

    if($headers[$key]!=$value)
    {
     $this->e->add("Get headers failed", "error", "Value doesn't match");

     $this->_restoreHeaders();
     return UT_FAILED;
    }
   }

   $this->_restoreHeaders();
   return UT_OK;
  }


  function Delete()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   if(!function_exists("header_remove"))
   {
    $this->e->add("Header_remove function doesn't exist", "warning");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   $h=new Headers();

   if(!$h->set("key", "value"))
   {
    $this->e->add("Adding new header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("key")===false)
   {
    $this->e->add("Adding new header failed", "error", "Key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!$h->delete("key"))
   {
    $this->e->add("Delete header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("key")!==false)
   {
    $this->e->add("Delete header failed", "error", "Key still exists");

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //all headers
   if(!$h->set("key", "value"))
   {
    $this->e->add("Adding new header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("key")===false)
   {
    $this->e->add("Adding new header failed", "error", "Key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!$h->delete())
   {
    $this->e->add("Delete headers failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get())
   {
    $this->e->add("Delete headers failed", "error", "Keys still exist");

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   $this->_restoreHeaders();
   return UT_OK;
  }


  function Basic()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   $h=new Headers();

   //no-cache
   if(!$h->basic())
   {
    $this->e->add("Adding basic headers failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Cache-Control")===false)
   {
    $this->e->add("Adding basic headers failed", "error", "Cache key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(isset($_SERVER["HTTPS"])&&isset($_SERVER["HTTP_USER_AGENT"])&&preg_match("/MSIE/i", $_SERVER["HTTP_USER_AGENT"])) //HTTPS + IE
   {
    if(!preg_match("/^private, no-store, max-age=[0-9\-]+$/", $h->get("Cache-Control")))
    {
     $this->e->add("Adding basic headers failed", "error", "Cache value doesn't match");
     $this->e->add($h->e);

     $this->_restoreHeaders();
     return UT_FAILED;
    }
   }
   else //no HTTPS or non IE
   {
    if(!preg_match("/^no-cache, must-revalidate, max-age=[0-9\-]+$/", $h->get("Cache-Control")))
    {
     $this->e->add("Adding basic headers failed", "error", "Cache value doesn't match");
     $this->e->add($h->e);

     $this->_restoreHeaders();
     return UT_FAILED;
    }
   }

   if($h->get("Pragma")===false)
   {
    $this->e->add("Adding basic headers failed", "error", "Pragma key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(isset($_SERVER["HTTPS"])&&isset($_SERVER["HTTP_USER_AGENT"])&&preg_match("/MSIE/i", $_SERVER["HTTP_USER_AGENT"])) //HTTPS + IE
   {
    if($h->get("Pragma")!="private")
    {
     $this->e->add("Adding basic headers failed", "error", "Pragma value doesn't match");
     $this->e->add($h->e);

     $this->_restoreHeaders();
     return UT_FAILED;
    }
   }
   else //no HTTPS or non IE
   {
    if($h->get("Pragma")!="no-cache")
    {
     $this->e->add("Adding basic headers failed", "error", "Pragma value doesn't match");
     $this->e->add($h->e);

     $this->_restoreHeaders();
     return UT_FAILED;
    }
   }

   if($h->get("Expires")===false)
   {
    $this->e->add("Adding basic headers failed", "error", "Expires key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Expires")!="Thu, 01, Jan 1970 00:00:00 GMT")
   {
    $this->e->add("Adding basic headers failed", "error", "Expires value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("X-Frame-Options")!=="DENY")
   {
    $this->e->add("Adding basic headers failed", "error", "X-Frame-Options value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   $this->_restoreHeaders();
   return UT_OK;
  }


  function Frames()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   $h=new Headers();

   //DENY
   if(!$h->frames(false))
   {
    $this->e->add("Adding frames header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("X-Frame-Options")===false)
   {
    $this->e->add("Adding frames header failed", "error", "X-Frame-Options key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("X-Frame-Options")!=="DENY")
   {
    $this->e->add("Adding frames header failed", "error", "X-Frame-Options value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //SAMEORIGIN
   if(!$h->frames(true))
   {
    $this->e->add("Adding frames header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("X-Frame-Options")!=="SAMEORIGIN")
   {
    $this->e->add("Adding frames header failed", "error", "X-Frame-Options value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //ALL
   if(!$h->frames("all"))
   {
    $this->e->add("Adding frames header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("X-Frame-Options")!=="")
   {
    $this->e->add("Adding frames header failed", "error", "X-Frame-Options value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   $this->_restoreHeaders();
   return UT_OK;
  }


  function Cache()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   $h=new Headers();

   //no-cache
   if(!$h->cache())
   {
    $this->e->add("Adding cache header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Cache-Control")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Cache key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(isset($_SERVER["HTTPS"])&&isset($_SERVER["HTTP_USER_AGENT"])&&preg_match("/MSIE/i", $_SERVER["HTTP_USER_AGENT"])) //HTTPS + IE
   {
    if(!preg_match("/^private, no-store, max-age=[0-9\-]+$/", $h->get("Cache-Control")))
    {
     $this->e->add("Adding cache header failed", "error", "Cache value doesn't match");
     $this->e->add($h->e);

     $this->_restoreHeaders();
     return UT_FAILED;
    }
   }
   else //no HTTPS or non IE
   {
    if(!preg_match("/^no-cache, must-revalidate, max-age=[0-9\-]+$/", $h->get("Cache-Control")))
    {
     $this->e->add("Adding cache header failed", "error", "Cache value doesn't match");
     $this->e->add($h->e);

     $this->_restoreHeaders();
     return UT_FAILED;
    }
   }

   if($h->get("Pragma")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Pragma key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(isset($_SERVER["HTTPS"])&&isset($_SERVER["HTTP_USER_AGENT"])&&preg_match("/MSIE/i", $_SERVER["HTTP_USER_AGENT"])) //HTTPS + IE
   {
    if($h->get("Pragma")!="private")
    {
     $this->e->add("Adding cache header failed", "error", "Pragma value doesn't match");
     $this->e->add($h->e);

     $this->_restoreHeaders();
     return UT_FAILED;
    }
   }
   else //no HTTPS or non IE
   {
    if($h->get("Pragma")!="no-cache")
    {
     $this->e->add("Adding cache header failed", "error", "Pragma value doesn't match");
     $this->e->add($h->e);

     $this->_restoreHeaders();
     return UT_FAILED;
    }
   }

   if($h->get("Expires")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Expires key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Expires")!="Thu, 01, Jan 1970 00:00:00 GMT")
   {
    $this->e->add("Adding cache header failed", "error", "Expires value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   //private+store+now
   if(!$h->cache("private", true, time(), false))
   {
    $this->e->add("Adding cache header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Cache-Control")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Cache key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Cache-Control")!="private, max-age=0")
   {
    $this->e->add("Adding cache header failed", "error", "Cache value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Pragma")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Pragma key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Pragma")!="private")
   {
    $this->e->add("Adding cache header failed", "error", "Pragma value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Expires")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Expires key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Expires")!=gmdate("D, d, M Y H:i:s T"))
   {
    $this->e->add("Adding cache header failed", "error", "Expires value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //public+no-store+now
   if(!$h->cache("public", false, 10, true))
   {
    $this->e->add("Adding cache header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Cache-Control")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Cache key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Cache-Control")!="public, no-store, max-age=10")
   {
    $this->e->add("Adding cache header failed", "error", "Cache value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Pragma")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Pragma key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Pragma")!="public")
   {
    $this->e->add("Adding cache header failed", "error", "Pragma value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Expires")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Expires key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Expires")!=gmdate("D, d, M Y H:i:s T", time()+10))
   {
    $this->e->add("Adding cache header failed", "error", "Expires value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   $this->_restoreHeaders();
   return UT_OK;
  }


  function Filename()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   $h=new Headers();

   //filename
   if(!$h->filename("file.tst"))
   {
    $this->e->add("Adding filename attachment header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Content-Disposition")===false)
   {
    $this->e->add("Adding filename attachment header failed", "error", "File key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Content-Disposition")!="attachment; filename=\"file.tst\"")
   {
    $this->e->add("Adding filename attachment header failed", "error", "File value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   //filename inline
   if(!$h->filename("file.tst", "inline"))
   {
    $this->e->add("Adding filename inline header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Content-Disposition")===false)
   {
    $this->e->add("Adding filename inline header failed", "error", "File key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Content-Disposition")!="inline; filename=\"file.tst\"")
   {
    $this->e->add("Adding filename header failed", "error", "File value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   //filename+no attachment
   if(!$h->filename("file.tst", false))
   {
    $this->e->add("Adding filename header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Content-Disposition")===false)
   {
    $this->e->add("Adding filename header failed", "error", "File key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Content-Disposition")!="filename=\"file.tst\"")
   {
    $this->e->add("Adding filename header failed", "error", "File value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //no filename
   if($h->filename(""))
   {
    $this->e->add("Adding filename header should fail", "error");

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   $this->_restoreHeaders();
   return UT_OK;
  }


  function Type()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   $h=new Headers();

   //type
   if(!$h->type("text/html"))
   {
    $this->e->add("Adding type header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Content-Type")===false)
   {
    $this->e->add("Adding type header failed", "error", "Type key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!strstr($h->get("Content-Type"), "text/html"))
   {
    $this->e->add("Adding type header failed", "error", "Type value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   //invalid type
   if($h->type("text"))
   {
    $this->e->add("Adding type header should fail", "error");

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   //no type
   if($h->type(""))
   {
    $this->e->add("Adding type header should fail", "error");

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   $this->_restoreHeaders();
   return UT_OK;
  }


  function Size()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   $h=new Headers();

   //type
   if(!$h->size(2346))
   {
    $this->e->add("Adding size header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Content-Length")===false)
   {
    $this->e->add("Adding size header failed", "error", "Size key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Content-Length")!=2346)
   {
    $this->e->add("Adding size header failed", "error", "Size value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   //invalid size
   if($h->type(-2346))
   {
    $this->e->add("Adding size header should fail", "error");

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->type("text"))
   {
    $this->e->add("Adding size header should fail", "error");

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   //no siz3
   if($h->type(""))
   {
    $this->e->add("Adding size header should fail", "error");

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   $this->_restoreHeaders();
   return UT_OK;
  }


  function File()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   $h=new Headers();

   //filename
   if(!$h->file("file.tst"))
   {
    $this->e->add("Adding file header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Content-Disposition")===false)
   {
    $this->e->add("Adding file header failed", "error", "File key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Content-Disposition")!="attachment; filename=\"file.tst\"")
   {
    $this->e->add("Adding file header failed", "error", "File value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //filename
   if(!$h->file("file.tst", false, false, "inline"))
   {
    $this->e->add("Adding file header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Content-Disposition")===false)
   {
    $this->e->add("Adding file header failed", "error", "File key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Content-Disposition")!="inline; filename=\"file.tst\"")
   {
    $this->e->add("Adding file header failed", "error", "File value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //file+type
   $this->_restoreHeaders();

   if(!$h->file("file.tst", "text/html"))
   {
    $this->e->add("Adding file header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Content-Disposition")===false)
   {
    $this->e->add("Adding file header failed", "error", "Filename key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Content-Disposition")!="attachment; filename=\"file.tst\"")
   {
    $this->e->add("Adding file header failed", "error", "Filename value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Content-Type")===false)
   {
    $this->e->add("Adding file header failed", "error", "Type key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!strstr($h->get("Content-Type"), "text/html"))
   {
    $this->e->add("Adding file header failed", "error", "Type value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //file+auto type
   $this->_restoreHeaders();

   if(!$h->file("file.css", true))
   {
    $this->e->add("Adding file header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Content-Disposition")===false)
   {
    $this->e->add("Adding file header failed", "error", "Filename key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Content-Disposition")!="attachment; filename=\"file.css\"")
   {
    $this->e->add("Adding file header failed", "error", "Filename value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Content-Type")===false)
   {
    $this->e->add("Adding file header failed", "error", "Type key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!strstr($h->get("Content-Type"), "text/css"))
   {
    $this->e->add("Adding file header failed", "error", "Type value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //file+type+size
   $this->_restoreHeaders();

   if(!$h->file("file.tst", "text/html", 245345))
   {
    $this->e->add("Adding file header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Content-Disposition")===false)
   {
    $this->e->add("Adding file header failed", "error", "Filename key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Content-Disposition")!="attachment; filename=\"file.tst\"")
   {
    $this->e->add("Adding file header failed", "error", "Filename value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Content-Type")===false)
   {
    $this->e->add("Adding file header failed", "error", "Type key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!strstr($h->get("Content-Type"), "text/html"))
   {
    $this->e->add("Adding file header failed", "error", "Type value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Content-Length")===false)
   {
    $this->e->add("Adding file header failed", "error", "Size key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Content-Length")!=245345)
   {
    $this->e->add("Adding file header failed", "error", "Size value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   $this->_restoreHeaders();
   return UT_OK;
  }


  function Server()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   $h=new Headers();

   //empty server
   if(!$h->server())
   {
    $this->e->add("Adding server header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Server")===false)
   {
    $this->e->add("Adding server header failed", "error", "Server key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Server")!="")
   {
    $this->e->add("Adding server header failed", "error", "Server value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //jas+apache+php+ssl
   if(!$h->server(true, true, true, true))
   {
    $this->e->add("Adding server header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Server")===false)
   {
    $this->e->add("Adding server header failed", "error", "Server key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!preg_match("!^JAS/[0-9\.]+ Apache/[0-9\.]+ \([^\)]+\) OpenSSL/[0-9a-z\.]+ PHP/[0-9\.]+$!", $h->get("Server")))
   {
    $this->e->add("Adding server header failed", "error", "Server value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //custom
   if(!$h->server(false, false, false, false, "Test/1.0"))
   {
    $this->e->add("Adding server header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Server")===false)
   {
    $this->e->add("Adding server header failed", "error", "Server key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!preg_match("!^Test/[0-9\.]+$!", $h->get("Server")))
   {
    $this->e->add("Adding server header failed", "error", "Server value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   //custom jas+php+custom
   if(!$h->server("JAS/1.0", false, true, false, "Test/1.0"))
   {
    $this->e->add("Adding server header failed", "error");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if($h->get("Server")===false)
   {
    $this->e->add("Adding server header failed", "error", "Server key not found");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!preg_match("!^JAS/[0-9\.]+ PHP/[0-9\.]+ Test/[0-9\.]+$!", $h->get("Server")))
   {
    $this->e->add("Adding server header failed", "error", "Server value doesn't match");
    $this->e->add($h->e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   $this->_restoreHeaders();
   return UT_OK;
  }

 }

?>
