<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/protocol_http.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/protocol.php");
 require_once($_path["protocol"]."http.php");

 class UT_Protocol_HTTP extends UT_Protocol
 {
  function Constructor()
  {
   $object=new Protocol_HTTP();

   if(!is_object($object)||get_class($object)!="Protocol_HTTP")
   {
    $this->e->add("Constructor doesn't return a Protocol_HTTP object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->connection)||get_class($object->connection)!="Socket")
   {
    $this->e->add("Object: \"connection\" isn't an Socket object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->listen)||get_class($object->listen)!="Socket")
   {
    $this->e->add("Object: \"listen\" isn't an Socket object", "error");
    return UT_FAILED;
   }

   if(!$object->linebreak)
   {
    $this->e->add("Object linebreak not set", "error");
    return UT_FAILED;
   }

   if(!$object->fork)
   {
    $this->e->add("Object fork not enabled", "error");
    return UT_FAILED;
   }

   $blacklist=Array(
    "Protocol",
    "main",
    "type",
    "setUserGroup",

    "Protocol_HTTP",
    "setVar",
    "setHeader",
    "setCookie",
    "compressVars",
    "request",
    "_responsePrepareg",
    "_responseParseMethodg",
    "_responseParseLocationg",
    "_responseFileRequestg",
    "_responseParseVarsg",
    "_responseSetVarg",
    "_responseServerVarsg",
    "_responsePHPExecg",
    "_responseCleanTmpg",
    "_responseStatusg",
   );

   foreach($blacklist as $function)
   {
    if(!in_array($function, $object->blacklist))
    {
     $this->e->add("Function: ".$function." not in blacklist", "error");
     return UT_FAILED;
    }
   }

   if($object->type!="HTTP")
   {
    $this->e->add("Protocol type not set to: HTTP", "error");
    return UT_FAILED;
   }

   if($object->get("split"))
   {
    $this->e->add("Object value: split set", "error");
    return UT_FAILED;
   }

   if($object->get("connection.default.port")!=80)
   {
    $this->e->add("Object value: connection.default.port not set to: 80", "error");
    return UT_FAILED;
   }

   if($object->get("connection.default.keepalive")!==true)
   {
    $this->e->add("Object value: connection.default.keepalive not set to: true", "error");
    return UT_FAILED;
   }

   if($object->get("connection.default.timeout")!=5)
   {
    $this->e->add("Object value: connection.default.timeout not set to: 5", "error");
    return UT_FAILED;
   }

   if($object->get("http.version")!=1.1)
   {
    $this->e->add("Object value: HTTP version not set to: 1.1", "error");
    return UT_FAILED;
   }

   if($object->get("mode")!="client")
   {
    $this->e->add("Object value: mode not set to: client", "error");
    return UT_FAILED;
   }

   if(!isset($object->commands["connect"]))
   {
    $this->e->add("Object command: connect not set", "error");
    return UT_FAILED;
   }

   if(!isset($object->commands["disconnect"]))
   {
    $this->e->add("Object command: disconnect not set", "error");
    return UT_FAILED;
   }

   if(!isset($object->commands["main"]))
   {
    $this->e->add("Object command: main not set", "error");
    return UT_FAILED;
   }

   if($object->userAgent!="JAS HTTP/".JAS::versionS())
   {
    $this->e->add("Object user agent: doesn't match expected output", "error");
    return UT_FAILED;
   }

   if($object->sessionName!="PHPSESSID")
   {
    $this->e->add("Object session name: doesn't match expected output", "error");
    return UT_FAILED;
   }


   //server mode
   $object=new Protocol_HTTP(true);

   if($object->get("mode")!="server")
   {
    $this->e->add("Object value: mode not set to: server", "error");
    return UT_FAILED;
   }

   if($object->serverString!="JAS HTTPD/".JAS::versionS()." PHP/".phpversion())
   {
    $this->e->add("Object server string: doesn't match expected output", "error");
    return UT_FAILED;
   }

   if($object->serverRoot!=getcwd())
   {
    $this->e->add("Object server root: doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new Protocol_HTTP();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Protocol_HTTP object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->userAgent=-1;
   if($orig->userAgent==$copy->userAgent)
   {
    $this->e->add("Protocol_HTTP object copy also modified original \"userAgent\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("Protocol_HTTP object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   //server mode
   $orig=new Protocol_HTTP(true);
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Protocol_HTTP object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->serverString=-1;
   if($orig->serverString==$copy->serverString)
   {
    $this->e->add("Protocol_HTTP object copy also modified original \"serverString\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("Protocol_HTTP object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   $copy->mime->fake=-1;
   if(isset($orig->mime->fake)&&$orig->mime->fake==$copy->mime->fake)
   {
    $this->e->add("Protocol_HTTP object copy also modified original \"Mime\" object", "error");
    return UT_FAILED;
   }

   return parent::Copy_Constructor(); //run parent Copy_Constructor tests
  }


  function Clear()
  {
   if(UT_Protocol::clear()==UT_FAILED)
    return UT_FAILED;

   $p=new Protocol_HTTP();
   $p->vars[]=Array("key" => "key", "value" => "value");

   $p->clear();

   if($p->vars)
   {
    $this->e->add("Clear left vars set", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function SetVar()
  {
   $vars=Array();
   $p=new Protocol_HTTP();

   //add
   $vars[]=Array("key" => "Key1", "value" => "Value1");
   if(!$p->setVar("Key1", "Value1"))
   {
    $this->e->add("Set var shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$this->_compare($vars, $p->vars))
    return UT_FAILED;

   $vars[]=Array("key" => "Key2", "value" => "Value2");
   if(!$p->setVar("Key2", "Value2"))
   {
    $this->e->add("Set var shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$this->_compare($vars, $p->vars))
    return UT_FAILED;

   //double
   $vars[]=Array("key" => "Key2", "value" => "Value2a");
   if(!$p->setVar("Key2", "Value2a"))
   {
    $this->e->add("Set var shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$this->_compare($vars, $p->vars))
    return UT_FAILED;

   //delete
   unset($vars[0]);
   if(!$p->setVar("Key1"))
   {
    $this->e->add("Set var shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$this->_compare($vars, $p->vars))
    return UT_FAILED;

   //Array
   $vars[]=Array("key" => "Array[0]", "value" => "Value1");
   $vars[]=Array("key" => "Array[1]", "value" => "Value2");
   if(!$p->setVar("Array", Array("Value1", "Value2")))
   {
    $this->e->add("Set var shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$this->_compare($vars, $p->vars))
    return UT_FAILED;

   //Object
   $vars[]=Array("key" => "Object", "value" => serialize($this));
   if(!$p->setVar("Object", $this))
   {
    $this->e->add("Set var shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$this->_compare($vars, $p->vars))
    return UT_FAILED;

   return UT_OK;
  }

  function GetVar()
  {
   $vars=Array();
   $p=new Protocol_HTTP();

   //single
   if(!$p->setVar("Key1", "Value1"))
   {
    $this->e->add("Set var shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getVar("Key1")!="Value1")
   {
    $this->e->add("Get var doesn't match expected output", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$p->setVar("Key2", "Value2"))
   {
    $this->e->add("Set var shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getVar("Key2")!="Value2")
   {
    $this->e->add("Get var doesn't match expected output", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   //double
   if(!$p->setVar("Key2", "Value2a"))
   {
    $this->e->add("Set var shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getVar("Key2")!=Array("Value2", "Value2a"))
   {
    $this->e->add("Get var doesn't match expected output", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   //delete
   if(!$p->setVar("Key1"))
   {
    $this->e->add("Set var shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getVar("Key1")!==false)
   {
    $this->e->add("Get var doesn't match expected output", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   //Array
   if(!$p->setVar("Array", Array("Value1", "Value2")))
   {
    $this->e->add("Set var shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getVar("Array")!==false)
   {
    $this->e->add("Get var doesn't match expected output", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getVar("Array[0]")!=="Value1")
   {
    $this->e->add("Get var doesn't match expected output", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getVar("Array[1]")!=="Value2")
   {
    $this->e->add("Get var doesn't match expected output", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   //Object
   $vars[]=Array("key" => "Object", "value" => serialize($this));
   if(!$p->setVar("Object", $this))
   {
    $this->e->add("Set var shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getVar("Object")!==serialize($this))
   {
    $this->e->add("Get var doesn't match expected output", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function SetHeaderClient($server=false, $type=false)
  {
   $p=new Protocol_HTTP($server);

   if(!$type)
   {
    if($server)
     $check="response";
    else
     $check="request";
   }
   else
    $check=$type;

   $p->set("response.headers", Array()); //clear default headers
   $p->set("request.headers", Array()); //clear default headers

   //add
   //normal
   if(!$p->setHeader("Key1", "Value1", $type))
   {
    $this->e->add("Set header shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(sizeof($p->values[$check]["headers"])!=1)
   {
    $this->e->add("Header should contain 1 value", "error");
    return UT_FAILED;
   }

   if($p->values[$check]["headers"][0]!="Key1: Value1")
   {
    $this->e->add("First header should be: \"Key1: Value1\"", "error");
    return UT_FAILED;
   }

   //end
   if(!$p->setHeader("Key2", "Value2", $type))
   {
    $this->e->add("Set header shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(sizeof($p->values[$check]["headers"])!=2)
   {
    $this->e->add("Header should contain 2 values", "error");
    return UT_FAILED;
   }

   if($p->values[$check]["headers"][0]!="Key1: Value1")
   {
    $this->e->add("First header should be: \"Key1: Value1\"", "error");
    return UT_FAILED;
   }

   if($p->values[$check]["headers"][1]!="Key2: Value2")
   {
    $this->e->add("First header should be: \"Key2: Value2\"", "error");
    return UT_FAILED;
   }

   //start
   if(!$p->setHeader("Key3", "Value3", $type, true))
   {
    $this->e->add("Set header shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(sizeof($p->values[$check]["headers"])!=3)
   {
    $this->e->add("Header should contain 3 values", "error");
    return UT_FAILED;
   }

   if($p->values[$check]["headers"][0]!="Key3: Value3")
   {
    $this->e->add("First header should be: \"Key31: Value3\"", "error");
    return UT_FAILED;
   }

   if($p->values[$check]["headers"][1]!="Key1: Value1")
   {
    $this->e->add("First header should be: \"Key1: Value1\"", "error");
    return UT_FAILED;
   }

   if($p->values[$check]["headers"][2]!="Key2: Value2")
   {
    $this->e->add("First header should be: \"Key2: Value2\"", "error");
    return UT_FAILED;
   }


   //edit
   //normal
   if(!$p->setHeader("Key1", "Updated", $type))
   {
    $this->e->add("Set header shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->values[$check]["headers"][1]!="Key1: Updated")
   {
    $this->e->add("First header should be: \"Key1: Updated\"", "error");
    return UT_FAILED;
   }

   if(!$p->setHeader("Key2", "Updated", $type))
   {
    $this->e->add("Set header shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->values[$check]["headers"][1]!="Key1: Updated")
   {
    $this->e->add("First header should be: \"Key1: Updated\"", "error");
    return UT_FAILED;
   }

   if($p->values[$check]["headers"][2]!="Key2: Updated")
   {
    $this->e->add("First header should be: \"Key2: Updated\"", "error");
    return UT_FAILED;
   }

   if(!$p->setHeader("Key3", "Updated", $type))
   {
    $this->e->add("Set header shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->values[$check]["headers"][0]!="Key3: Updated")
   {
    $this->e->add("First header should be: \"Key31: Updated\"", "error");
    return UT_FAILED;
   }

   if($p->values[$check]["headers"][1]!="Key1: Updated")
   {
    $this->e->add("First header should be: \"Key1: Updated\"", "error");
    return UT_FAILED;
   }

   if($p->values[$check]["headers"][2]!="Key2: Updated")
   {
    $this->e->add("First header should be: \"Key2: Updated\"", "error");
    return UT_FAILED;
   }


   //delete
   if(!$p->setHeader("Key1", false, $type))
   {
    $this->e->add("Set header shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(sizeof($p->values[$check]["headers"])!=2)
   {
    $this->e->add("Header should contain 2 values", "error");
    return UT_FAILED;
   }

   if($p->values[$check]["headers"][0]!="Key3: Updated")
   {
    $this->e->add("First header should be: \"Key3: Updated\"", "error");
    return UT_FAILED;
   }

   if($p->values[$check]["headers"][1]!="Key2: Updated")
   {
    $this->e->add("First header should be: \"Key2: Updated\"", "error");
    return UT_FAILED;
   }

   if(!$p->setHeader("Key2", false, $type))
   {
    $this->e->add("Set header shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(sizeof($p->values[$check]["headers"])!=1)
   {
    $this->e->add("Header should contain 1 value", "error");
    return UT_FAILED;
   }

   if($p->values[$check]["headers"][0]!="Key3: Updated")
   {
    $this->e->add("First header should be: \"Key3: Updated\"", "error");
    return UT_FAILED;
   }

   if(!$p->setHeader("Key3", false, $type))
   {
    $this->e->add("Set header shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(sizeof($p->values[$check]["headers"])!=0)
   {
    $this->e->add("Header should contain 0 values", "error");
    return UT_FAILED;
   }

   //add
   //value
   if(!$p->setHeader(false, "Value", $type))
   {
    $this->e->add("Set header shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(sizeof($p->values[$check]["headers"])!=1)
   {
    $this->e->add("Header should contain 1 value", "error");
    return UT_FAILED;
   }

   if($p->values[$check]["headers"][0]!="Value")
   {
    $this->e->add("First header should be: \"Value\"", "error");
    return UT_FAILED;
   }


   //fail
   if($p->setHeader())
   {
    $this->e->add("Set header should have failed", "error");
    return UT_FAILED;
   }

   if($p->setHeader("Key", "Value", "kjahskjdhakjhsdkasd"))
   {
    $this->e->add("Set header should have failed", "error");
    return UT_FAILED;
   }


   return UT_OK;
  }

  function SetHeaderServer()
  {
   return $this->SetHeaderClient(true);
  }

  function SetHeaderClientResponse()
  {
   return $this->SetHeaderClient(false, "response");
  }


  function GetHeaderClient($server=false, $type=false)
  {
   $p=new Protocol_HTTP($server);

   if(!$type)
   {
    if($server)
     $set="request";
    else
     $set="response";
   }
   else
    $set=$type;

   $p->set("response.headers", Array()); //clear default headers
   $p->set("request.headers", Array()); //clear default headers

   if(!$p->setHeader("Key1", "Value1", $set))
   {
    $this->e->add("Set header shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getHeader("Key1", $type)!="Value1")
   {
    $this->e->add("Get header value doesn't match", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$p->setHeader("Key2", "Value2", $set))
   {
    $this->e->add("Set header shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getHeader("Key2", $type)!="Value2")
   {
    $this->e->add("Get header value doesn't match", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$p->setHeader("Key3", "Value3", $set, true))
   {
    $this->e->add("Set header shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getHeader("Key3", $type)!="Value3")
   {
    $this->e->add("Get header value doesn't match", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }


   //fail
   if($p->getHeader("kjahskjdhakjhsdkasd", $type)!==false)
   {
    $this->e->add("Get header should have failed", "error");
    return UT_FAILED;
   }

   if($p->getHeader("Key3", "kjahskjdhakjhsdkasd")!==false)
   {
    $this->e->add("Get header should have failed", "error");
    return UT_FAILED;
   }


   return UT_OK;
  }

  function GetHeaderServer()
  {
   return $this->GetHeaderClient(true);
  }

  function GetHeaderClientRequest()
  {
   return $this->GetHeaderClient(false, "request");
  }


  function SetCookieClient($server=false, $type=false)
  {
   $p=new Protocol_HTTP($server);

   if(!$type)
   {
    if($server)
     $check="response";
    else
     $check="request";
   }
   else
    $check=$type;

   //set corresponding cookie header
   if($check=="response")
    $header="Set-Cookie";
   else //request
    $header="Cookie";

   $p->set("response.headers", Array()); //clear default headers
   $p->set("request.headers", Array()); //clear default headers

   //add
   if(!$p->setCookie("Key1", "Value1", $type))
   {
    $this->e->add("Set cookie shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getHeader($header, $check)!="Key1=Value1")
   {
    $this->e->add("Get header value doesn't match", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$p->setCookie("Key2", "Value2", $type))
   {
    $this->e->add("Set cookie shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getHeader($header, $check)!="Key1=Value1; Key2=Value2")
   {
    $this->e->add("Get header value doesn't match", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$p->setCookie("Key3", "", $type))
   {
    $this->e->add("Set cookie shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getHeader($header, $check)!="Key1=Value1; Key2=Value2; Key3=")
   {
    $this->e->add("Get header value doesn't match", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   //edit
   if(!$p->setCookie("Key1", "Updated1", $type))
   {
    $this->e->add("Set cookie shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getHeader($header, $check)!="Key1=Updated1; Key2=Value2; Key3=")
   {
    $this->e->add("Get header value doesn't match", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$p->setCookie("Key2", "Updated2", $type))
   {
    $this->e->add("Set cookie shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getHeader($header, $check)!="Key1=Updated1; Key2=Updated2; Key3=")
   {
    $this->e->add("Get header value doesn't match", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$p->setCookie("Key3", "Updated3", $type))
   {
    $this->e->add("Set cookie shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getHeader($header, $check)!="Key1=Updated1; Key2=Updated2; Key3=Updated3")
   {
    $this->e->add("Get header value doesn't match", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   //delete
   if(!$p->setCookie("Key2", false, $type))
   {
    $this->e->add("Set cookie shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getHeader($header, $check)!="Key1=Updated1; Key3=Updated3")
   {
    $this->e->add("Get header value doesn't match", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$p->setCookie("Key1", false, $type))
   {
    $this->e->add("Set cookie shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getHeader($header, $check)!="Key3=Updated3")
   {
    $this->e->add("Get header value doesn't match", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$p->setCookie("Key3", false, $type))
   {
    $this->e->add("Set cookie shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getHeader($header, $check)!="")
   {
    $this->e->add("Get header value doesn't match", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }


   //fail
   if($p->setCookie("Key", "Value", "kjahskjdhakjhsdkasd"))
   {
    $this->e->add("Set cookie should have failed", "error");
    return UT_FAILED;
   }


   return UT_OK;
  }

  function SetCookieServer()
  {
   return $this->SetCookieClient(true);
  }

  function SetCookieClientResponse()
  {
   return $this->SetCookieClient(false, "response");
  }


  function GetCookieClient($server=false, $type=false)
  {
   $p=new Protocol_HTTP($server);

   if(!$type)
   {
    if($server)
     $set="request";
    else
     $set="response";
   }
   else
    $set=$type;

   $p->set("response.headers", Array()); //clear default headers
   $p->set("request.headers", Array()); //clear default headers

   if(!$p->setCookie("Key1", "Value1", $set))
   {
    $this->e->add("Set cookie shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getCookie("Key1", $type)!="Value1")
   {
    $this->e->add("Get cookie value doesn't match", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$p->setCookie("Key2", "Value2", $set))
   {
    $this->e->add("Set cookie shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getCookie("Key2", $type)!="Value2")
   {
    $this->e->add("Get cookie value doesn't match", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$p->setCookie("Key3", "Value3", $set, true))
   {
    $this->e->add("Set cookie shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getCookie("Key3", $type)!="Value3")
   {
    $this->e->add("Get cookie value doesn't match", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }


   //fail
   if($p->getCookie("kjahskjdhakjhsdkasd", $set)!==false)
   {
    $this->e->add("Get cookie should have failed", "error");
    return UT_FAILED;
   }

   if($p->getCookie("Key3", "kjahskjdhakjhsdkasd")!==false)
   {
    $this->e->add("Get cookie should have failed", "error");
    return UT_FAILED;
   }


   return UT_OK;
  }

  function GetCookieServer()
  {
   return $this->GetCookieClient(true);
  }

  function GetCookieClientRequest()
  {
   return $this->GetCookieClient(false, "request");
  }


  function SetSessionClient($server=false, $type=false)
  {
   $p=new Protocol_HTTP($server);

   if(!$type)
   {
    if($server)
     $check="response";
    else
     $check="request";
   }
   else
    $check=$type;

   //set corresponding cookie header
   if($check=="response")
    $header="Set-Cookie";
   else //request
    $header="Cookie";

   $p->set("response.headers", Array()); //clear default headers
   $p->set("request.headers", Array()); //clear default headers

   //add
   if(!$p->setSession("Value1", $type))
   {
    $this->e->add("Set session shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getCookie("PHPSESSID", $check)!="Value1")
   {
    $this->e->add("Get session value doesn't match", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   //edit
   if(!$p->setSession("Updated1", $type))
   {
    $this->e->add("Set session shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getCookie("PHPSESSID", $check)!="Updated1")
   {
    $this->e->add("Get session value doesn't match", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   //delete
   if(!$p->setSession(false, $type))
   {
    $this->e->add("Set session shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getCookie("PHPSESSID", $check)!==false)
   {
    $this->e->add("Get session value doesn't match", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   //fail
   if($p->setSession("Value", "kjahskjdhakjhsdkasd")!==false)
   {
    $this->e->add("Set session should have failed", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function SetSessionServer()
  {
   return $this->SetSessionClient(true);
  }

  function SetSessionClientResponse()
  {
   return $this->SetSessionClient(false, "response");
  }


  function GetSessionClient($server=false, $type=false)
  {
   $p=new Protocol_HTTP($server);

   if(!$type)
   {
    if($server)
     $set="request";
    else
     $set="response";
   }
   else
    $set=$type;

   $p->set("response.headers", Array()); //clear default headers
   $p->set("request.headers", Array()); //clear default headers

   if(!$p->setSession("Value1", $set))
   {
    $this->e->add("Set session shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if($p->getSession($type)!="Value1")
   {
    $this->e->add("Get session value doesn't match", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   //fail
   if($p->getSession("kjahskjdhakjhsdkasd")!==false)
   {
    $this->e->add("Get session should have failed", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetSessionServer()
  {
   return $this->GetSessionClient(true);
  }

  function GetSessionClientRequest()
  {
   return $this->GetSessionClient(false, "request");
  }


  function Request()
  {
   return UT_SKIPPED;
  }

  function Response()
  {
   return UT_SKIPPED;
  }

  function Read()
  {
   return UT_SKIPPED;
  }

 }

?>
