<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/session.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."session.php");

 class UT_Session extends UT_Classes
 {
  function Constructor()
  {
   global $_jseskey;

   $object=new Session();

   if(!is_object($object)||get_class($object)!="Session")
   {
    $this->e->add("Constructor doesn't return a Session object", "error");
    return UT_FAILED;
   }

   if(!session_id())
   {
    $this->e->add("Constructor started a session", "error");
    return UT_FAILED;
   }

   if($object->global!="_global")
   {
    $this->e->add("Object: global session name incorrect", "error");
    return UT_FAILED;
   }

   if($object->key!=$_jseskey)
   {
    $this->e->add("Object: session key doesn't match jses key", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function SetGlobal()
  {
   $session=new Session();

   $session->setCurrent();

   if(!$session->setGlobal())
   {
    $this->e->add("Session set global shouldn't have failed", "error");
    $session->setCurrent();
    return UT_FAILED;
   }

   if(!$this->_compare($_SESSION["jses"]["_global"], $session->values))
   {
    $this->e->add("Session global doesn't match expected values", "error");
    $session->setCurrent();
    return UT_FAILED;
   }

   $session->setCurrent();
   return UT_OK;
  }

  function SetCurrent()
  {
   $session=new Session();

   $session->setGlobal();

   if(!$session->setCurrent())
   {
    $this->e->add("Session set current shouldn't have failed", "error");
    $session->setCurrent();
    return UT_FAILED;
   }

   if(!$this->_compare($_SESSION["jses"][$session->key], $session->values))
   {
    $this->e->add("Session current doesn't match expected values", "error");
    $session->setCurrent();
    return UT_FAILED;
   }

   $session->setCurrent();
   return UT_OK;
  }

  function SetKey()
  {
   global $_jseskey;

   $session=new Session();

   if(!$session->setKey("ASDADASDAS"))
   {
    $this->e->add("Session set key shouldn't have failed", "error");
    $session->setKey($_jseskey);
    unset($_SESSION["jses"]["ASDADASDAS"]);
    return UT_FAILED;
   }

   if($session->key!="ASDADASDAS")
   {
    $this->e->add("Session key doesn't match expected values", "error");
    $session->setKey($_jseskey);
    unset($_SESSION["jses"]["ASDADASDAS"]);
    return UT_FAILED;
   }

   if($_SESSION["jseskey"]!="ASDADASDAS")
   {
    $this->e->add("Session jses key doesn't match expected values", "error");
    $session->setKey($_jseskey);
    unset($_SESSION["jses"]["ASDADASDAS"]);
    return UT_FAILED;
   }

   if(!$this->_compare($_SESSION["jses"]["ASDADASDAS"], $session->values))
   {
    $this->e->add("Session key values doesn't match expected values", "error");
    $session->setKey($_jseskey);
    unset($_SESSION["jses"]["ASDADASDAS"]);
    return UT_FAILED;
   }

   $session->setKey($_jseskey);
   unset($_SESSION["jses"]["ASDADASDAS"]);
   return UT_OK;
  }

  function GetKey()
  {
   global $_jseskey;

   $session=new Session();

   if($session->getKey()!=$_jseskey)
   {
    $this->e->add("Session get key doesn't match jses key", "error");
    $session->setKey($_jseskey);
    return UT_FAILED;
   }

   if(!$session->setKey("ASDADASDAS"))
   {
    $this->e->add("Session set current shouldn't have failed", "error");
    $session->setKey($_jseskey);
    unset($_SESSION["jses"]["ASDADASDAS"]);
    return UT_FAILED;
   }

   if($session->getKey()!="ASDADASDAS")
   {
    $this->e->add("Session get key doesn't match expected values", "error");
    $session->setKey($_jseskey);
    unset($_SESSION["jses"]["ASDADASDAS"]);
    return UT_FAILED;
   }

   if(!$session->setKey($_jseskey))
   {
    $this->e->add("Session set key shouldn't have failed", "error");
    $session->setKey($_jseskey);
    unset($_SESSION["jses"]["ASDADASDAS"]);
    return UT_FAILED;
   }

   if($session->getKey()!=$_jseskey)
   {
    $this->e->add("Session get key doesn't match jses key", "error");
    $session->setKey($_jseskey);
    unset($_SESSION["jses"]["ASDADASDAS"]);
    return UT_FAILED;
   }

   $session->setKey($_jseskey);
   unset($_SESSION["jses"]["ASDADASDAS"]);
   return UT_OK;
  }


  function GetKeys()
  {
   global $_jseskey;

   $session=new Session();

   if(!($keys=$session->getKeys()))
   {
    $this->e->add("Session get keys shouldn't have failed", "error");
    return UT_FAILED;
   }

   if(!in_array("default", $keys))
   {
    $this->e->add("Session default key not in key list", "error");
    return UT_FAILED;
   }

   if(!$session->setKey("ASDADASDAS"))
   {
    $this->e->add("Session set current shouldn't have failed", "error");
    $session->setKey($_jseskey);
    unset($_SESSION["jses"]["ASDADASDAS"]);
    return UT_FAILED;
   }

   if(!($keys=$session->getKeys()))
   {
    $this->e->add("Session get keys shouldn't have failed", "error");
    $session->setKey($_jseskey);
    unset($_SESSION["jses"]["ASDADASDAS"]);
    return UT_FAILED;
   }

   if(!in_array("default", $keys))
   {
    $this->e->add("Session default key not in key list", "error");
    $session->setKey($_jseskey);
    unset($_SESSION["jses"]["ASDADASDAS"]);
    return UT_FAILED;
   }

   if(!in_array("ASDADASDAS", $keys))
   {
    $this->e->add("Session new key not in key list", "error");
    $session->setKey($_jseskey);
    unset($_SESSION["jses"]["ASDADASDAS"]);
    return UT_FAILED;
   }


   $session->setKey($_jseskey);
   unset($_SESSION["jses"]["ASDADASDAS"]);
   return UT_OK;
  }


  function Set()
  {
   $session=new Session();

   foreach(Array("value", 123, 0, 0.1, false, true, Array()) as $value)
   {
    if(!$session->set("key", $value))
    {
     $this->e->add("Set shouldn't have failed", "error", $value);
     $this->e->add($session->e);
     return UT_FAILED;
    }

    if(!isset($session->values["key"]))
    {
     $this->e->add("Values list doesn't have a key: key", "error", $value);
     return UT_FAILED;
    }

    if($session->values["key"]!==$value)
    {
     $this->e->add("Values list value doesn't match for key: key", "error", $value);
     return UT_FAILED;
    }
   }

   if(!$session->set("key"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($session->e);
    return UT_FAILED;
   }

   if(isset($session->values["key"]))
   {
    $this->e->add("Values list shouldn't have a key: key", "error");
    return UT_FAILED;
   }


   //array
   unset($session->values["key"]);
   if(!$session->set("key.0", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($session->e);
    return UT_FAILED;
   }

   if(!isset($session->values["key"]))
   {
    $this->e->add("Values list doesn't have a key: key", "error");
    return UT_FAILED;
   }

   if(!is_array($session->values["key"]))
   {
    $this->e->add("Values list value isn't an array for key: key", "error");
    return UT_FAILED;
   }

   if(!isset($session->values["key"][0]))
   {
    $this->e->add("Values list doesn't have a key: key.sub", "error");
    return UT_FAILED;
   }

   if($session->values["key"][0]!="value")
   {
    $this->e->add("Values list value doesn't match for key: key.sub", "error");
    return UT_FAILED;
   }

   unset($session->values["key"]);
   if(!$session->set("key.sub", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($session->e);
    return UT_FAILED;
   }

   if(!isset($session->values["key"]))
   {
    $this->e->add("Values list doesn't have a key: key", "error");
    return UT_FAILED;
   }

   if(!is_array($session->values["key"]))
   {
    $this->e->add("Values list value isn't an array for key: key", "error");
    return UT_FAILED;
   }

   if(!isset($session->values["key"]["sub"]))
   {
    $this->e->add("Values list doesn't have a key: key.sub", "error");
    return UT_FAILED;
   }

   if($session->values["key"]["sub"]!="value")
   {
    $this->e->add("Values list value doesn't match for key: key.sub", "error");
    return UT_FAILED;
   }


   //object
   unset($session->values["key"]);
   if(!$session->set("key", $this))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($session->e);
    return UT_FAILED;
   }

   if(!isset($session->values["key"]))
   {
    $this->e->add("Values list doesn't have a key: key", "error");
    return UT_FAILED;
   }

   if(!is_object($session->values["key"]))
   {
    $this->e->add("Values list value isn't an object for key: key", "error");
    return UT_FAILED;
   }

   if(!$session->set("key.sub", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($session->e);
    return UT_FAILED;
   }

   if(!isset($session->values["key"]->sub))
   {
    $this->e->add("Values list doesn't have a key: key.sub", "error");
    return UT_FAILED;
   }

   if($session->values["key"]->sub!="value")
   {
    $this->e->add("Values list value doesn't match for key: key.sub", "error");
    return UT_FAILED;
   }

   if(!$session->set("key.e.sub", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($session->e);
    return UT_FAILED;
   }

   if(!isset($session->values["key"]->e->sub))
   {
    $this->e->add("Values list doesn't have a key: key.e.sub", "error");
    return UT_FAILED;
   }

   if($session->values["key"]->e->sub!="value")
   {
    $this->e->add("Values list value doesn't match for key: key.e.sub", "error");
    return UT_FAILED;
   }


   //failures
   unset($session->values["key"]);
   if(!$session->set("key", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($session->e);
    return UT_FAILED;
   }

   if($session->set("key.sub", "value")) //try to use string as object/array
   {
    $this->e->add("Set should have failed", "error");
    return UT_FAILED;
   }

   unset($session->values["key"]);
   return UT_OK;
  }

  function Get()
  {
   $session=new Session();

   foreach(Array("value", 123, 0, 0.1, false, true, Array()) as $value)
   {
    if(!$session->set("key", $value))
    {
     $this->e->add("Set shouldn't have failed", "error", $value);
     $this->e->add($session->e);
     return UT_FAILED;
    }

    if($session->get("key")!==$value)
    {
     $this->e->add("Values list value doesn't match for key: key", "error", $value);
     return UT_FAILED;
    }
   }


   //array
   unset($session->values["key"]);
   if(!$session->set("key.sub", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($session->e);
    return UT_FAILED;
   }

   if($session->get("key")!=Array("sub" => "value"))
   {
    $this->e->add("Values list value doesn't match for key: key", "error");
    return UT_FAILED;
   }

   if($session->get("key.sub")!="value")
   {
    $this->e->add("Values list value doesn't match for key: key.sub", "error");
    return UT_FAILED;
   }


   //object
   unset($session->values["key"]);
   if(!$session->set("key", $this))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($session->e);
    return UT_FAILED;
   }

   if($session->get("key")!=$this)
   {
    $this->e->add("Values list value doesn't match for key: key", "error");
    return UT_FAILED;
   }

   if(!$session->set("key.sub", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($session->e);
    return UT_FAILED;
   }

   if($session->get("key.sub")!="value")
   {
    $this->e->add("Values list value doesn't match for key: key.sub", "error");
    return UT_FAILED;
   }

   if(!$session->set("key.e.sub", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($session->e);
    return UT_FAILED;
   }

   if($session->get("key.e.sub")!="value")
   {
    $this->e->add("Values list value doesn't match for key: key.e.sub", "error");
    return UT_FAILED;
   }

   //failures
   if($session->get("asd")!==NULL)
   {
    $this->e->add("Values list value shouldn't be set for key: asd", "error");
    return UT_FAILED;
   }

   unset($session->values["key"]);
   if(!$session->set("key", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($session->e);
    return UT_FAILED;
   }

   if($session->get("key.sub")!==NULL)
   {
    $this->e->add("Values list value shouldn't be set for key: key.sub", "error");
    return UT_FAILED;
   }


   unset($session->values["key"]);
   if(!$session->set("key", Array()))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($session->e);
    return UT_FAILED;
   }

   if($session->get("key.sub")!==NULL)
   {
    $this->e->add("Values list value shouldn't be set for key: key.sub", "error");
    return UT_FAILED;
   }

   unset($session->values["key"]);
   return UT_OK;
  }

  function GetReference()
  {
   $session=new Session();

   foreach(Array("value", 123, 0, 0.1, false, true, Array()) as $value)
   {
    if(!$session->set("key", $value))
    {
     $this->e->add("Set shouldn't have failed", "error", $value);
     $this->e->add($session->e);
     return UT_FAILED;
    }

    if($session->getReference("key")!==$value)
    {
     $this->e->add("Values list value doesn't match for key: key", "error", $value);
     return UT_FAILED;
    }

    $tmp=&$session->getReference("key");
    $tmp="asd";
    if($session->get("key")!==$tmp)
    {
     $this->e->add("Values list referenced value doesn't match for key: key", "error", $value);
     return UT_FAILED;
    }
   }


   //array
   unset($session->values["key"]);
   if(!$session->set("key.sub", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($session->e);
    return UT_FAILED;
   }

   if($session->getReference("key")!=Array("sub" => "value"))
   {
    $this->e->add("Values list value doesn't match for key: key", "error");
    return UT_FAILED;
   }

   if($session->getReference("key.sub")!="value")
   {
    $this->e->add("Values list value doesn't match for key: key.sub", "error");
    return UT_FAILED;
   }

   $tmp=&$session->getReference("key.sub");
   $tmp="asd";
   if($session->get("key.sub")!==$tmp)
   {
    $this->e->add("Values list referenced value doesn't match for key: key.sub", "error", $value);
    return UT_FAILED;
   }


   //object
   unset($session->values["key"]);
   if(!$session->set("key", $this))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($session->e);
    return UT_FAILED;
   }

   if($session->getReference("key")!=$this)
   {
    $this->e->add("Values list value doesn't match for key: key", "error");
    return UT_FAILED;
   }

   if(!$session->set("key.sub", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($session->e);
    return UT_FAILED;
   }

   if($session->getReference("key.sub")!="value")
   {
    $this->e->add("Values list value doesn't match for key: key.sub", "error");
    return UT_FAILED;
   }

   if(!$session->set("key.e.sub", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($session->e);
    return UT_FAILED;
   }

   if($session->getReference("key.e.sub")!="value")
   {
    $this->e->add("Values list value doesn't match for key: key.e.sub", "error");
    return UT_FAILED;
   }

   unset($session->values["key"]);
   return UT_OK;
  }



  function BackendKey()
  {
   global $_jseskey;

   $session=new Session();

   if($session->backendKey()!="jas")
   {
    $this->e->add("Session backend key doesn't match expected values", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function InBackend()
  {
   global $_jseskey;

   $session=new Session();

   if($session->inBackend())
   {
    $this->e->add("Session in backend should fail", "error");
    return UT_FAILED;
   }

   if(!$session->setKey("jas"))
   {
    $this->e->add("Session set key shouldn't have failed", "error");
    $session->setKey($_jseskey);
    return UT_FAILED;
   }

   if(!$session->inBackend())
   {
    $this->e->add("Session in backend shouldn't fail", "error");
    return UT_FAILED;
   }

   $session->setKey($_jseskey);
   return UT_OK;
  }


  function Annex()
  {
   $this->e->add("Session annex unit test will more than likely break the session", "warning");
   return UT_SKIPPED;
  }


  function SwitchBackend()
  {
   global $_session, $_path;

   if($_session->inBackend())
   {
    $this->e->add("Session switch backend requires a backend project to be set", "warning");
    return UT_SKIPPED;
   }

//switch to backend
   if(!$_session->switchBackend(true))
   {
    $this->e->add("Session switch to backend shouldn't fail", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }

   if(!$_session->inBackend())
   {
    $this->e->add("Session switch to backend should have switched to backend", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }

   if($_path["project"]["root"]!=$_path["projects"]."jas/")
   {
    $this->e->add("Session switch to backend should have updated the path", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }

   if($_session->key!="jas")
   {
    $this->e->add("Session switch to backend should have updated the session key", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }


//switch to frontend
   if(!$_session->switchBackend(false))
   {
    $this->e->add("Session switch to frontend shouldn't fail", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }

   if($_session->inBackend())
   {
    $this->e->add("Session switch to frontend should have switched to frontend", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }

   if($_path["project"]["root"]!=$_path["projects"].$_session->get("project")."/")
   {
    $this->e->add("Session switch to backend should have updated the path", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }

   if($_session->key!=$_session->get("project"))
   {
    $this->e->add("Session switch to frontend should have updated the session key", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }


//switch to backend (auto detect)
   if(!$_session->switchBackend())
   {
    $this->e->add("Session switch to backend shouldn't fail", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }

   if(!$_session->inBackend())
   {
    $this->e->add("Session switch to backend should have switched to backend", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }

   if($_path["project"]["root"]!=$_path["projects"]."jas/")
   {
    $this->e->add("Session switch to backend should have updated the path", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }

   if($_session->key!="jas")
   {
    $this->e->add("Session switch to backend should have updated the session key", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }


//switch to frontend (auto detect)
   if(!$_session->switchBackend())
   {
    $this->e->add("Session switch to frontend shouldn't fail", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }

   if($_session->inBackend())
   {
    $this->e->add("Session switch to fronted should have switched to frontend", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }

   if($_path["project"]["root"]!=$_path["projects"].$_session->get("project")."/")
   {
    $this->e->add("Session switch to frontend should have updated the path", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }

   if($_session->key!=$_session->get("project"))
   {
    $this->e->add("Session switch to backend should have updated the session key", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }


   $_session->switchBackend(false); //switch to frontend
   return UT_OK;
  }

  function Frontend()
  {
   global $_session, $_path;

   if($_session->inBackend())
   {
    $this->e->add("Session frontend requires a backend project to be set", "warning");
    return UT_SKIPPED;
   }

   if(!$_session->switchBackend(true)) //switch to backend
   {
    $this->e->add("Session switch to backend shouldn't fail", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }

   if(!$_session->frontend())
   {
    $this->e->add("Session switch to frontend shouldn't fail", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }

   if($_session->inBackend())
   {
    $this->e->add("Session switch to frontend should have switched to frontend", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }

   if($_path["project"]["root"]!=$_path["projects"].$_session->get("project")."/")
   {
    $this->e->add("Session switch to frontend should have updated the path", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }

   if($_session->key!=$_session->get("project"))
   {
    $this->e->add("Session switch to frontend should have updated the session key", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }


//again!
   if(!$_session->frontend())
   {
    $this->e->add("Session switch to frontend shouldn't fail", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }

   if($_session->inBackend())
   {
    $this->e->add("Session switch to frontend should have switched to frontend", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }

   if($_path["project"]["root"]!=$_path["projects"].$_session->get("project")."/")
   {
    $this->e->add("Session switch to frontend should have updated the path", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }

   if($_session->key!=$_session->get("project"))
   {
    $this->e->add("Session switch to frontend should have updated the session key", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }


   $_session->switchBackend(false); //switch to frontend
   return UT_OK;
  }

  function Backend()
  {
   global $_session, $_path;

   if($_session->inBackend())
   {
    $this->e->add("Session backend requires a backend project to be set", "warning");
    return UT_SKIPPED;
   }

   if(!$_session->backend())
   {
    $this->e->add("Session switch to backend shouldn't fail", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }

   if(!$_session->inBackend())
   {
    $this->e->add("Session switch to backend should have switched to backend", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }

   if($_path["project"]["root"]!=$_path["projects"]."jas/")
   {
    $this->e->add("Session switch to backend should have updated the path", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }

   if($_session->key!="jas")
   {
    $this->e->add("Session switch to backend should have updated the session key", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }


//again!
   if(!$_session->backend())
   {
    $this->e->add("Session switch to backend shouldn't fail", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }

   if(!$_session->inBackend())
   {
    $this->e->add("Session switch to backend should have switched to backend", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }

   if($_path["project"]["root"]!=$_path["projects"]."jas/")
   {
    $this->e->add("Session switch to backend should have updated the path", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }

   if($_session->key!="jas")
   {
    $this->e->add("Session switch to backend should have updated the session key", "error");
    $_session->switchBackend(false); //switch to frontend
    return UT_FAILED;
   }


   $_session->switchBackend(false); //switch to frontend
   return UT_OK;
  }


  function SetBackendProject()
  {
   global $_session;

   if(!$_session->backend())
   {
    $this->e->add("Session switch to backend shouldn't fail", "error");
    $_session->frontend();
    return UT_FAILED;
   }

   if($project=$_session->get("backendProject"))
   {
    if(!$_session->setBackendProject(false)) //no backend project
    {
     $this->e->add("Session unset backend project shouldn't fail", "error");
     $_session->setBackendProject($project);
     $_session->frontend();
     return UT_FAILED;
    }

    if(!$_session->setBackendProject($project)) //reset backend project
    {
     $this->e->add("Session reset backend project shouldn't fail", "error");
     $_session->setBackendProject($project);
     $_session->frontend();
     return UT_FAILED;
    }
   }
   else //no backend project set
   {
    if(!$_session->setBackendProject("jas")) //set backend project
    {
     $this->e->add("Session reset backend project shouldn't fail", "error");
     $_session->setBackendProject($project);
     $_session->frontend();
     return UT_FAILED;
    }

    if(!$_session->setBackendProject(false)) //no backend project
    {
     $this->e->add("Session unset backend project shouldn't fail", "error");
     $_session->setBackendProject($project);
     $_session->frontend();
     return UT_FAILED;
    }
   }

   $_session->frontend();
   return UT_OK;
  }

 }

?>
