<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/stp.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."stp.php");

 class UT_STP extends UT_Classes
 {
  function Constructor()
  {
   $object=new STP();

   if(!is_object($object)||get_class($object)!="STP")
   {
    $this->e->add("Constructor doesn't return a STP object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   if($object->name!="_stp")
   {
    $this->e->add("Variable: \"name\" doesn't match", "error");
    return UT_FAILED;
   }

   if($object->maxTokens!=10)
   {
    $this->e->add("Variable: \"maxTokens\" doesn't match", "error");
    return UT_FAILED;
   }

   if(!is_array($object->tokens))
   {
    $this->e->add("Variable: \"tokens\" isn't array", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new STP();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("STP object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->name=Array();
   if($orig->name==$copy->name)
   {
    $this->e->add("STP object copy also modified original \"name\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("STP object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  /*
   * Generate
   */

  function Generate()
  {
   $stp=new STP();

   if(!($token=$stp->generate()))
   {
    $this->e->add("STP generate shouldn't fail", "error");
    $this->e->add($stp->e);
    return UT_FAILED;
   }

   if(!preg_match("/^[0-F]{32}$/", $token))
   {
    $this->e->add("STP generated token incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function NewToken()
  {
   global $_session;

   $_tokens=$_session->get("stp.tokens"); //store current tokens

   $stp=new STP();

   if(!($token=$stp->newToken()))
   {
    $this->e->add("STP new token shouldn't fail", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   if(!preg_match("/^[0-F]{32}$/", $token))
   {
    $this->e->add("STP generated token incorrect", "error");

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   $_session->set("stp.tokens", $_tokens); //restore tokens
   return UT_OK;
  }


  /*
   * Name
   */

  function GetName()
  {
   $stp=new STP();

   if(!($name=$stp->getName()))
   {
    $this->e->add("STP get name shouldn't fail", "error");
    $this->e->add($stp->e);
    return UT_FAILED;
   }

   if($name!="_stp")
   {
    $this->e->add("STP name incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  /*
   * Current
   */

  function GetCurrent()
  {
   global $_session;

   $_tokens=$_session->get("stp.tokens"); //store current tokens

   $stp=new STP();

   if(!($token=$stp->getCurrent()))
   {
    $this->e->add("STP get current shouldn't fail", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   if(!preg_match("/^[0-F]{32}$/", $token))
   {
    $this->e->add("STP current token incorrect", "error");

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }


//new token
   if(!($token=$stp->newToken()))
   {
    $this->e->add("STP new token shouldn't fail", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   if($stp->getCurrent()!=$token)
   {
    $this->e->add("STP get current doesn't match new token", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   $_session->set("stp.tokens", $_tokens); //restore tokens
   return UT_OK;
  }

  function GetCurrentURL()
  {
   global $_session;

   $_tokens=$_session->get("stp.tokens"); //store current tokens

   $stp=new STP();

   if(!($url=$stp->getCurrentURL()))
   {
    $this->e->add("STP get current URL shouldn't fail", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   if(!preg_match("/^_stp=[0-F]{32}$/", $url))
   {
    $this->e->add("STP current url incorrect", "error");

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }


//new token
   if(!($token=$stp->newToken()))
   {
    $this->e->add("STP new token shouldn't fail", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   if($stp->getCurrentURL()!="_stp=".$token)
   {
    $this->e->add("STP get current url doesn't match new token", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   $_session->set("stp.tokens", $_tokens); //restore tokens
   return UT_OK;
  }

  function GetCurrentInput()
  {
   global $_session;

   $_tokens=$_session->get("stp.tokens"); //store current tokens

   $stp=new STP();

   if(!($input=$stp->getCurrentInput()))
   {
    $this->e->add("STP get current input shouldn't fail", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   if(!preg_match("/^<input type=\"hidden\" name=\"_stp\" value=\"[0-F]{32}\"\/>$/", $input))
   {
    $this->e->add("STP current input incorrect", "error");

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }


//new token
   if(!($token=$stp->newToken()))
   {
    $this->e->add("STP new token shouldn't fail", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   if($stp->getCurrentInput()!="<input type=\"hidden\" name=\"_stp\" value=\"".$token."\"/>")
   {
    $this->e->add("STP get current input doesn't match new token", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   $_session->set("stp.tokens", $_tokens); //restore tokens
   return UT_OK;
  }


  function UpdateCurrent()
  {
   global $_session;

   $_tokens=$_session->get("stp.tokens"); //store current tokens

   $stp=new STP();

   $text="
<a href=\"?_stp=0123456789ABCDEF0123456789ABCDEF&test=1\">Testing</a>
<input name=\"_stp\" value=\"0123456789ABCDEF0123456789ABCDEF\">
";
   if(!($output=$stp->updateCurrent($text)))
   {
    $this->e->add("STP update current shouldn't fail", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   if($output!=str_replace("0123456789ABCDEF0123456789ABCDEF", $stp->getCurrent(), $text))
   {
    $this->e->add("STP update current output incorrect", "error");

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

//incorrect stp
   $text="
<a href=\"?_stp=0123456789ABCDEF&test=1\">Testing</a>
<input name=\"_stp\" value=\"0123456789abcdef0123456789abcdef\">
";
   if(!($output=$stp->updateCurrent($text)))
   {
    $this->e->add("STP update current shouldn't fail", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   if($output!=$text)
   {
    $this->e->add("STP update current output incorrect", "error");

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   $_session->set("stp.tokens", $_tokens); //restore tokens
   return UT_OK;
  }


  /*
   * Validate
   */

  function Check()
  {
   global $_session;

   $_tokens=$_session->get("stp.tokens"); //store current tokens

   $stp=new STP();

   if(!($token=$stp->getCurrent()))
   {
    $this->e->add("STP get current shouldn't fail", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   $_GET["_stp"]=$token;
   if(!$stp->check())
   {
    $this->e->add("STP check shouldn't fail", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    unset($_GET["_stp"]);
    return UT_FAILED;
   }
   unset($_GET["_stp"]);

   if(!$stp->check($token))
   {
    $this->e->add("STP check shouldn't fail", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   if($stp->tokens[$token]!==false)
   {
    $this->e->add("STP token should be marked as unused", "error");
    return UT_FAILED;
   }


//new token
   if(!($token=$stp->newToken()))
   {
    $this->e->add("STP new token shouldn't fail", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   $_GET["_stp"]=$token;
   if(!$stp->check())
   {
    $this->e->add("STP check shouldn't fail", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    unset($_GET["_stp"]);
    return UT_FAILED;
   }
   unset($_GET["_stp"]);

   if(!$stp->check($token))
   {
    $this->e->add("STP check shouldn't fail", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   if($stp->tokens[$token]!==false)
   {
    $this->e->add("STP token should be marked as unused", "error");
    return UT_FAILED;
   }


//old token
   if(!($token=reset(array_keys($stp->tokens))))
   {
    $this->e->add("STP can't get old token", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   $_GET["_stp"]=$token;
   if(!$stp->check())
   {
    $this->e->add("STP check shouldn't fail", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    unset($_GET["_stp"]);
    return UT_FAILED;
   }
   unset($_GET["_stp"]);

   if(!$stp->check($token))
   {
    $this->e->add("STP check shouldn't fail", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   if($stp->tokens[$token]!==true)
   {
    $this->e->add("STP token should be marked as used", "error");
    return UT_FAILED;
   }


//fail
   if($stp->check())
   {
    $this->e->add("STP check shouldn't fail", "error");
    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   if($stp->check("asdasdasd"))
   {
    $this->e->add("STP check shouldn't fail", "error");
    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   $_session->set("stp.tokens", $_tokens); //restore tokens
   return UT_OK;
  }


  /*
   * Clean up
   */

  function CleanUp()
  {
   global $_session;

   $_tokens=$_session->get("stp.tokens"); //store current tokens

   $stp=new STP();

//no cleanup
   $tokens=Array();
   $stp->tokens=Array();
   for($c=0;$c<$stp->maxTokens;$c++)
   {
    if(!($token=$stp->newToken()))
    {
     $this->e->add("STP new token shouldn't fail", "error");
     $this->e->add($stp->e);

     $_session->set("stp.tokens", $_tokens); //restore tokens
     return UT_FAILED;
    }

    $tokens[$token]=false;
   }

   if(!$this->_compare($tokens, $stp->tokens))
   {
    $this->e->add("STP new tokens list incorrect", "error");
    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   if(!$stp->cleanUp())
   {
    $this->e->add("STP clean up shouldn't fail", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   if(!$this->_compare($tokens, $stp->tokens))
   {
    $this->e->add("STP clean up incorrect", "error");
    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }


//used tokens
   $token=array_keys($tokens)[3];
   unset($tokens[$token]);
   if(!$stp->check($token))
   {
    $this->e->add("STP check token shouldn't fail", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   if(!$stp->cleanUp())
   {
    $this->e->add("STP clean up shouldn't fail", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   if(!$this->_compare($tokens, $stp->tokens))
   {
    $this->e->add("STP clean up incorrect", "error");
    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }


//used tokens (current)
   $token=array_keys($tokens)[8];
   if(!$stp->check($token))
   {
    $this->e->add("STP check token shouldn't fail", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   if(!$stp->cleanUp())
   {
    $this->e->add("STP clean up shouldn't fail", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   if(!$this->_compare($tokens, $stp->tokens))
   {
    $this->e->add("STP clean up incorrect", "error");
    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }


//old tokens
   $tokens=Array(); //clear expected output
   for($c=0;$c<$stp->maxTokens;$c++)
   {
    if(!($token=$stp->newToken()))
    {
     $this->e->add("STP new token shouldn't fail", "error");
     $this->e->add($stp->e);

     $_session->set("stp.tokens", $_tokens); //restore tokens
     return UT_FAILED;
    }

    $tokens[$token]=false;
   }

   if(!$stp->cleanUp())
   {
    $this->e->add("STP clean up shouldn't fail", "error");
    $this->e->add($stp->e);

    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   if(!$this->_compare($tokens, $stp->tokens))
   {
    $this->e->add("STP clean up incorrect", "error");
    $_session->set("stp.tokens", $_tokens); //restore tokens
    return UT_FAILED;
   }

   $_session->set("stp.tokens", $_tokens); //restore tokens
   return UT_OK;
  }

 }

?>
