<?

 /*****************************************************************************
 * Filename: include/storage/sql.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."storage/storage.php");

 define("ALTER_FIELD_TYPE", 1);
 define("ALTER_FIELD_SIZE", 3);
 define("ALTER_FIELD_OPTIONS", 2);

 class System_Storage_sql extends Storage
 {
  function __construct(&$d, $type)
  {
   parent::__construct($d, $type);

   $this->dbcon=false;

   $this->engine=false;
   $this->collation=false;
   $this->charset=false;
   $this->enum_type=false;

   $this->setQuotes("\"", "`", "`", "`");

   $this->clear();

   $this->crossConvert=Array(
    "int" => Array("float", "varchar", "text", "bigint"),
    "float" => Array("int", "varchar", "text"),
    "varchar" => Array("tinytext", "text", "mediumtext", "longtext"),
    "tinytext" => Array("text", "mediumtext", "longtext"),
    "text" => Array("mediumtext", "longtext"),
    "mediumtext" => Array("longtext"),
    "tinyblob" => Array("blob", "mediumblob", "longblob"),
    "blob" => Array("mediumblob", "longblob"),
    "mediumblob" => Array("longblob"),
    "date" => Array("datetime"),
    "time" => Array("datetime"),
   );
  }


  function clear()
  {
   $this->txt=NULL;
   $this->sql=NULL;

   return parent::clear();
  }


  function setQuotes($values, $tableName, $fieldName, $indexName)
  {
   $this->qv=$values;
   $this->qt=$tableName;
   $this->qf=$fieldName;
   $this->qi=$indexName;

   return true;
  }


  function escape($text, $blob=false)
  {
   return addslashes($text);
  }


  function exec($sql, $txt=false)
  {
   //d($txt, $sql);

   if($this->e->any("fatal"))
    return false;

   if($this->type=="fake")
    return true;

   $this->sql=$sql;
   $this->txt=$txt;

   $this->results=NULL;

   if(!$this->dbcon)
   {
    $this->e->add("SQL error: Database connection not set up", "system", false, ($this->data->module!="log")); //don't log
    return false;
   }

   return true;
  }


  function rightsField($action=false)
  {
   global $_config;

   $d=&$this->data;

   if($action==false)
    $action=$d->action;

   $sql="";

//Root user has full rights
   if($_config->userRoot())
    $sql.=$d->getRootRights()." ";
   else
   {
    if($refer=$d->getRightsReference())
     $rightsReference=$d->reference[$refer]["from"];
    else
     $rightsReference=false;

    if($action=="add"||$action=="copy")
    {
     if($rightsReference)
     {
      $sql.="IF((";
      $sql.=$this->qt."categories".$this->qt.".".$this->qf."owner".$this->qf."=".$this->qv.$_config->get("usernr").$this->qv." ";

      $sql.="), ".$d->getRootRights().", ";
     }
     else
      $sql.="(";
    }
    else if($fields=$d->check("properties", "owner"))
    {
//check that first and give the owner full rights
     $sql.="IF((";
     foreach($fields as $k)
      $sql.=$this->qt.$d->field[$k]["module"].$this->qt.".".$this->qf.$k.$this->qf."=".$this->qv.$_config->get("usernr").$this->qv." OR ";

     $sql=substr($sql, 0, -3);

     $sql.="), ".$d->getRootRights().", ";
    }
    else
     $sql.="(";

//No category reference means no rights if
//Meaning no rights when not the owner
    if($rightsReference===false)
     $sql.="0";
    else
    {
     $sql.="IF(".$this->qt."userRights".$this->qt.".".$this->qf."rights".$this->qf." IS NULL, 0, ".$this->qt."userRights".$this->qt.".".$this->qf."rights".$this->qf.") ";

     $l=sizeof($_config->get("groupnrs"));
     for($c=0;$c<$l;$c++)
      $sql.=" | IF(".$this->qt."groupRights".$c.$this->qt.".".$this->qf."rights".$this->qf." IS NULL, 0, ".$this->qt."groupRights".$c.$this->qt.".".$this->qf."rights".$this->qf.")";
    }

    //rights4all
    if($action=="add"||$action=="copy")
    {
     foreach($d->check("properties", "allusers") as $k)
      $sql.="|".$d->field[$k]["defaultvalue"];
    }
    else
    {
     foreach($d->check("properties", "allusers") as $k)
     {
      if($d->check("use", "sql", $k))
       $sql.="|".$this->qt.$d->field[$k]["module"].$this->qt.".".$this->qf.$k.$this->qf;
      else
       $sql.="|".$d->field[$k]["defaultvalue"];
     }
    }

    $sql.=") ";
   }

   return $sql;
  }


  function from($action=false, $modules=false)
  {
   global $_config;

   $d=&$this->data;

   if($action==false)
    $action=$d->action;

   $joins=Array();
   if($modules) //gather joined modules and reference (latter only needed if in where clause)
   {
    $id=$d->getID();
    foreach($modules as $references)
    {
     $d2=&$d;
     $path=$name=$last="";
     foreach(explode(".", $references) as $reference)
     {
      $name=($path?$path.".":"").$reference;

      if(!isset($joins[$name])) //not yet in joins
      {
       if(isset($d2->extends[$reference])||isset($d2->extendedBy[$reference])) //extended module
       {
        $joins[$name]=$this->qt.$reference.$this->qt." AS ".$this->qt.$name.$this->qt." ON ".$this->qt.$d->module.$this->qt.".".$this->qf.$id.$this->qf."=".$this->qt.$name.$this->qt.".".$this->qf.$id.$this->qf." ";
        break; //done with this reference (can't be anything deeper)
       }

       if(!isset($d2->reference[$reference])) //references
       {
        $this->e->add("Unknown reference ".$references." for join", "system");
        return false;
       }

       //determine from table
       if(!$last)
        $from=$d2->field[$d2->reference[$reference]["from"]]["module"];
       else
       {
        $from=$last;

        if($d2->field[$d2->reference[$reference]["from"]]["module"]!=$d2->module) //extended
        {
         if(preg_match("/\.".$d2->module."$/", $from)) //already contains extending module (*:*)
          $from=preg_replace("/\.".$d2->module."$/", ".".$d2->field[$d2->reference[$reference]["from"]]["module"], $from); //replace it with extended module
         else //extended module not yet in referece
          $from.=".".$d2->field[$d2->reference[$reference]["from"]]["module"];
        }
       }

       if($d2->reference[$reference]["type"]=="*:*") //*:*
       {
        //get from fields from m2m module
        $reffrom=$d2->getReferenceData($reference)->check("use", "from");
        $reffrom=$reffrom[0];
        $refto=$d2->getReferenceData($reference)->check("use", "to");
        $refto=$refto[0];

        //add m2m table
        $joins[$name]=$this->qt.$reference.$this->qt." AS ".$this->qt.$name.$this->qt." ON ".$this->qt.$from.$this->qt.".".$this->qf.$d2->reference[$reference]["from"].$this->qf."=".$this->qt.$reference.$this->qt.".".$this->qf.$reffrom.$this->qf." ";

        $name2=$name.".".$d2->getReferenceData($reference)->reference[$refto]["module"];
        $joins[$name2]=$this->qt.$d2->getReferenceData($reference)->reference[$refto]["module"].$this->qt." AS ".$this->qt.$name2.$this->qt." ON ".$this->qt.$name.$this->qt.".".$this->qf.$refto.$this->qf."=".$this->qt.$name2.$this->qt.".".$this->qf.$d2->reference[$reference]["to"].$this->qf." ";

        foreach(array_keys($d2->getReferenceData($reference)->getReferenceData($refto)->extends) as $extends)
         $joins[$name.".".$extends]=$this->qt.$extends.$this->qt." AS ".$this->qt.$name.".".$extends.$this->qt." ON ".$this->qt.$name2.$this->qt.".".$this->qf.$d2->getReferenceData($reference)->getReferenceData($refto)->getID().$this->qf."=".$this->qt.$name.".".$extends.$this->qt.".".$this->qf.$d2->getReferenceData($reference)->getReferenceData($refto)->extends[$extends]->getID().$this->qf." ";
       }
       else //0:*, 1:*, *:0 and *:1
       {
        $joins[$name]=$this->qt.$d2->reference[$reference]["module"].$this->qt." AS ".$this->qt.$name.$this->qt." ON ".$this->qt.$from.$this->qt.".".$this->qf.$d2->reference[$reference]["from"].$this->qf."=".$this->qt.$name.$this->qt.".".$this->qf.$d2->reference[$reference]["to"].$this->qf." ";

        foreach(array_keys($d2->getReferenceData($reference)->extends) as $extends)
         $joins[$name.".".$extends]=$this->qt.$extends.$this->qt." AS ".$this->qt.$name.".".$extends.$this->qt." ON ".$this->qt.$name.$this->qt.".".$this->qf.$d2->getReferenceData($reference)->getID().$this->qf."=".$this->qt.$name.".".$extends.$this->qt.".".$this->qf.$d2->getReferenceData($reference)->extends[$extends]->getID().$this->qf." ";
       }
      }

      $last=$name;
      $path.=($path?".":"").$reference;
      $d2=&$d2->getReferenceData($reference);
     }

     unset($d2);
    }
   }

   if($_config->userRoot()||!($refer=$d->getRightsReference()))
   {
    $sql="FROM ".$this->qt.$d->module.$this->qt." ";

    if($joins) //add joined modules
     $sql.="LEFT JOIN ".implode(" LEFT JOIN ", $joins);
   }
   else
   {
    $rightsReference=$d->reference[$refer]["from"];

    if($action=="add")
     $sql="FROM ".$this->qt."categories".$this->qt." ";
    else
    {
     $sql="FROM ".$this->qt.$d->module.$this->qt." ";

     if($action=="copy"&&$d->module!="categories")
      $sql.="LEFT JOIN ".$this->qt."categories".$this->qt." ON ".$this->qt."categories".$this->qt.".".$this->qf."nr".$this->qf."=".$this->qt.$d->field[$rightsReference]["module"].$this->qt.".".$this->qf.$rightsReference.$this->qf." ";
    }

    if($joins) //add joined modules
     $sql.="LEFT JOIN ".implode(" LEFT JOIN ", $joins);

    if($d->module!="userRights")
    {
     $sql.="LEFT JOIN ".$this->qt."userRights".$this->qt." ";

     if($d->module=="categories"||isset($d->extends["categories"])||$action=="add"||$action=="copy")
      $sql.="ON ".$this->qt."userRights".$this->qt.".".$this->qf."catnr".$this->qf."=".$this->qt."categories".$this->qt.".".$this->qf."nr".$this->qf." ";
     else
      $sql.="ON ".$this->qt."userRights".$this->qt.".".$this->qf."catnr".$this->qf."=".$this->qt.$d->field[$rightsReference]["module"].$this->qt.".".$this->qf.$rightsReference.$this->qf." ";

     $sql.="AND ".$this->qt."userRights".$this->qt.".".$this->qf."nr".$this->qf."=".$this->qv.$_config->get("usernr").$this->qv." ";
    }

    $l=sizeof($_config->get("groupnrs"));
    for($c=0;$c<$l;$c++)
    {
     $sql.="LEFT JOIN ".$this->qt."groupRights".$this->qt." AS ".$this->qt."groupRights".$c.$this->qt." ";

     if($d->module=="categories"||isset($d->extends["categories"])||$action=="add"||$action=="copy")
      $sql.="ON ".$this->qt."groupRights".$c.$this->qt.".".$this->qf."catnr".$this->qf."=".$this->qt."categories".$this->qt.".".$this->qf."nr".$this->qf." ";
     else
      $sql.="ON ".$this->qt."groupRights".$c.$this->qt.".".$this->qf."catnr".$this->qf."=".$this->qt.$d->field[$rightsReference]["module"].$this->qt.".".$this->qf.$rightsReference.$this->qf." ";

     $sql.="AND ".$this->qt."groupRights".$c.$this->qt.".".$this->qf."nr".$this->qf."=".$this->qv.$_config->get("groupnrs.".$c).$this->qv." ";
    }
   }

   return $sql;
  }


  function where($action=false)
  {
   global $_config;

   $d=&$this->data;

   if($action==false)
    $action=$d->action;

   if($_config->userRoot())
    $sql="WHERE ";
   else
   {
    if($d->module=="categories"||$d->module=="userRights"||$d->module=="groupRights")
     $rights=pow(2, $d->rights["category"][$action]);
    else
     $rights=pow(2, $d->rights["content"][$action]);

    $sql="WHERE (".$this->rightsField($action)." & ".$rights.")>0 AND ";
   }

   return $sql;
  }


  function order($skipReferences=false)
  {
   $d=&$this->data;

   $found=0;
   $sql="ORDER BY ";
   foreach($d->sortOrder as $k)
   {
    if(preg_match("/^(.*)(\-|\+)$/", $k, $field))
    {
     if(isset($d->reference[$field[1]])) //reference
     {
      if($d->reference[$field[1]]["type"]=="*:*") //*:* reference
      {
       if($skipReferences)
        continue;

       $dtmp=$d->getReferenceData($field[1]);
       $refto=$dtmp->check("use", "to");
       $refto=$refto[0];
       $dtmp=$dtmp->getReferenceData($refto); //to module

       if(!$dtmp->sortOrder) //no sorting in reference
        continue;

       foreach($dtmp->sortOrder as $k2)
       {
        if(preg_match("/^(.*)(\-|\+)$/", $k2, $field2))
        {
         if($dtmp->check("use", "sql", $field2[1]))
         {
          if($dtmp->field[$field2[1]]["module"]!=$dtmp->module) //extended
           $sql.=$this->qt.$field[1].".".$dtmp->field[$field2[1]]["module"].$this->qt.".".$this->qf.$field2[1].$this->qf." ".($field2[2]=="-"?"DESC, ":"ASC, ");
          else
           $sql.=$this->qt.$field[1].".".$dtmp->field[$field2[1]]["module"].$this->qt.".".$this->qf.$field2[1].$this->qf." ".($field2[2]=="-"?"DESC, ":"ASC, ");

          $found++;
         }
        }
       }

       unset($dtmp);
      }
      else if($d->reference[$field[1]]["type"]=="0:*"||$d->reference[$field[1]]["type"]=="1:*") //0:* and 1:* references
      {
       if($skipReferences)
        continue;

       $dtmp=$d->getReferenceData($field[1]);

       if(!$dtmp->sortOrder) //no sorting in reference
        continue;

       foreach($dtmp->sortOrder as $k2)
       {
        if(preg_match("/^(.*)(\-|\+)$/", $k2, $field2))
        {
         if($dtmp->check("use", "sql", $field2[1]))
         {
          if($dtmp->field[$field2[1]]["module"]!=$dtmp->module) //extended
           $sql.=$this->qt.$field[1].".".$dtmp->field[$field2[1]]["module"].$this->qt.".".$this->qf.$field2[1].$this->qf." ".($field2[2]=="-"?"DESC, ":"ASC, ");
          else
           $sql.=$this->qt.$field[1].$this->qt.".".$this->qf.$field2[1].$this->qf." ".($field2[2]=="-"?"DESC, ":"ASC, ");

          $found++;
         }
        }
       }

       unset($dtmp);
      }
      else //*:0 and *:1 references
      {
       $dtmp=$d->getReferenceData($field[1]);

       if(!$dtmp->sortOrder||$skipReferences) //no sorting in reference or references skipped
       {
        //sort on field in "local" table: *:0 and *:1 allows reference and field to have the same name so if the reference doesn't have sort fields the local field is assumed
        if($d->check("use", "sql", $field[1]))
        {
         $sql.=$this->qt.$d->field[$field[1]]["module"].$this->qt.".".$this->qf.$field[1].$this->qf." ";
         $sql.=($field[2]=="-"?"DESC, ":"ASC, ");
         $found++;
        }

        continue;
       }

       foreach($dtmp->sortOrder as $k2)
       {
        if(preg_match("/^(.*)(\-|\+)$/", $k2, $field2))
        {
         if($dtmp->check("use", "sql", $field2[1]))
         {
          if($dtmp->field[$field2[1]]["module"]!=$dtmp->module) //extended
           $sql.=$this->qt.$field[1].".".$dtmp->field[$field2[1]]["module"].$this->qt.".".$this->qf.$field2[1].$this->qf." ".($field2[2]=="-"?"DESC, ":"ASC, ");
          else
           $sql.=$this->qt.$field[1].$this->qt.".".$this->qf.$field2[1].$this->qf." ".($field2[2]=="-"?"DESC, ":"ASC, ");

          $found++;
         }
        }
       }

       unset($dtmp);
      }
     }
     else if($d->check("use", "sql", $field[1]))
     {
      $sql.=$this->qt.$d->field[$field[1]]["module"].$this->qt.".".$this->qf.$field[1].$this->qf." ";
      $sql.=($field[2]=="-"?"DESC, ":"ASC, ");
      $found++;
     }
     else if($d->action=="search"&&$d->extra=="index"&&isset($d->fulltextIndex[$field[1]])) //full text index
     {
      if($d->check("use", "show", $field[1])) //already in show fields
       $sql.=$this->qf.$field[1].$this->qf." "; //use index name
      else //not shown
       $sql.="MATCH (".$this->qt.$d->module.$this->qt.".".$this->qf.implode($this->qf.", ".$this->qt.$d->module.$this->qt.".".$this->qf, $d->fulltextIndex[$field[1]]).$this->qf.") AGAINST (".$this->qv.$this->escape($d->sqlValue("search")).$this->qv.") "; //use full match clause

      $sql.=($field[2]=="-"?"DESC, ":"ASC, ");
      $found++;
     }
    }
   }

   if($found)
    return substr($sql, 0, -2)." ";
   else
    return "";
  }


  function checkRights($action)
  {
   global $_config;

   if($_config->userRoot())
    return true;

   $d=&$this->data;

   $txt="Checking ".$action." rights: ".$d->title;

   switch($action)
   {
    case "add":
    case "copy":
    {
     $sql="SELECT ".$this->rightsField($action)."AS r ";

     if($refer=$d->getRightsReference())
     {
      $rightsReference=$d->reference[$refer]["from"];
      $sql.=$this->from($action);
      $sql.=$this->where($action)."(".$this->qt."categories".$this->qt.".".$this->qf."nr".$this->qf."=".$this->qv.$this->escape($d->sqlValue($rightsReference)).$this->qv.") ";
     }
     else //no rights reference
     {
      if($this->exec("SHOW TABLE STATUS LIKE ".$this->qv.$d->module.$this->qv, "Show table status")&&$this->next()&&!$this->res["Rows"]) //no rows in table
      {
       //check rights against rights fields, without using a query
       if($d->module=="categories"||$d->module=="userRights"||$d->module=="groupRights")
        $rights=pow(2, $d->rights["category"][$action]);
       else
        $rights=pow(2, $d->rights["content"][$action]);

       $userrights=0;
       foreach($d->check("properties", "allusers") as $k)
        $userrights|=$d->field[$k]["defaultvalue"];

       if(!($rights&$userrights))
       {
        $this->e->add("No ".$action." rights!", "hack");
        return false;
       }

       return true;
      }

      $sql.=$this->from($action);
      $sql.=$this->where($action)."(1) ";
     }

     break;
    }
    case "edit":
    case "delete":
    case "undelete":
    case "owner":
    case "rights":
    {
     $where="";
     $modules=Array();
     foreach($d->check("properties", "identifier") as $k)
     {
      if(!$d->check("valuetype", "none", $k))
      {
       $where.=$this->qt.$d->field[$k]["module"].$this->qt.".".$this->qf.$k.$this->qf."=".$this->qv.$this->escape($d->sqlValue($k)).$this->qv." AND ";

       if($d->field[$k]["module"]!=$d->module&&!in_array($d->field[$k]["module"], $modules))
        $modules[]=$d->field[$k]["module"];
      }

      //filter out deleted items (or non deleted items for "undelete" action)
      foreach($d->check("properties", "deleted") as $k)
      {
       if($d->check("use", "sql", $k))
       {
        if($d->field[$k]["module"]!=$d->module&&!in_array($d->field[$k]["module"], $modules))
         $modules[]=$d->field[$k]["module"];

        $where.=$this->qt.$d->field[$k]["module"].$this->qt.".".$this->qf.$k.$this->qf."=".$this->qv.($action=="undelete"?"true":"false").$this->qv." AND ";
       }
      }
     }

//select
     $sql="SELECT ".$this->rightsField($action)."AS r ";

//from
     $sql.=$this->from($action, $modules);

//where options
     $sql.=$this->where($action).$where;

     $sql=substr($sql, 0, -4);
     break;
    }
    default:
    {
     $this->e->add("No ".$action." rights!", "hack");
     return false;
    }
   }

   $sql.="LIMIT 0,1";

   if(!$this->exec($sql, $txt))
    return false;

   if(!$this->next())
   {
    if($action=="rights")
     $this->e->add("Not allowed to change rights!", "hack");
    else
     $this->e->add("No ".$action." rights!", "hack");

    return false;
   }

   return true;
  }


  function _table()
  {
   $d=&$this->data;
   $sql="SHOW TABLES LIKE ".$this->qv.$d->module.$this->qv;
   return $sql;
  }

  function table()
  {
   $d=&$this->data;
   $txt="Table exists: ".$d->title;

   if(!($sql=$this->_table()))
    return false;

//execute
   if(!$this->exec($sql, $txt))
    return false;

   if($this->next())
    return true;

   return false;
  }


  function info()
  {
   global $_config;

   $d=&$this->data;

   if(!$this->exec("SHOW TABLE STATUS WHERE Name=".$this->qv.$d->module.$this->qv, "Show table status"))
    return false;

   $results=Array();
   if($this->next())
   {
    $results["info_type"]=$this->res["Engine"]." ".$this->res["Version"];

    $results["info_rows"]=$this->res["Rows"];
    $results["info_insertID"]=$this->res["Auto_increment"];

    //look up SQL timezone offset
    $current=date_default_timezone_get();
    if($current!=$_config->get("sqltimezone"))
    {
     date_default_timezone_set($_config->get("sqltimezone"));
     $timezoneOffset=date("P");
     date_default_timezone_set($current);
    }
    else
     $timezoneOffset=date("P");

    //convert time zone to UTC
    if(!$this->exec("SELECT CONVERT_TZ('".$this->res["Create_time"]."', @@session.time_zone, '".$timezoneOffset."') as Create_time,  CONVERT_TZ('".$this->res["Update_time"]."', @@session.time_zone, '".$timezoneOffset."') as Update_time", "Convert create/update timezones"))
     return false;

    if($this->next())
    {
     $results["info_created"]=$this->res["Create_time"];
     $results["info_updated"]=$this->res["Update_time"];
    }

    $this->clear();
    $this->type="fake";
    $this->results=Array($results);

    return true;
   }

   return false;
  }


  function _get($gather)
  {
   global $_config;

   $d=&$this->data;

   $id=$d->getID();

   if($d->check("valuetype", "set", "aggregation_field"))
   {
    $agg_field=$d->getValue("aggregation_field");
    $agg_type=($d->getValue("aggregation_type")=="average"?"AVG":strtoupper($d->getValue("aggregation_type")));
    $agg_method=$d->getValue("aggregation_method");

    if($agg_method=="before")
     $agg_method="CEIL";
    else if($agg_method=="after")
     $agg_method="FLOOR";
    else
     $agg_method="ROUND";

    $agg_interval=$d->getValue("aggregation_interval");
   }
   else
   {
    $agg_field=false;
    $agg_type=false;
    $agg_method=false;
    $agg_interval=false;
   }

//fields
   $select="SELECT ";
   $modules=Array();

   if($gather=="count"&&!$agg_field)
    $select.="COUNT(*) AS ".$this->qf."show_count".$this->qf." ";
   else
   {
    $select.="DISTINCT "; //make sure rows are returned only once

    if($id)
     $select.=$this->qt.$d->module.$this->qt.".".$this->qf.$id.$this->qf.", ";

    foreach($d->check("use", "sql") as $k)
    {
     if($k==$id) //skip id
      continue;
     else if($k=="r"&&$d->check("type", "rights", $k))
      $select.=$this->rightsField()."AS r, ";
     else if($d->check("use", "show", $k)||$d->check("type", "rights", $k)||(!$agg_type&&$agg_field==$k)) //also include aggregation field for simple grouping other wise group fails
     {
      if(isset($d->fulltextIndex[$k])) //full text index
      {
       if($d->action=="search"&&$d->extra=="index") //only for index search
        $select.="MATCH (".$this->qt.$d->module.$this->qt.".".$this->qf.implode($this->qf.", ".$this->qt.$d->module.$this->qt.".".$this->qf, $d->fulltextIndex[$k]).$this->qf.") AGAINST (".$this->qv.$this->escape($d->sqlValue("search")).$this->qv.") AS ".$this->qf.$k.$this->qf.", "; //add match clause as index name column
      }

      if(!$agg_field) //no aggregation
       $select.=$this->qt.$d->field[$k]["module"].$this->qt.".".$this->qf.$k.$this->qf.", ";
      else if(!$agg_type) //simple aggregation
       $select.=$this->qt.$d->field[$k]["module"].$this->qt.".".$this->qf.$k.$this->qf.", ";
      else if($agg_field==$k) //aggregate by $agg_field field
      {
       if(!$agg_interval||$agg_interval==1) //no interval based aggregation
        $select.=$this->qt.$d->field[$k]["module"].$this->qt.".".$this->qf.$k.$this->qf.", ";
       else //interval based aggregation, round(value/interval)*interval
       {
        $type=$d->field[$agg_field]["type"];

        if($type=="date"||$type=="time"||$type=="datetime") //convert date/time before conversion
         $select.="FROM_UNIXTIME(".$agg_method."(UNIX_TIMESTAMP(".$this->qt.$d->field[$agg_field]["module"].$this->qt.".".$this->qf.$agg_field.$this->qf.")/".$agg_interval.")*".$agg_interval.") AS ".$this->qf.$k.$this->qf.", ";
        else //all other fields
         $select.=$agg_method."(".$this->qt.$d->field[$agg_field]["module"].$this->qt.".".$this->qf.$agg_field.$this->qf."/".$agg_interval.")*".$agg_interval." AS ".$this->qf.$k.$this->qf.", ";
       }
      }
      else //apply aggregation type to fields
      {
       $type=$d->field[$k]["type"];

       if($agg_type=="COUNT") //use field to set count value
        $select.=$agg_type."(".$this->qt.$d->field[$k]["module"].$this->qt.".".$this->qf.$k.$this->qf.") AS ".$this->qf.$k.$this->qf.", ";
       else if(($type=="date"||$type=="time"||$type=="datetime")&&$agg_type!="SUM") //no sum for date/time
        $select.="FROM_UNIXTIME(".$agg_type."(UNIX_TIMESTAMP(".$this->qt.$d->field[$k]["module"].$this->qt.".".$this->qf.$k.$this->qf."))) AS ".$this->qf.$k.$this->qf.", ";
       else if($type=="float"||$type=="price")
        $select.=$agg_type."(".$this->qt.$d->field[$k]["module"].$this->qt.".".$this->qf.$k.$this->qf.") AS ".$this->qf.$k.$this->qf.", ";
       else if($type=="nr") //nr, only non references
       {
        //find reference link
        $found=false;
        foreach(array_keys($d->reference) as $ref)
         if($d->reference[$ref]["from"]==$k)
          $found=true;

        if(!$found) //normal nr
         //$select.="ROUND(".$agg_type."(".$this->qt.$d->field[$k]["module"].$this->qt.".".$this->qf.$k.$this->qf.")) AS ".$this->qf.$k.$this->qf.", ";
         $select.=$agg_type."(".$this->qt.$d->field[$k]["module"].$this->qt.".".$this->qf.$k.$this->qf.") AS ".$this->qf.$k.$this->qf.", "; //don't round average
        else //don't apply to references
         $select.=$this->qt.$d->field[$k]["module"].$this->qt.".".$this->qf.$k.$this->qf.", ";
       }
       else //normal, field aggregation not possible
        $select.=$this->qt.$d->field[$k]["module"].$this->qt.".".$this->qf.$k.$this->qf.", ";
      }

      if($d->field[$k]["module"]!=$d->module&&!in_array($d->field[$k]["module"], $modules))
       $modules[]=$d->field[$k]["module"];
     }
     else if($agg_field) //not a show/aggregation field but could be needed in order by (since the table is obscured by the group by when aggregating)
     {
      foreach($d->sortOrder as $sortField) //chec
      {
       if(preg_match("/^$k(\-|\+)$/", $sortField)) //field in sort field, add to select
       {
        $select.=$this->qt.$d->field[$k]["module"].$this->qt.".".$this->qf.$k.$this->qf.", ";

        if($d->field[$k]["module"]!=$d->module&&!in_array($d->field[$k]["module"], $modules))
         $modules[]=$d->field[$k]["module"];

        break;
       }
      }
     }
    }

    $select=substr($select, 0, -2);
   }

//add full text index fields for index search
   if($d->action=="search"&&$d->extra=="index")
   {
    foreach(array_keys($d->fulltextIndex) as $index)
     if($d->check("use", "show", $index))
      $select.=", MATCH (".$this->qt.$d->module.$this->qt.".".$this->qf.implode($this->qf.", ".$this->qt.$d->module.$this->qt.".".$this->qf, $d->fulltextIndex[$index]).$this->qf.") AGAINST (".$this->qv.$this->escape($d->sqlValue("search")).$this->qv.") AS ".$this->qf.$index.$this->qf; //add match clause as index name column
   }

//where options
   $where=$this->where()."(";

   switch($d->extra)
   {
//Get by identifier (view, edit, etc)
    case "item":
    {
     $fields=Array();

     foreach($d->check("properties", "identifier") as $k)
     {
      if(!$d->check("valuetype", "none", $k))
      {
       $fields[]=$k;

       if($d->field[$k]["module"]!=$d->module&&!in_array($d->field[$k]["module"], $modules))
        $modules[]=$d->field[$k]["module"];
      }
     }


     if($d->vmax)
     {
      $l=sizeof($fields);
      for($d->vc=0;$d->vc<$d->vmax;$d->vc++)
      {
       $where.="(";
       for($c=0;$c<$l;$c++)
        $where.=$this->qt.$d->field[$fields[$c]]["module"].$this->qt.".".$this->qf.$fields[$c].$this->qf."=".$this->qv.$this->escape($d->sqlValue($fields[$c], $d->vc)).$this->qv." AND ";

       $where=preg_replace("/\s[A-Z]+\s$/", "", $where).") OR ";
      }

      $where=preg_replace("/\s[A-Z]+\s$/", "", $where);
     }
     else
      $where.="false";

     $d->vc=0;

     break;
    }

//List by category
    case "list":
    {
     $fields=Array();

     if(($refer=$d->getRightsReference())&&($k=$d->reference[$refer]["from"]))
     {
      if($d->check("use", "sql", $k))
      {
       if(!$d->check("valuetype", "none", $k))
       {
        $fields[]=$k;

        if($d->field[$k]["module"]!=$d->module&&!in_array($d->field[$k]["module"], $modules))
         $modules[]=$d->field[$k]["module"];
       }
      }
     }


     if($d->vmax)
     {
      $l=sizeof($fields);
      for($d->vc=0;$d->vc<$d->vmax;$d->vc++)
      {
       $where.="(";
       for($c=0;$c<$l;$c++)
        $where.=$this->qt.$d->field[$fields[$c]]["module"].$this->qt.".".$this->qf.$fields[$c].$this->qf."=".$this->qv.$this->escape($d->sqlValue($fields[$c], $d->vc)).$this->qv." AND ";

       $where=preg_replace("/\s[A-Z]+\s$/", "", $where).") OR ";
      }

      $where=preg_replace("/\s[A-Z]+\s$/", "", $where);
     }
     else
      $where.="false";

     $d->vc=0;

     break;
    }

    case "filter":
    {
     if(!$d->filters)
     {
      $this->e->add("No filters set for module: \"".$d->module."\"", "error");
      return false;
     }

     $depth=0;
     foreach($d->filters as $filter)
     {
      //brackets
      if($diff=$filter["depth"]-$depth)
      {
       if($diff>0) //deeper
        $where.=str_repeat("(", $diff);
       else if($diff<0) //less deep
        $where=substr($where, 0, -1).str_repeat(")", -$diff);

       $depth=$filter["depth"];
      }

      if(isset($filter["operant"])) //operant
       $where.=($diff<0?" ":"").$filter["operant"]." ";
      else //field
      {
       //get field and referenced modules
       $d2=&$d;
       $reference="";
       $l=sizeof($filter["field"])-1;
       for($c=0;$c<$l;$c++)
       {
        $d2=&$d2->getReferenceData($filter["field"][$c]);
        $reference.=($reference?".":"").$filter["field"][$c];
       }

       //add all references to from modules clause
       if($reference&&!in_array($reference, $modules))
        $modules[]=$reference;

       if($reference)
       {
        if($d2->field[$filter["field"][$l]]["module"]!=$d2->module) //extended
        {
         if(preg_match("/\.".$d2->module."$/", $reference)) //already contains extending module (*:*)
          $reference=preg_replace("/\.".$d2->module."$/", ".".$d2->field[$filter["field"][$l]]["module"], $reference); //replace it with extended module
         else //extended module not yet in referece
          $reference.=".".$d2->field[$filter["field"][$l]]["module"];
        }
       }

       //determine values
       if(isset($filter["function"]))
        $values=$filter["arguments"];
       else
        $values=Array($filter);

       foreach($values as $k => $v)
       {
        if(isset($v["value"]))
         $values[$k]=Array($v["value"]);
        else if(isset($v["variable"]))
        {
         if(isset($v["variable"][1])) //entry specified
         {
          if(!($d3=$d->sqlValue($v["variable"][0], $v["variable"][1])))
           $d3="";

          $values[$k]=Array($d3);
          unset($d3);
         }
         else //no entry, get all values
         {
          $values[$k]=Array();
          foreach(array_keys($d->field[$v["variable"][0]]["value"]) as $entry)
           $values[$k][]=$d->sqlValue($v["variable"][0], $entry);
         }
        }
        else if(isset($v["reference"]))
        {
         $d3=&$d;
         $reference2="";
         $l2=sizeof($v["reference"])-1;
         for($c=0;$c<$l2;$c++)
         {
          $d3=&$d3->getReferenceData($v["reference"][$c]);
          $reference2.=($reference2?".":"").$v["reference"][$c];
         }

         //add all references to from modules clause
         if($reference2&&!in_array($reference2, $modules))
          $modules[]=$reference2;

         if($reference2)
         {
          if($d3->field[$v["reference"][$l2]]["module"]!=$d3->module) //extended
          {
           if(preg_match("/\.".$d3->module."$/", $reference2)) //already contains extending module (*:*)
            $reference2=preg_replace("/\.".$d3->module."$/", ".".$d3->field[$v["reference"][$l2]]["module"], $reference2); //replace it with extended module
           else //extended module not yet in referece
            $reference2.=".".$d3->field[$v["reference"][$l2]]["module"];
          }
         }

         $values[$k]=Array("reference" => ($reference2?$reference2:$d3->field[$v["reference"][$l2]]["module"]), "field" => $v["reference"][$l2]);
         unset($d3);
        }
       }

       //set value/function
       if(isset($d2->fulltextIndex[$filter["field"][$l]])) //add match clause for all full text indexes
       {
        $where.="MATCH (".$this->qt.$d2->module.$this->qt.".".$this->qf.implode($this->qf.", ".$this->qt.$d2->module.$this->qt.".".$this->qf, $d2->fulltextIndex[$filter["field"][$l]]).$this->qf.") ";
        $where.="AGAINST (";

        if(array_type($values[0])=="associative") //reference
         $where.=$this->qt.$values[0]["reference"].$this->qt.".".$this->qf.$values[0]["field"].$this->qf;
        else //value, only use first value
         $where.=$this->qv.$this->escape($values[0][0]).$this->qv;

        $where.=")";
       }
       else if(isset($filter["function"]))
       {
        $where.=$this->qt.($reference?$reference:$d2->field[$filter["field"][$l]]["module"]).$this->qt.".".$this->qf.$filter["field"][$l].$this->qf;
        $where.=$this->mapOperator($filter["function"], $prefix, $postfix);

        if($filter["function"]=="in"||$filter["function"]=="not in") //multiple values
        {
         $where.="(";

         foreach($values as $v)
         {
          if(array_type($v)=="associative") //reference
           $where.=$this->qt.$v["reference"].$this->qt.".".$this->qf.$v["field"].$this->qf.", ";
          else //value, only use first value
          {
           foreach($v as $v2)
           {
            if(is_array($v2))
            {
             foreach($v2 as $v3)
              $where.=$this->qv.$prefix.$this->escape($v3).$postfix.$this->qv.", ";
            }
            else
             $where.=$this->qv.$prefix.$this->escape($v2).$postfix.$this->qv.", ";
           }
          }
         }

         $where=substr($where, 0, -2).")";
        }
        else if($filter["function"]=="between"||$filter["function"]=="not between") //value1 AND value2
        {
         if(array_type($values[0])=="associative") //reference
          $where.=$this->qt.$values[0]["reference"].$this->qt.".".$this->qf.$values[0]["field"].$this->qf;
         else //value, only use first value
          $where.=$this->qv.$prefix.$this->escape($values[0][0]).$postfix.$this->qv;

         $where.=" AND ";

         if(array_type($values[1])=="associative") //reference
          $where.=$this->qt.$values[1]["reference"].$this->qt.".".$this->qf.$values[1]["field"].$this->qf;
         else //value, only use first value
          $where.=$this->qv.$prefix.$this->escape($values[1][0]).$postfix.$this->qv;
        }
        else //single value
        {
         if(array_type($values[0])=="associative") //reference
          $where.=$this->qt.$values[0]["reference"].$this->qt.".".$this->qf.$values[0]["field"].$this->qf;
         else //value, only use first value
          $where.=$this->qv.$prefix.$this->escape($values[0][0]).$postfix.$this->qv;
        }
       }
       else //value
       {
        $where.=$this->qt.($reference?$reference:$d2->field[$filter["field"][$l]]["module"]).$this->qt.".".$this->qf.$filter["field"][$l].$this->qf;
        $where.=$this->mapOperator($filter["operator"], $prefix, $postfix);

        if(array_type($values[0])=="associative") //reference
         $where.=$this->qt.$values[0]["reference"].$this->qt.".".$this->qf.$values[0]["field"].$this->qf;
        else //value, only use first value
         $where.=$this->qv.$prefix.$this->escape($values[0][0]).$postfix.$this->qv;
       }

       $where.=" ";
							unset($d2);
      }
     }

     $where=substr($where, 0, -1);

     if($depth)
      $where.=str_repeat(")", $depth);

     break;
    }

//All extra options (view->all, user->login, search: title, field, show, full, index, etc): value type = set
    default:
    {
     $fields=Array();

     foreach($d->check("valuetype", "set") as $k) //values should be set
     {
      if((isset($d->reference[$k])&&($d->reference[$k]["type"]=="0:*"||$d->reference[$k]["type"]=="1:*"||$d->reference[$k]["type"]=="*:*"))) //0:*, 1:* and *:* reference
      {
       $fields[]=$k;

       if(!in_array($k, $modules))
        $modules[]=$k;

       if($d->reference[$k]["type"]=="*:*") //add extended module of from field
        if($d->field[$d->reference[$k]["from"]]["module"]!=$d->module&&!in_array($d->field[$d->reference[$k]["from"]]["module"], $modules))
         $modules[]=$d->field[$d->reference[$k]["from"]]["module"];
      }
      else if($d->check("use", "sql", $k)) //normal fields (include *:0 and *:1 references)
      {
       $fields[]=$k;

       if($d->field[$k]["module"]!=$d->module&&!in_array($d->field[$k]["module"], $modules))
        $modules[]=$d->field[$k]["module"];
      }
     }

     if($d->action=="search"&&$d->extra=="index") //add full text index fields for index search
     {
      foreach(array_keys($d->fulltextIndex) as $index)
       if($d->check("valuetype", "set", $index))
        $fields[]=$index;
     }

     $l=sizeof($fields);

     $any=false;
     $open=false;
     for($d->vc=0;$d->vc<$d->vmax;$d->vc++)
     {
      for($c=0;$c<$l;$c++)
      {
       if(isset($d->field[$fields[$c]]["value"][$d->vc])&&isset($d->field[$fields[$c]]["operant"][$d->vc]))
       {
        if(!$open)
        {
         $where.="(";
         $open=true;
        }

        $prefix="";
        $postfix="";

        if($fields[$c]=="r")
         $where.="(".$this->rightsField($d->action)." ".$this->mapOperator($d->getOperator($fields[$c], $d->vc), $prefix, $postfix)." ".$this->escape($d->sqlValue($fields[$c], $d->vc)).")>0 ".$d->getOperant($fields[$c], $d->vc)." ";
        else if(isset($d->fulltextIndex[$fields[$c]])) //add match clause for all full text indexes
         $where.="MATCH (".$this->qt.$d->module.$this->qt.".".$this->qf.implode($this->qf.", ".$this->qt.$d->module.$this->qt.".".$this->qf, $d->fulltextIndex[$fields[$c]]).$this->qf.") AGAINST (".$this->qv.$this->escape($d->sqlValue($fields[$c], $d->vc)).$this->qv.") ".$d->getOperant($fields[$c], $d->vc)." ";
        else
        {
         if(isset($d->reference[$fields[$c]])&&($d->reference[$fields[$c]]["type"]=="*:*"))
         {
          //get from/to fields from m2m module
          $refto=$d->reference[$fields[$c]]["data"]->check("use", "to");
          $refto=$refto[0];

          $where.=$this->qt.$fields[$c].$this->qt.".".$this->qf.$refto.$this->qf;
         }
         else if(isset($d->reference[$fields[$c]])&&($d->reference[$fields[$c]]["type"]=="0:*"||$d->reference[$fields[$c]]["type"]=="1:*"))
          $where.=$this->qt.$fields[$c].$this->qt.".".$this->qf.$d->getReferenceData($fields[$c])->getID().$this->qf;
         else //normal fields (include *:0 and *:1 references)
          $where.=$this->qt.$d->field[$fields[$c]]["module"].$this->qt.".".$this->qf.$fields[$c].$this->qf;

         if($d->getOperator($fields[$c], $d->vc)=="in"||$d->getOperator($fields[$c], $d->vc)=="not in")
         {
          $where.=$this->mapOperator($d->getOperator($fields[$c], $d->vc), $prefix, $postfix)."(";

          foreach($d->sqlValue($fields[$c], $d->vc) as $value)
           $where.=$this->qv.$prefix.$this->escape($value).$postfix.$this->qv.", ";

          $where=substr($where, 0, -2).") ".$d->getOperant($fields[$c], $d->vc)." ";
         }
         else //normal operator
          $where.=$this->mapOperator($d->getOperator($fields[$c], $d->vc), $prefix, $postfix).$this->qv.$prefix.$this->escape($d->sqlValue($fields[$c], $d->vc)).$postfix.$this->qv." ".$d->getOperant($fields[$c], $d->vc)." ";
        }

        if(!$any)
         $any=true;
       }
      }

      if($open)
      {
       $where=preg_replace("/(\s[A-Z]+\s)$/", ")\\1", $where);
       $open=false;
      }
     }

     $d->vc=0;

     if($any)
      $where=preg_replace("/\s[A-Z]+\s$/", "", $where);
     else
      $where.="true";

     break;
    }
   }

   $where.=") ";


   //filter out deleted items (or non deleted items for "undelete" actions)
   foreach($d->check("properties", "deleted") as $k)
   {
    if($d->check("use", "sql", $k))
    {
     if($d->field[$k]["module"]!=$d->module&&!in_array($d->field[$k]["module"], $modules))
      $modules[]=$d->field[$k]["module"];

     $where.=" AND ".$this->qt.$d->field[$k]["module"].$this->qt.".".$this->qf.$k.$this->qf."=".$this->qv.($d->action=="undelete"||$d->extra=="deleted"?"true":"false").$this->qv;
    }
   }

   //get rights fields modules
   if(!$_config->userRoot())
   {
    if(($refer=$d->getRightsReference())&&($rightsReference=$d->reference[$refer]["from"]))
     if($d->field[$rightsReference]["module"]!=$d->module&&!in_array($d->field[$rightsReference]["module"], $modules))
      $modules[]=$d->field[$rightsReference]["module"];

    if($fields=$d->check("properties", "owner"))
    {
     foreach($fields as $k)
      if($d->field[$k]["module"]!=$d->module&&!in_array($d->field[$k]["module"], $modules))
       $modules[]=$d->field[$k]["module"];
    }
   }


//get modules in order by clause
   if($gather=="data")
   {
    foreach($d->sortOrder as $k)
    {
     if(preg_match("/^(.*)(\-|\+)$/", $k, $field))
     {
      if(isset($d->reference[$field[1]])) //reference
      {
       if($d->reference[$field[1]]["type"]=="*:*") //*:* reference
       {
        $dtmp=$d->getReferenceData($field[1]);
        $refto=$dtmp->check("use", "to");
        $refto=$refto[0];
        $dtmp=$dtmp->getReferenceData($refto); //to module

        if($dtmp->sortOrder) //only added module if sort fields are defined
         if(!in_array($field[1], $modules))
          $modules[]=$field[1];
       }
       else //0:*, 1:*, *:0 and *:1 references
       {
        $dtmp=$d->getReferenceData($field[1]);

        if($dtmp->sortOrder) //only added module if sort fields are defined
        {
         if(!in_array($field[1], $modules))
          $modules[]=$field[1];
        }
        else if($d->reference[$field[1]]["type"]=="*:0"||$d->reference[$field[1]]["type"]=="*:1") //*:0 and *:1 references
        {
         //sort on field in "local" table: *:0 and *:1 allows reference and field to have the same name so if the reference doesn't have sort fields the local field is assumed
         if($d->check("use", "sql", $field[1]))
          if($d->field[$field[1]]["module"]!=$d->module&&!in_array($d->field[$field[1]]["module"], $modules))
           $modules[]=$d->field[$field[1]]["module"];
        }
       }
      }
      else if($d->check("use", "sql", $field[1])) //normal field
      {
       if($d->field[$field[1]]["module"]!=$d->module&&!in_array($d->field[$field[1]]["module"], $modules))
        $modules[]=$d->field[$field[1]]["module"];
      }

      unset($dtmp);
     }
    }
   }

//make proper query adding from with all modules used in select and where clause
   $sql=$select." ".$this->from(false, $modules).$where;


//aggregating using group by / subqueries
   if($agg_field)
   {
    if(!$agg_type) //simple aggregation
    {
     $sql="SELECT *, "
      ."@agg_row:=if(@agg_value=".$this->qt.$d->field[$agg_field]["module"].$this->qt.".".$this->qf.$agg_field.$this->qf.", @agg_row+1, 1) AS ".$this->qf."_agg_row".$this->qf.", " //row count for aggregation
      ."@agg_value:=".$this->qt.$d->field[$agg_field]["module"].$this->qt.".".$this->qf.$agg_field.$this->qf." AS ".$this->qf."_agg_value".$this->qf." " //row value
      ."FROM (".$sql.") AS ".$this->qt.$d->module.$this->qt." " //order on aggregation_field first to ensure _agg_row is counted correctly
      ."HAVING ".$this->qf."_agg_row".$this->qf."=1 "
      .str_replace("ORDER BY ", "ORDER BY ".$this->qt.$d->field[$agg_field]["module"].$this->qt.".".$this->qf.$agg_field.$this->qf.", ", $this->order(true))." "; //order on aggregation_field first to ensure _agg_row is counted correctly

     if($gather=="count")
      $sql="SELECT COUNT(*) AS ".$this->qf."show_count".$this->qf." FROM (".$sql.") AS ".$this->qt."show_count".$this->qt." ";
     else //data
      $sql="SELECT * FROM (".$sql.") AS ".$this->qt.$d->module.$this->qt." ";

     //init simple aggregation vars (use in select section)
     if(!$this->exec("SET @agg_row:=0, @agg_value:=".$this->qv.$this->qv, "Init simple aggregation vars"))
      return false;
    }
    else
    {
     if(!$agg_interval||$agg_interval==1)
      $sql.="GROUP BY ".$this->qt.$d->field[$agg_field]["module"].$this->qt.".".$this->qf.$agg_field.$this->qf." ";
     else
     {
      $type=$d->field[$agg_field]["type"];

      if($type=="date"||$type=="time"||$type=="datetime")
       $sql.="GROUP BY FROM_UNIXTIME(".$agg_method."(UNIX_TIMESTAMP(".$this->qt.$d->field[$agg_field]["module"].$this->qt.".".$this->qf.$agg_field.$this->qf.")/".$agg_interval.")*".$agg_interval.") ";
      else
       $sql.="GROUP BY ".$agg_method."(".$this->qt.$d->field[$agg_field]["module"].$this->qt.".".$this->qf.$agg_field.$this->qf."/".$agg_interval.")*".$agg_interval." ";
     }

     if($gather=="count")
      $sql="SELECT COUNT(*) AS ".$this->qf."show_count".$this->qf." FROM (".$sql.") AS ".$this->qt."show_count".$this->qt." ";
    }
   }

   if($gather=="data")
   {
//sort
    $sql.=$this->order(($agg_field?true:false)); //skip reference sort on aggregation

//limit
    if($d->sqlValue("show_total")>0)
     $sql.="LIMIT ".$d->sqlValue("show_from").", ".$d->sqlValue("show_total");
   }

   return $sql;
  }

  function get($gather="data")
  {
   if($gather=="both")
   {
    if(!$this->get("count"))
     return false;

    if(!$this->get("data"))
     return false;

    return true;
   }

   $d=&$this->data;
   $txt="Get ".$d->action.": ".$d->title;

   if(!($sql=$this->_get($gather)))
    return false;


//execute
   if(!$this->exec($sql, $txt))
    return false;


   if($gather=="count")
   {
    if(!$this->next())
     return false;

    $this->count=$this->res["show_count"];
    return true;
   }

   if($d->action=="search"&&$d->extra=="index") //make fake results with percentage relevance for full text indexes
   {
    $indexes=Array();
    foreach(array_keys($d->fulltextIndex) as $index)
     if($d->check("use", "show", $index))
      $indexes[]=$index;

    if(!($l=sizeof($indexes))) //no relevance score
     return true; //done

    $max=Array();
    for($c=0;$c<$l;$c++)
     $max[$indexes[$c]]=0;

    $count=$this->count; //store count (this->next clears it)

    $results=Array();
    while($this->next())
    {
     //get highest relevance score for all full text indexes
     for($c=0;$c<$l;$c++)
      if($this->res[$indexes[$c]]>$max[$indexes[$c]])
       $max[$indexes[$c]]=$this->res[$indexes[$c]];

     //store fake results
     $results[]=$this->res;
    }

    $this->count=$count; //restore count

    $l2=sizeof($results);
    for($x=0;$x<$l2;$x++)
    {
     //set percentage relevance score
     for($c=0;$c<$l;$c++)
     {
      if($max[$indexes[$c]])
       $results[$x][$indexes[$c]]=($results[$x][$indexes[$c]]/$max[$indexes[$c]])*100;
      else
       $results[$x][$indexes[$c]]=0;
     }
    }

    $this->results=$results;
    $this->type="fake";
   }

   return true;
  }


  function treeUp($type="data")
  {
//use copy for gathering the data
   $d=clone $this->data;

//get reference+ID fields
   if(!($refer=$d->getSelfReference()))
   {
    $this->e->add("Unable to get tree: no self reference for module: \"".$d->module."\"", "module");
    return false;
   }

   $id=$d->getID();
   $refer=$d->reference[$refer]["from"];

   $d->setOperator($id, "="); //force operator

   $d->clearValues($id); //unset id field
   $d->setValue($id, $d->getValue($refer)); //set reference value to id field
   $d->clearValues($refer); //unset reference field

   //only return first row
   $d->setValue("show_from", 0);
   $d->setValue("show_total", 1);

   //show fields
   if($type=="count")
    $d->setShowFields($refer); //only get refer
   else if(!$d->check("use", "show", $refer)) //ensure refer field is shown
    $d->setShowFields(array_merge($d->check("use", "show"), Array($refer)));

//execute
   $results=Array();
   $class=get_class($this);
   $db=new $class($d);
   while(true)
   {
    if(!$db->get())
    {
     $this->e->add($db->e);
     break;
    }

    if(!$db->next()) //no results
     break; //done

    $results[]=$db->res; //add results

    if(!$db->res[$refer]) //no parent
     break; //done

    $d->setValue($id, $db->res[$refer]); //set parent for next run
   }

   //use fake storage
   $this->type="fake";
   $this->clear();
   $this->results=array_reverse($results); //reverse values

   if($type=="count"||$type=="both")
   {
    if(!$this->_setCount())
     return false;

    if($type=="count")
     return true;
   }

   if(!$this->_filterShowTotal())
    return false;

   return true;
  }


  function treeDown($type="data")
  {
//use copy for gathering the data
   $d=clone $this->data;

//get reference+ID fields
   if(!($refer=$d->getSelfReference()))
   {
    $this->e->add("Unable to get tree: no self reference for module: \"".$d->module."\"", "module");
    return false;
   }

   $id=$d->getID();
   $refer=$d->reference[$refer]["from"];

   $d->setOperator($id, "="); //force operator
   $d->setOperator($refer, "="); //force operator

   if(!$d->check("valuetype", "none", $refer)) //self reference based (i.e. only include sub entries in results)
   {
    $d->setValue($refer, $d->getValue($refer));
    $d->unsetValue($id);
   }
   else //id based (i.e. also include current (id) entry in results)
   {
    $d->unsetValue($refer);
    $d->setValue($id, $d->getValue($id));
   }

//show fields
   if($type=="count")
    $d->setShowFields($refer); //only get refer
   else if(!$d->check("use", "show", $refer)) //ensure refer field is shown
    $d->setShowFields(array_merge($d->check("use", "show"), Array($refer)));

//ensure output isn't limited
   if($show_total=$d->getValue("show_total"))
    $d->setValue("show_total", 0);

//execute
   $this->type="fake";
   $this->clear();

   $class=get_class($this);
   $db=new $class($d);
   if(($this->results=$db->_treeDownRecurse(0, $refer, $id))===false)
   {
    if($show_total) //restore show_total
     $d->setValue("show_total", $show_total);

    $this->e->add($db->e);
    return false;
   }

   if($show_total) //restore show_total
    $d->setValue("show_total", $show_total);

   if($type=="count"||$type=="both")
   {
    if(!$this->_setCount())
     return false;

    if($type=="count")
     return true;
   }

   if(!$this->_filterShowTotal())
    return false;

   return true;
  }

  function _treeDownRecurse($treeDepth, $refer, $id=false)
  {
   $d=&$this->data;

   $results=Array();

   if(!$this->get())
    return false;

   while($this->next())
   {
    $results[]=$this->res; //add results

//enforce tree depth limit
    if($d->field["treedepth"]["valuetype"]!="set"||$treeDepth<$d->getValue("treedepth"))
    {
     $d->setValue($refer, $this->res[$id]);
     $d->unsetValue($id);

     $class=get_class($this);
     $db=new $class($d);
     if(($results2=$db->_treeDownRecurse($treeDepth+1, $refer, $id))===false)
     {
      $this->e->add($db->e);
      return false;
     }

     $results=array_merge($results, $results2);
    }
   }

   return $results;
  }


  function _lost()
  {
   $d=&$this->data;

   if($refer=$d->getRightsReference())
    $rightsReference=$d->reference[$refer]["from"];
   else
    return false;

//title fields
   $sql="SELECT ";
   $modules=Array();
   foreach($d->check("use", "sql") as $k)
   {
    if($d->check("properties", "title", $k)||$d->check("properties", "identifier", $k)||$k==$rightsReference)
    {
     $sql.=$this->qt.$d->field[$k]["module"].$this->qt.".".$this->qf.$k.$this->qf.", ";

     if($d->field[$k]["module"]!=$d->module&&!in_array($d->field[$k]["module"], $modules))
      $modules[]=$d->field[$k]["module"];
    }
   }

//from
   $sql=substr($sql, 0, -2)." ".$this->from(false, $modules);

//where
   $c=0;
   $sql.="WHERE ";
   $sql.=$this->qt.$d->field[$rightsReference]["module"].$this->qt.".".$this->qf.$rightsReference.$this->qf." NOT IN (";
   $sql.="SELECT ".$this->qt."categories".$this->qt.".".$this->qf."nr".$this->qf." FROM ".$this->qt."categories".$this->qt." ";

   if($d->module!="categories"&&$d->module!="userRights")
    $sql.="WHERE ".$this->qt."categories".$this->qt.".".$this->qf."module".$this->qf." LIKE ".$this->qv."%".$d->module."%".$this->qv;

   $sql.=")";

   if($d->module=="categories")
   {
    $id=$d->getID();
    $sql.=" AND ".$this->qt.$d->field[$id]["module"].$this->qt.".".$this->qf.$id.$this->qf."<>".$this->qv."1".$this->qv;
   }

   return $sql;
  }

  function lost()
  {
   $d=&$this->data;
   $txt="Get ".$d->action.": ".$d->title;

   if(!($sql=$this->_lost()))
    return false;

   return $this->exec($sql, $txt);
  }


  function _unique($fields)
  {
   $d=&$this->data;

   $modules=Array();
   $where="WHERE ";
   foreach($fields as $field)
   {
    $where.=$this->qt.$d->field[$field]["module"].$this->qt.".".$this->qf.$field.$this->qf."=".$this->qv.$this->escape($d->sqlValue($field)).$this->qv." AND ";

    if($d->field[$field]["module"]!=$d->module&&!in_array($d->field[$field]["module"], $modules))
     $modules[]=$d->field[$field]["module"];
   }

   if($id=$d->getID())
   {
    if($d->action!="add"&&$d->action!="copy")
     $where.=$this->qt.$d->field[$id]["module"].$this->qt.".".$this->qf.$id.$this->qf."<>".$this->qv.$this->escape($d->sqlValue($id)).$this->qv." AND ";

    if($d->field[$id]["module"]!=$d->module&&!in_array($d->field[$id]["module"], $modules))
     $modules[]=$d->field[$id]["module"];
   }

   $sql="SELECT COUNT(*) as show_count ";

   $sql.=$this->from("unique", $modules);

   $sql.=substr($where, 0, -4);

   return $sql;
  }

  function unique()
  {
   $fields=func_get_args();
   if(is_array($fields[0]))
    $fields=$fields[0];

   $d=&$this->data;
   $txt="Check uniqueness: ".$d->title;

   if(!($sql=$this->_unique($fields)))
    return false;

   if(!$this->exec($sql, $txt))
    return false;

   if(!$this->next()||$this->res["show_count"])
    return false;

   return true;
  }


  function _add()
  {
   $d=&$this->data;

   if(!($values=$this->_getValues("add"))) //nothing to add... ?
    return false;

   foreach($values as $k => $v)
    $values[$k]=$this->escape($v, $d->check("type", "filedata", $k));

   $sql="INSERT INTO ".$this->qt.$d->module.$this->qt." (".$this->qf.join($this->qf.", ".$this->qf, array_keys($values)).$this->qf.") VALUES (".$this->qv.join($this->qv.", ".$this->qv, $values).$this->qv.")";

   return $sql;
  }

  function add()
  {
   $d=&$this->data;
   $txt="Insert new ".$d->title;

   if(!($sql=$this->_add()))
    return false;

   if(!$this->exec($sql, $txt))
    return false;

   if($insertID=$this->insertID())
    $d->setValue($d->getID(), $insertID, $d->vc);

   $this->e->add($d->title." inserted successfully", "note");
   return true;
  }


  function _edit($looseFiltering)
  {
   $d=&$this->data;

   if(!($values=$this->_getValues("edit"))) //nothing to update
    return false;

   $sql="UPDATE ".$this->qt.$d->module.$this->qt." SET ";

   foreach($values as $k => $v)
    $sql.=$this->qf.$k.$this->qf."=".$this->qv.$this->escape($v, $d->check("type", "filedata", $k)).$this->qv.", ";

   $sql=substr($sql, 0, -2)." ";

   $sql.="WHERE ";

   if(!$looseFiltering) //identifier based
   {
    $found=false;
    foreach($d->check("properties", "identifier") as $k)
    {
     $sql.=$this->qf.$k.$this->qf."=".$this->qv.$this->escape($d->sqlValue($k)).$this->qv." AND ";
     $found=true;
    }

    if(!$found)
    {
     $this->e->add("No identifiers set for updating!", "sql");
     return false;
    }

    $sql=substr($sql, 0, -4);
   }
   else //looseFiltering: watch what you're doing!
   {
    $found=false;
    foreach($d->check("valuetype", "set") as $k)
    {
     if($d->check("module", $d->module, $k)&&$d->check("use", "sql", $k))
     {
      $sql.=$this->qf.$k.$this->qf."=".$this->qv.$this->escape($d->sqlValue($k)).$this->qv." AND ";
      $found=true;
     }
    }

    if(!$found)
    {
     $this->e->add("Update all not permitted!", "sql");
     return false;
    }

    $sql=substr($sql, 0, -4);
   }

   return $sql;
  }

  function edit($looseFiltering=false)
  {
   $d=&$this->data;
   $txt="Update ".$d->title;

   if(!($sql=$this->_edit($looseFiltering)))
    return false;

   if(!$this->exec($sql, $txt))
    return false;

   $this->e->add($d->title." updated successfully", "note");
   return true;
  }


  function _delete($looseFiltering)
  {
   $d=&$this->data;

   $sql="DELETE FROM ".$this->qt.$d->module.$this->qt." ";

   $sql.="WHERE ";

   if(!$looseFiltering) //identifier based
   {
    $found=false;
    foreach($d->check("properties", "identifier") as $k)
    {
     $sql.=$this->qf.$k.$this->qf."=".$this->qv.$this->escape($d->sqlValue($k)).$this->qv." AND ";
     $found=true;
    }

    if(!$found)
    {
     $this->e->add("No identifiers set for deletion!", "sql");
     return false;
    }

    $sql=substr($sql, 0, -4);
   }
   else //looseFiltering: watch what you're doing!
   {
    $found=false;
    foreach($d->check("valuetype", "set") as $k)
    {
     if($d->check("module", $d->module, $k)&&$d->check("use", "sql", $k))
     {
      $sql.=$this->qf.$k.$this->qf."=".$this->qv.$this->escape($d->sqlValue($k)).$this->qv." AND ";
      $found=true;
     }
    }

    if(!$found)
    {
     $this->e->add("Delete all not permitted!", "sql");
     return false;
    }

    $sql=substr($sql, 0, -4);
   }

   return $sql;
  }

  function delete($looseFiltering=false)
  {
   $d=&$this->data;
   $txt="Delete ".$d->title;

   if(!($sql=$this->_delete($looseFiltering)))
    return false;

   if(!$this->exec($sql, $txt))
    return false;

   $this->e->add($d->title." deleted successfully", "note");
   return true;
  }




  function _create()
  {
   $d=&$this->data;

   $sql="CREATE TABLE ".$this->qt.$d->module.$this->qt." (";

   $c=0;
   $keys="";
   $autoincr=false;
   foreach($d->field as $k => $v)
   {
    if($d->field[$k]["module"]!=$d->module&&!$d->check("properties", "identifier", $k))
     continue;

    if($d->check("use", "sql", $k)&&!($k=="r"&&$v["type"]=="rights"))
    {
     $field=$this->getFieldType($k);

     $sql.=$this->qf.$k.$this->qf." ".$field["type"]." ";

     if($field["collation"])
     {
      if($this->charset)
       $sql.="CHARACTER SET ".$this->qv.$this->charset.$this->qv." ";
      if($this->collation)
       $sql.="COLLATE ".$this->qv.$this->collation.$this->qv." ";
     }

     $sql.="NOT NULL ";

     if($d->check("properties", "identifier", $k))
     {
      $keys.=$this->qf.$k.$this->qf.", ";

      if($d->check("type", "nr", $k)&&!$autoincr&&!$d->extends)
      {
       $autoincr=true;
       $sql.="auto_increment ";
      }
     }
     else if($field["default"]!==false)
      $sql.="DEFAULT ".$this->qv.$field["default"].$this->qv." ";

     $sql.=", ";
     $c++;
    }
   }

   if($c==0)
   {
    $this->e->add("No fields for module \"".$d->title."\"", "error");
    return false;
   }

   if($keys!="")
    $sql.="PRIMARY KEY (".substr($keys, 0, -2).") ";
   else
    $sql=substr($sql, 0, -2)." ";

   $sql.=") ";

   if($this->engine)
    $sql.="ENGINE ".$this->qv.$this->engine.$this->qv." ";
   if($this->charset)
    $sql.="CHARACTER SET ".$this->qv.$this->charset.$this->qv." ";
   if($this->collation)
    $sql.="COLLATE ".$this->qv.$this->collation.$this->qv." ";

   return Array($sql);
  }

  function _alter($submit=false)
  {
   $d=&$this->data;

   if(($collation=$this->getCollation())===false)
    return false;

   if(($fields=$this->getFields())===false)
    return false;

   if(($indexes=$this->getIndexes())===false)
    return false;

   //determine unused fields
   $tmpFields=Array();
   $unusedFields=Array();
   foreach(array_keys($d->field) as $field)
    if($d->field[$field]["module"]==$d->module||$d->check("properties", "identifier", $field))
     $tmpFields[]=$field;

   foreach(array_keys($fields) as $field)
    if(!in_array($field, $tmpFields))
     $unusedFields[]=$field;

   unset($tmpFields);


   $sql="ALTER TABLE ".$this->qt.$d->module.$this->qt." ";

   $c=0;
   $altered=false;
   $autoincr=false;
   $primaryKeyFields=Array();
   foreach($d->field as $k => $v)
   {
    if($d->field[$k]["module"]!=$d->module&&!$d->check("properties", "identifier", $k))
     continue;

    if($d->check("use", "sql", $k)&&!($k=="r"&&$v["type"]=="rights"))
    {
     if($d->check("properties", "identifier", $k))
      $primaryKeyFields[]=$k;

     $field=$this->getFieldType($k);

     if(!isset($fields[$k])) //field doesn't exist in database, so create it
     {
      $altered=true;
      $renamed=false;

//XXX: rename checking needs enum extraction
      foreach($unusedFields as $key => $uf) //check all unused database field for similar properties
      {
       if($fields[$uf]["type"]==$field["type"]) //found a matching field in unused database fields
       {
        $renamed=true;
        $sql.="CHANGE ".$this->qf.$uf.$this->qf." ".$this->qf.$k.$this->qf." ".$field["type"];
        $this->e->add("Renaming database field from: \"".$uf."\" to: \"".$k."\" in table: \"".$d->module."\"", "note");
        unset($unusedFields[$key]);
        break;
       }
      }

      if(!$renamed)
      {
       $sql.="ADD ".$this->qf.$k.$this->qf." ".$field["type"];
       $this->e->add("Adding field: \"".$k."\" in table: \"".$d->module."\"", "note");
      }
     }
     //field exists
     else if($this->enum_type&&($v["type"]=="enum"||$v["type"]=="boolean")) //should be enum or boolean type
     {
      if(!preg_match("/^(\w+)\((.+)\)$/i", $fields[$k]["type"], $current)) //get database field type
       $this->e->add("Can't determine current database type of field: \"".$k."\" in table: \"".$d->module."\"", "warning");
      else if(!preg_match_all("/('([^']*)')+/", $current[2], $current)) //get database enum options
       $this->e->add("Can't determine current database enum options of field: \"".$k."\" in table: \"".$d->module."\"", "warning");
      else
      {
       $current[0]=NULL;
       $current[ALTER_FIELD_TYPE]="enum";

       //compare enum/boolean options
       if($v["type"]=="boolean")
       {
        $booloptions=Array("false", "true");
        $newOptions=array_diff($booloptions, $current[ALTER_FIELD_OPTIONS]);
        $deletedOptions=array_diff($current[ALTER_FIELD_OPTIONS], $booloptions);
       }
       else
       {
        if(array_type($d->listOptions[$k])=="indexed")
        {
         $newOptions=array_diff($d->listOptions[$k], $current[ALTER_FIELD_OPTIONS]);
         $deletedOptions=array_diff($current[ALTER_FIELD_OPTIONS], $d->listOptions[$k]);
        }
        else //associative
        {
         $newOptions=array_diff(array_keys($d->listOptions[$k]), $current[ALTER_FIELD_OPTIONS]);
         $deletedOptions=array_diff($current[ALTER_FIELD_OPTIONS], array_keys($d->listOptions[$k]));
        }
       }

       if(sizeof($newOptions)) //new options to be added
       {
        $altered=true;
        $this->e->add("Adding new enum option(s): \"".join("\", \"", $newOptions)."\" for field: \"".$k."\" in table: \"".$d->module."\"", "note");

        if(isset($d->listOptions[$k]))
        {
         if(array_type($d->listOptions[$k])=="indexed")
          $newOptions=array_merge($d->listOptions[$k], $deletedOptions);
         else //associative
          $newOptions=array_merge(array_keys($d->listOptions[$k]), $deletedOptions);
        }

        $sql.="CHANGE ".$this->qf.$k.$this->qf." ".$this->qf.$k.$this->qf." enum(".$this->qv.join($this->qv.", ".$this->qv, $newOptions).$this->qv.")";
       }

       if(sizeof($deletedOptions)) //old options, leave them in case they're in use
        $this->e->add("Leaving unused enum option(s): \"".join("\", \"", $deletedOptions)."\" for field: \"".$k."\" in table: \"".$d->module."\"", "note");
      }
     }
     //all but enum/boolean fields
     else if(!preg_match("/^([\w\s]+)(\((\d+)\))?$/", $field["type"], $new)) //get module field type
      $this->e->add("Can't determine new database type of field: \"".$k."\" in table: \"".$d->module."\"", "warning");
     else
     {
      if(!preg_match("/^([\w\s]+)(\((\d+)\))?$/", $fields[$k]["type"], $current)) //get database field type
       $this->e->add("Can't determine current database type of field: \"".$k."\" in table: \"".$d->module."\"", "warning");
      else if($current[ALTER_FIELD_TYPE]!=$new[ALTER_FIELD_TYPE]) //database and module field type don't match
      {
       //try to cross convert field types
       if(isset($this->crossConvert[$current[ALTER_FIELD_TYPE]])&&in_array($new[ALTER_FIELD_TYPE], $this->crossConvert[$current[ALTER_FIELD_TYPE]]))
       {
        $altered=true;
        $sql.="CHANGE ".$this->qf.$k.$this->qf." ".$this->qf.$k.$this->qf." ".$field["type"];
        $this->e->add("Convert database type from: \"".$current[ALTER_FIELD_TYPE]."\" to: \"".$new[ALTER_FIELD_TYPE]."\" for field: \"".$k."\" in table: \"".$d->module."\"", "note");
       }
       else
        $this->e->add("Can't convert database type from: \"".$current[ALTER_FIELD_TYPE]."\" to: \"".$new[ALTER_FIELD_TYPE]."\" for field: \"".$k."\" in table: \"".$d->module."\"", "warning");
      }
      else if(isset($current[ALTER_FIELD_SIZE])&&isset($new[ALTER_FIELD_SIZE])&&$current[ALTER_FIELD_SIZE]!=$new[ALTER_FIELD_SIZE]) //type the same, check size change
      {
       //new type size is larger, resize
       if($current[ALTER_FIELD_SIZE]<$new[ALTER_FIELD_SIZE])
       {
        $altered=true;
        $sql.="CHANGE ".$this->qf.$k.$this->qf." ".$this->qf.$k.$this->qf." ".$field["type"];
        $this->e->add("Increase database field size from: \"".$current[ALTER_FIELD_SIZE]."\" to: \"".$new[ALTER_FIELD_SIZE]."\" for field: \"".$k."\" in table: \"".$d->module."\"", "note");
       }
       else if($current[ALTER_FIELD_SIZE]>$new[ALTER_FIELD_SIZE]) //new type size is smaller, leave it
        $this->e->add("Not shrinking database field size from: \"".$current[ALTER_FIELD_SIZE]."\" to: \"".$new[ALTER_FIELD_SIZE]."\" for field: \"".$k."\" in table: \"".$d->module."\"", "note");
      }

      //check if auto increment
      if(stristr($fields[$k]["extra"], "auto_increment")) //database has auto increment set
      {
       if(!$d->check("properties", "identifier", $k)||!$d->check("type", "nr", $k)) //non identifier, remove auto increment
       {
        if(!$altered) //change if not already changed earlier, auto increment done below
        {
         $altered=true;
         $sql.="CHANGE ".$this->qf.$k.$this->qf." ".$this->qf.$k.$this->qf." ".$field["type"];
        }

        $this->e->add("Removing auto increment property for field: \"".$k."\" in table: \"".$d->module."\"", "note");
       }
      }
      else //no auto increment set in database
      {
       if($d->check("properties", "identifier", $k)&&$d->check("type", "nr", $k)&&!$d->extends) //non extending identifier, needs auto increment
       {
        if(!$altered) //change if not already changed earlier, auto increment done below
        {
         $altered=true;
         $sql.="CHANGE ".$this->qf.$k.$this->qf." ".$this->qf.$k.$this->qf." ".$field["type"];
        }

        $this->e->add("Adding auto increment property for field: \"".$k."\" in table: \"".$d->module."\"", "note");
       }
      }
     }

     if($field["collation"])
     {
      if(!isset($fields[$k])||$fields[$k]["collation"]==$this->collation) //new field / collation stayed the same
      {
       if($altered)
       {
        if($this->charset)
         $sql.=" CHARACTER SET ".$this->qv.$this->charset.$this->qv;
        if($this->collation)
         $sql.=" COLLATE ".$this->qv.$this->collation.$this->qv;
       }
      }
      else //collation changed
      {
       $this->e->add("Changing collation from: \"".$fields[$k]["collation"]."\" for field: \"".$k."\" to: \"".$this->collation."\" in table: \"".$d->module."\"", "note");

       if($altered)
       {
        if($this->charset)
         $sql.=" CHARACTER SET ".$this->qv.$this->charset.$this->qv;
        if($this->collation)
         $sql.=" COLLATE ".$this->qv.$this->collation.$this->qv;
       }
       else
       {
        $altered=true;
        $sql.="CHANGE ".$this->qf.$k.$this->qf." ".$this->qf.$k.$this->qf." ".$fields[$k]["type"]; //don't change current field type

        if($this->charset)
         $sql.=" CHARACTER SET ".$this->qv.$this->charset.$this->qv;
        if($this->collation)
         $sql.=" COLLATE ".$this->qv.$this->collation.$this->qv;
       }
      }
     }

     if($altered)
     {
      $sql.=" NOT NULL ";

      if($d->check("properties", "identifier", $k))
      {
       if($d->check("type", "nr", $k)&&!$autoincr&&!$d->extends)
       {
        $autoincr=true;
        $sql.="auto_increment, ";
       }
       else
        $sql.=", ";
      }
      else if($field["default"]!==false)
       $sql.="DEFAULT ".$this->qv.$field["default"].$this->qv.", ";
      else
       $sql.=", ";

      $altered=false;
      $c++;
     }
    }
   }

   if(sizeof($unusedFields))
    $this->e->add("Not removing database field(s): \"".join("\", \"", $unusedFields)."\" in table: \"".$d->module."\"", "note");


   if($indexes["primary"]&&(array_diff($indexes["primary"], $primaryKeyFields)||array_diff($primaryKeyFields, $indexes["primary"])))
   {
    $this->e->add("Dropping primary key (".implode(", ", $indexes["primary"]).") for table: \"".$d->module."\"", "note");
    $sql.="DROP PRIMARY KEY, ";
    $c++;
   }

   if($primaryKeyFields&&(array_diff($indexes["primary"], $primaryKeyFields)||array_diff($primaryKeyFields, $indexes["primary"])))
   {
    $this->e->add("Creating primary key (".implode(", ", $primaryKeyFields).") for table: \"".$d->module."\"", "note");
    $sql.="ADD PRIMARY KEY (".$this->qf.join($this->qf.", ".$this->qf, $primaryKeyFields).$this->qf."), ";
    $c++;
   }

   if($collation!=$this->collation)
   {
    $this->e->add("Changing collation from: \"".$collation."\" to: \"".$this->collation."\" for table: \"".$d->module."\"", "note");

    if($this->charset)
     $sql.="CHARACTER SET ".$this->qv.$this->charset.$this->qv." ";
    if($this->collation)
     $sql.="COLLATE ".$this->qv.$this->collation.$this->qv." ";

    if($this->charset||$this->collation)
     $sql.=", ";

    $c++;
   }

   if($c>0)
    return Array(substr($sql, 0, -2));

   return Array();
  }


  function _alterSequences()
  {
   return Array(); //no query
  }

  function _alterFieldTypes()
  {
   return Array(); //no query
  }

  function _alterIndexes()
  {
   $sqls=Array();

   $d=&$this->data;

   if(($indexes=$this->getIndexes())===false)
    return false;

   foreach($indexes["index"] as $index => $fields)
   {
    if(!isset($d->index[$index])||array_diff($fields, $d->index[$index])||array_diff($d->index[$index], $fields))
    {
     $this->e->add("Dropping index: \"".$index."\" (".implode(", ", $fields).") for table: \"".$d->module."\"", "note");
     $sqls[]="DROP INDEX ".$this->qi.$index.$this->qi." ON ".$this->qt.$d->module.$this->qt;
    }
   }
   foreach($d->index as $index => $fields)
   {
    if(!isset($indexes["index"][$index])||array_diff($fields, $indexes["index"][$index])||array_diff($indexes["index"][$index], $fields))
    {
     $this->e->add("Creating index: \"".$index."\" (".implode(", ", $fields).") for table: \"".$d->module."\"", "note");
     $sqls[]="CREATE INDEX ".$this->qi.$index.$this->qi." ON ".$this->qt.$d->module.$this->qt." (".$this->qf.join($this->qf.", ".$this->qf, $fields).$this->qf.")";
    }
   }

   foreach($indexes["unique"] as $index => $fields)
   {
    if(!isset($d->unique[$index])||array_diff($fields, $d->unique[$index])||array_diff($d->unique[$index], $fields))
    {
     $this->e->add("Dropping unique index: \"".$index."\" (".implode(", ", $fields).") for table: \"".$d->module."\"", "note");
     $sqls[]="DROP INDEX ".$this->qi.$index.$this->qi." ON ".$this->qt.$d->module.$this->qt;
    }
   }
   foreach($d->unique as $index => $fields)
   {
    if(!isset($indexes["unique"][$index])||array_diff($fields, $indexes["unique"][$index])||array_diff($indexes["unique"][$index], $fields))
    {
     $this->e->add("Creating unique index: \"".$index."\" (".implode(", ", $fields).") for table: \"".$d->module."\"", "note");
     $sqls[]="CREATE UNIQUE INDEX ".$this->qi.$index.$this->qi." ON ".$this->qt.$d->module.$this->qt." (".$this->qf.join($this->qf.", ".$this->qf, $fields).$this->qf.")";
    }
   }

   foreach($indexes["fulltext"] as $index => $fields)
   {
    if(!isset($d->fulltextIndex[$index])||array_diff($fields, $d->fulltextIndex[$index])||array_diff($d->fulltextIndex[$index], $fields))
    {
     $this->e->add("Dropping fulltext index: \"".$index."\" (".implode(", ", $fields).") for table: \"".$d->module."\"", "note");
     $sqls[]="DROP INDEX ".$this->qi.$index.$this->qi." ON ".$this->qt.$d->module.$this->qt;
    }
   }
   foreach($d->fulltextIndex as $index => $fields)
   {
    if(!isset($indexes["fulltext"][$index])||array_diff($fields, $indexes["fulltext"][$index])||array_diff($indexes["fulltext"][$index], $fields))
    {
     $this->e->add("Creating fulltext index: \"".$index."\" (".implode(", ", $fields).") for table: \"".$d->module."\"", "note");
     $sqls[]="CREATE FULLTEXT INDEX ".$this->qi.$index.$this->qi." ON ".$this->qt.$d->module.$this->qt." (".$this->qf.join($this->qf.", ".$this->qf, $fields).$this->qf.")";
    }
   }

   return $sqls;
  }

  function create($submit=false){return $this->alter($submit);}
  function alter($submit=false)
  {
   $sqls=Array();
   $d=&$this->data;

   if($d->extra=="create")
    $txt="Create table ".$d->title;
   else //alter
    $txt="Alter table ".$d->title;

   if(($tmp=$this->_alterSequences())===false)
    return false;
   else if($tmp)
    $sqls=array_merge($sqls, $tmp);

   if(($tmp=$this->_alterFieldTypes())===false)
    return false;
   else if($tmp)
    $sqls=array_merge($sqls, $tmp);

   if($d->extra=="create")
   {
    if(($tmp=$this->_create())===false)
     return false;
    else if($tmp)
     $sqls=array_merge($sqls, $tmp);
   }
   else //alter
   {
    if(($tmp=$this->_alter())===false)
     return false;
    else if($tmp)
     $sqls=array_merge($sqls, $tmp);
   }

   if(($tmp=$this->_alterIndexes())===false)
    return false;
   else if($tmp)
    $sqls=array_merge($sqls, $tmp);

   if($submit)
   {
    foreach($sqls as $sql)
     if(!$this->exec($sql, $txt))
      return false;

    $this->e->add("Table: \"".$d->module."\" ".($d->extra=="create"?"created":"altered")." successfully", "note");
   }
   else if($sqls)
   {
    if($d->extra=="create") //don't show invidiual notices when creating the table
     $this->e->clear();

    $this->e->add("Queries for table: \"".$d->module."\": ".implode(";\n", $sqls).";", "note");
   }
   else
    $this->e->add("No changes for table: \"".$d->module."\"", "note");

   return true;
  }



  function getCollation()
  {
   $d=&$this->data;

   $txt="Get table status of: ".$d->title;
   $sql="SHOW TABLE STATUS WHERE Name=".$this->qv.$d->module.$this->qv;

   if(!$this->exec($sql, $txt))
    return false;

   if($this->next())
    return $this->res["Collation"];

   return false;
  }

  function getFields()
  {
   $d=&$this->data;

   $txt="Get table columns of: ".$d->title;
   $sql="SHOW FULL COLUMNS FROM ".$this->qt.$d->module.$this->qt;

   if(!$this->exec($sql, $txt))
    return false;

   $fields=Array();
   while($this->next())
   {
    $fields[$this->res["Field"]]=Array(
     "name" => $this->res["Field"],
     "type" => $this->res["Type"],
     "null" => $this->res["Null"],
     "default" => $this->res["Default"],
     "collation" => $this->res["Collation"],
     "extra" => $this->res["Extra"],
    );
   }

   return $fields;
  }

  function getIndexes()
  {
   $indexes=Array("primary" => Array(), "index" => Array(), "unique" => Array(), "fulltext" => Array());

   if(!$this->table())
    return $indexes;

   $d=&$this->data;
   $txt="Get table indexes of: ".$d->title;
   $sql="SHOW INDEX FROM ".$this->qt.$d->module.$this->qt;

   if(!$this->exec($sql, $txt))
    return false;

   while($this->next())
   {
    if($this->res["Key_name"]=="PRIMARY")
     $key="primary";
    else if($this->res["Index_type"]=="FULLTEXT")
     $key="fulltext";
    else if($this->res["Non_unique"]) //index
     $key="index";
    else //unique index
     $key="unique";

    if($key=="primary")
     $indexes[$key][]=$this->res["Column_name"];
    else
    {
     if(!isset($indexes[$key][$this->res["Key_name"]]))
      $indexes[$key][$this->res["Key_name"]]=Array();

     $indexes[$key][$this->res["Key_name"]][]=$this->res["Column_name"];
    }
   }

   return $indexes;
  }


  function getFieldType($field)
  {
   $d=&$this->data;
   $v=&$d->field[$field];

   $default=false;
   $collation=false;
   switch($v["type"])
   {
    case "nr":
    case "rights":
     if($d->check("properties", "big", $field))
      $type="bigint(20)";
     else
      $type="int(11)";
     $default=0;
    break;

    case "price":
    case "float":
     $type="double";
     $default=0;
    break;

    case "ip":
    case "ipv6":
     $type="varchar(40)";
     $collation=true;
    break;

    case "ipv4":
     $type.="int UNSIGNED";
     $default=0;
    break;

    case "password":
     if($d->check("properties", "nocrypt", $field))
      $type="varchar(".($v["max"]).")";
     else
      $type="varchar(255)";

     $default="";
     $collation=true;
    break;

    case "url":
    case "html":
    case "text":
    case "title":
    case "phone":
    case "domain":
    case "hostname":
    case "email":
    case "regexp":
    case "filename":
    case "alphabetic":
    case "captcha":

    case "list":
    case "multilist":
    case "module":
    case "modules":
    case "plugin":
    case "action":
    case "extra":
    case "project":

     if($v["max"]<=255)
      $type="varchar(".($v["max"]).")";
     else if($v["max"]<=pow(2,8))
      $type="tinytext";
     else if($v["max"]<=pow(2,16))
      $type="text";
     else if($v["max"]<=pow(2,24))
      $type="mediumtext";
     else if($v["max"]<=pow(2,32))
      $type="longtext";

     $default="";
     $collation=true;
    break;

    case "date":
     $type="date";
     $default="0000-00-00";
    break;

    case "time":
     $type="time";
     $default="00:00:00";
    break;

    case "datetime":
     $type="datetime";
     $default="0000-00-00 00:00:00";
    break;

    case "file":
    case "image":
    case "localfile":
    case "localdirectory":
     $type="varchar(255)";
     $default="";
     $collation=true;
    break;

    case "localfiles":
     $type="text";
     $default="";
     $collation=true;
    break;

    case "filedata":
     if($v["max"]<=pow(2,8))
      $type="tinyblob";
     else if($v["max"]<=pow(2,16))
      $type="blob";
     else if($v["max"]<=pow(2,24))
      $type="mediumblob";
     else if($v["max"]<=pow(2,32))
      $type="longblob";

     $default="";
    break;

    case "boolean":
     $type="enum('false','true')";
     $collation=true;
    break;

    case "enum":
     if(array_type($d->listOptions[$field])=="indexed")
      $type="enum('".join("','", $d->listOptions[$field])."')";
     else //associative
      $type="enum('".join("','", array_keys($d->listOptions[$field]))."')";

     $collation=true;
    break;

    case "version":
     $type="varchar(255)";
     $default="";
     $collation=true;
    break;

    default:
     $type=$v["type"];
     $default="";
    break;
   }

   $r=Array("type" => $type, "default" => $default, "collation" => $collation);
   return $r;
  }


  function mapOperator($operator, &$prefix, &$postfix)
  {
   $prefix="";
   $postfix="";
   switch($operator)
   {
    case "!=":
     return "<>";
    case "like":
     $prefix="%";
     $postfix="%";
     return " LIKE ";
    case "not like":
     $prefix="%";
     $postfix="%";
     return " NOT LIKE ";
    case "start like":
     $postfix="%";
     return " LIKE ";
    case "not start like":
     $postfix="%";
     return " NOT LIKE ";
    case "end like":
     $prefix="%";
     return " LIKE ";
    case "not end like":
     $prefix="%";
     return " NOT LIKE ";
    case "in":
     return " IN ";
    case "not in":
     return " NOT IN ";
    case "between":
     return " BETWEEN ";
    case "not between":
     return " NOT BETWEEN ";
   }

   return $operator;
  }


 }

?>
